//******************************************************************************
// In this example the DAC MCP4822 is set to generate a voltage reference.
// The MCP4822 is connected to the SPI module 1 but it can be connected also 
// to module 2 by changing the definition before the library. 
// Check the device configuration file for the peripheral pin out.
//
// The voltage selection is made via push buttons.
//
// BT1 -> pin 4 PORTB ->  Output 1.5V
// BT2 -> pin 5 PORTB ->  Output 1.8V
// BT3 -> pin 6 PORTB ->  Output 3.3V
// BT4 -> pin 7 PORTB ->  ON/OFF toggle
//
// The 3.3V would properly work if Vcc is high enough (tested on Vcc=5V).
// 
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "module_IO.h"
#include "module_IO.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_SPI.h"
#include "module_SPI.c"

#include "MCP4822.h"
#include "MCP4822.c"

//*************************************
//      Voltage reference
//*************************************

#define REFERENCE_0000 0
#define REFERENCE_1500 1500
#define REFERENCE_1800 1800
#define REFERENCE_3300 3300

//*************************************
//          Constants
//*************************************

#define STATUS_ON 0x01
#define STATUS_OFF 0x00

#define WAIT_TIME 1000
#define DEBOUNCE_TIME 10

#define BUTTON_1 IO_BIT4
#define BUTTON_2 IO_BIT5
#define BUTTON_3 IO_BIT6
#define BUTTON_4 IO_BIT7
#define BUTTON_PRESSED 0x00


void main (void) {

    uint8_t system_status = STATUS_OFF;
    uint16_t last_voltage = REFERENCE_0000;

    IO_set_all_ports_as_inputs();
    IO_enable_pull_up_resistors(IO_PORTB, 0xFF);
    IO_set_port_direction(IO_PORTD, IO_ALL_PORT_OUTPUT);

    MCP4822_initialize (MCP4822_CLOCK_FOSC_64);

    IO_write_port(IO_PORTD, STATUS_OFF);

    while (1) {

        //*************************************
        //          BT1
        //*************************************

        if (IO_read_port_bit(IO_PORTB, BUTTON_1) == BUTTON_PRESSED) {

            delay_ms (DEBOUNCE_TIME);

             if (IO_read_port_bit(IO_PORTB, BUTTON_1) == BUTTON_PRESSED) {
                MCP4822_set_amplitude (REFERENCE_1500, MCP4822_DAC_A);
                last_voltage = REFERENCE_1500;
             }
        }
         
        //*************************************
        //          BT2
        //*************************************

        if (IO_read_port_bit(IO_PORTB, BUTTON_2) == BUTTON_PRESSED) {

            delay_ms (DEBOUNCE_TIME);

            if (IO_read_port_bit(IO_PORTB, BUTTON_2) == BUTTON_PRESSED) {
                MCP4822_set_amplitude (REFERENCE_1800, MCP4822_DAC_A);
                last_voltage = REFERENCE_1800;
            }
        }
           
        //*************************************
        //          BT3
        //*************************************
        if (IO_read_port_bit(IO_PORTB, BUTTON_3) == BUTTON_PRESSED){

            delay_ms (DEBOUNCE_TIME);

            if (IO_read_port_bit(IO_PORTB, BUTTON_3) == BUTTON_PRESSED){
                MCP4822_set_amplitude (REFERENCE_3300, MCP4822_DAC_A);
                last_voltage = REFERENCE_3300;
            }
            
        }

        //*************************************
        //          BT4
        //*************************************

        if (IO_read_port_bit(IO_PORTB, BUTTON_4) == BUTTON_PRESSED && system_status == STATUS_ON){

            delay_ms (DEBOUNCE_TIME);

            if (IO_read_port_bit(IO_PORTB, BUTTON_4) == BUTTON_PRESSED){
                MCP4822_set_options (MCP4822_GAIN_2, MCP4822_SHUTDOWN_ON, MCP4822_DAC_A);
                MCP4822_set_amplitude (REFERENCE_0000, MCP4822_DAC_A);
              
                IO_write_port(IO_PORTD, STATUS_OFF);
                system_status = STATUS_OFF;
                delay_ms (1000);
            }
        }

        if (IO_read_port_bit(IO_PORTB, BUTTON_4) == BUTTON_PRESSED && system_status == STATUS_OFF){

            delay_ms (DEBOUNCE_TIME);
            
            if (IO_read_port_bit(IO_PORTB, BUTTON_4) == BUTTON_PRESSED){
                MCP4822_set_options (MCP4822_GAIN_2, MCP4822_SHUTDOWN_OFF, MCP4822_DAC_A);
                MCP4822_set_amplitude (last_voltage, MCP4822_DAC_A);

                IO_write_port(IO_PORTD, STATUS_ON);
                system_status = STATUS_ON;
                delay_ms (1000);
            }
        }
    }
}
