//******************************************************************************
// The software writes the Date and Time to the LCD.
// The LCD must be connected as specified in the library included above.
// The MCP7940 must be connected to the I2C Module 1 of the MCU.  
// 4 push buttons must be connected between RB4-RB8.
//
// BT1 connected to RB4: Select Either Date or Time (pressed)
// BT2 connected to RB5: Increments days
// BT3 connected to RB6: Increments Months/Minutes
// BT4 connected to RB7: Increments Years/Hours
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "LTlib_utilities.h"
#include "LTlib_utilities.c"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "MCP7940.h"
#include "MCP7940.c"

#include "MCP2300x.h"
#include "MCP2300x.c"

#include "LCD_44780_I2C.h"
#include "LCD_44780_I2C.c"

//******************************************
//           Program Definitions
//******************************************

#define BT1 0xE0    //0b11100000
#define BT2 0xD0    //0b11010000
#define BT3 0xB0    //0b10110000
#define BT4 0x70    //0b01110000

#define BEEP IO_BIT0

#define STATE_CHANGE_DONE 0x00
#define STATE_CHANGE_MINUTES 0x01
#define STATE_CHANGE_HOURS 0x02
#define STATE_CHANGE_YEARS 0x03
#define STATE_CHANGE_MONTHS 0x04
#define STATE_CHANGE_DAYS 0x05

volatile uint8_t clock_state = STATE_CHANGE_DONE;


//******************************************
//           Interrupt Handling
//******************************************

__interrupt (high_priority) void ISR_alta (void) {

    uint8_t button;

    // Check the interrupt source
    if (INTCONbits.RBIF == 1 ) {

        INTCONbits.RBIF = 0;
        
        button = PORTB;
        button = button & 0xF0;

        // Check the button that got pressed
        switch(button) {

            case BT1 & BT3:
                    clock_state = STATE_CHANGE_MINUTES;
                    break;

            case BT1 & BT4:
                    clock_state = STATE_CHANGE_HOURS;
                    break;
            case BT2:
                    clock_state = STATE_CHANGE_YEARS;
                    break;
            case BT3:
                    clock_state = STATE_CHANGE_MONTHS;
                    break;
            case BT4:
                    clock_state = STATE_CHANGE_DAYS;
                    break;
         }
    }
}


//******************************************
//           Main Program
//******************************************
void main(void) {
    
    uint8_t button_still_pressed;
    
    IO_set_all_ports_as_inputs();
    
    IO_set_port_direction(IO_PORTB, IO_ALL_PORT_INPUT);
    IO_set_port_direction(IO_PORTC, 0xFE);
    
    IO_enable_pull_up_resistors(IO_PORTB, 0xF0);
    
    // RTCC initialization at 100KHz
    RTCC_initialize(100);

    // Small delay to enable 32KHz crystal to get stabilized 
    delay_ms(500);

    LCD_initialize (20);
    LCD_backlight (LCD_TURN_ON_LED);

    // Initialize date
    RTCC_set_days (0x30);
    RTCC_set_months (0x12);
    RTCC_set_years (0x08);

    // Initialize Time
    RTCC_set_hours (0x02);
    RTCC_set_minutes (0x56);
    RTCC_set_seconds (0x33);


    // Clear Interrupt Flag
    INTCONbits.RBIF = 0;

    // Interrupt on PORTB
    IOCB = 0xF0;
    INTCONbits.RBIE = 1;

    // Compatible Interrupt Mode
    RCONbits.IPEN = 0;

    // Global Interrupt
    INTCONbits.GIE = 1;

    // Peripheral Interrupt
    INTCONbits.PEIE = 1 ;


    while (1) {

        LCD_write_message ("Time :  ");

        LCD_write_string (RTCC_get_time_seconds ());

        LCD_goto_line(2);
        
        LCD_write_message ("Date :  ");

        LCD_write_string (RTCC_get_date ());

        LCD_home ();
        
        if (clock_state != STATE_CHANGE_DONE){
            
            //Anti Spikes filter
            delay_ms (30);
            button_still_pressed = PORTB & 0xF0;
            
            //Check if any button is still pressed
            if (button_still_pressed != 0xE0){
                                          
                IO_write_port(IO_PORTC, 0x01);        
                delay_ms (100);
                IO_write_port(IO_PORTC, 0x00);
                    
                //check which button was pressed
                switch (clock_state) {

                    case STATE_CHANGE_MINUTES : RTCC_increment_minutes ();
                                                break;

                    case STATE_CHANGE_HOURS :   RTCC_increment_hours ();
                                                break;
                    case STATE_CHANGE_YEARS :   RTCC_increment_years ();
                                                break;

                    case STATE_CHANGE_MONTHS :  RTCC_increment_months ();
                                                break;

                    case STATE_CHANGE_DAYS :    RTCC_increment_days ();
                                                break;
                }
            
            }
     
            clock_state = STATE_CHANGE_DONE;               
            
        }          
    }
}
