//******************************************************************************
// The software is an I2C master. It sends the data reflecting the push button 
// values. The push buttons are connected to RB4-RB7. A second MCU must be 
// programmed with the slave firmware to receive the data. The I2C module can 
// be selected in the Firmware and it could be different from the Slave.
// The device address selected in the master must be the same one set
// within the Slave.
// 
//
// Tested Platform: Freedom III
// Tested MCU: PIC18F46K22
// Tested Compiler: XC8 2.05
//******************************************************************************

#include <xc.h>

#include "LTlib.h"

#include "module_IO.h"
#include "module_IO.c"

#include "module_I2C.h"
#include "module_I2C.c"

#define SLAVE_ADDRESS 0xA6
#define READ  0x01
#define WRITE 0x00

#define BUTTON_MASK  0xF0
#define BUTTON_1  0x01
#define BUTTON_2  0x02
#define BUTTON_3  0x04
#define BUTTON_4  0x08

#define BUTTON_1_PRESSED   0xE0 //0b11100000
#define BUTTON_2_PRESSED   0xD0 //0b11010000
#define BUTTON_3_PRESSED   0xB0 //0b10110000
#define BUTTON_4_PRESSED   0x70 //0b01110000


//*************************************
//     Module Switch
//*************************************

#define I2C_MODULE_1
//#define I2C_MODULE_2

#ifdef I2C_MODULE_1
    #define I2C_Module_open I2C1_open
    #define I2C_Module_baud_rate I2C1_baud_rate
    #define I2C_Module_wait_bus_IDLE I2C1_wait_bus_IDLE
    #define I2C_Module_start_bit I2C1_start_bit
    #define I2C_Module_read_byte I2C1_read_byte
    #define I2C_Module_reset_write_collision_flag I2C1_reset_write_collision_flag
    #define I2C_Module_write_byte I2C1_write_byte 
    #define I2C_Module_stop_bit I2C1_stop_bit
#endif

#ifdef I2C_MODULE_2 
    #define I2C_Module_open I2C2_open
    #define I2C_Module_baud_rate I2C2_baud_rate
    #define I2C_Module_wait_bus_IDLE I2C2_wait_bus_IDLE
    #define I2C_Module_start_bit I2C2_start_bit
    #define I2C_Module_read_byte I2C2_read_byte
    #define I2C_Module_reset_write_collision_flag I2C2_reset_write_collision_flag
    #define I2C_Module_write_byte I2C2_write_byte 
    #define I2C_Module_stop_bit I2C2_stop_bit
#endif

//*************************************
//     Function prototypes
//*************************************
void write_data (uint8_t data);

//*************************************
//           I2C Master
//*************************************

void main(void) {

    IO_set_all_ports_as_inputs();
    
    //Enable Pull-up resistor on PORTB
    IO_enable_pull_up_resistors (IO_PORTB, IO_BIT4 + IO_BIT5 + IO_BIT6 + IO_BIT7);
    
    //I2C Initialization
    //Baudrate 400kHz @20MHz    
	I2C_Module_baud_rate (400);  
    I2C_Module_open (I2C_MASTER);    
 
   
    //Check for any pressed button
    while (1) {

        switch (PORTB & BUTTON_MASK) {

            case BUTTON_1_PRESSED : write_data (BUTTON_1);                                  
                                    break;
                                    
            case BUTTON_2_PRESSED : write_data (BUTTON_2);
                                    break;
                                    
            case BUTTON_3_PRESSED : write_data (BUTTON_3);
                                    break;
                                    
            case BUTTON_4_PRESSED : write_data (BUTTON_4);
                                    break;
           }
    }
    
}

//*************************************
//    write_data implementation
//*************************************
 void write_data (uint8_t data){

    uint8_t dummy_read;

    //check if the bus is Idle, before continuing
    I2C_Module_wait_bus_IDLE();

    I2C_Module_start_bit();

    //Lettura fittizia per cancellare il buffer in ingresso
    dummy_read = I2C_Module_read_byte ();

    // Invio del primo Byte (indirizzo) - Scrittura
    while (I2C_Module_write_byte( SLAVE_ADDRESS | WRITE )) {
       data = I2C_Module_read_byte ();
       I2C_Module_reset_write_collision_flag ();
    }

    // Invio del secondo Byte (data)
    while(I2C_Module_write_byte (data));

    I2C_Module_wait_bus_IDLE ();

    I2C_Module_stop_bit ();
 }
