/*******************************************************************************

Author : Dario Di Turi
Version : 1.4
Date : 08/12/2012

Last Update: Mauro Laurenti
Last Update: 28/07/2019
 
CopyRight 2012-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Dario Di Turi (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

This library contains all functions needed to control the DDS waveform generator
AD9833. Information on the AD9833 can be found on the website
Analog Devices www.analog.com	

More details and pictures can be downloaded from www.LaurTec.com	
 	
*******************************************************************************/


#ifndef LTLIB_AD9833_H
#define LTLIB_AD9833_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_SPI_MODULE_H
    #error The AD9833 Library requires the "module_SPI" library to be included.
#endif


//*****************************************
//   SPI Module Switch
//*****************************************
#if defined (AD9833_SPI_MODULE_1) || defined (AD9833_SPI_MODULE_2)

#else
    #define AD9833_SPI_MODULE_1
#endif 

#ifdef AD9833_SPI_MODULE_1
    #define AD9833_SPI_open SPI1_open
    #define AD9833_SPI_baudrate SPI1_baudrate
    #define AD9833_SPI_mode SPI1_mode
    #define AD9833_SPI_write_byte SPI1_write_byte
#endif

#ifdef AD9833_SPI_MODULE_2 
    #define AD9833_SPI_open SPI2_open
    #define AD9833_SPI_baudrate SPI2_baudrate
    #define AD9833_SPI_mode SPI2_mode
    #define AD9833_SPI_write_byte SPI2_write_byte
#endif


//**************************************************
// PIN CONNECTIONS
// Use the default pin to SPI connection
// SDATA(SDO) --> It depends on the MCU 
// SCLK(SCK)  --> It depends on the MCU 
// FSYNC(SS)  --> free to be chosen
//
//**************************************************
#ifdef COMPILER_XC8
    #ifdef _PIC18
        #define FSYNC LATAbits.LATA5
        #define FSYNC_DIRECTION TRISAbits.RA5
    #endif

    #ifndef _PIC18
        #define FSYNC PORTAbits.RA5
        #define FSYNC_DIRECTION TRISAbits.TRISA5
    #endif
#endif

#ifdef COMPILER_XC32
    #define FSYNC PORTAbits.RA0
    #define FSYNC_DIRECTION TRISAbits.TRISA0
#endif


//**************************************************
// COSTANTS
//**************************************************

#define AD9833_RESET 0x0100
#define AD9833_REG0 0
#define AD9833_REG1 1
#define AD9833_DOWN 0
#define AD9833_UP 1
#define AD9833_SIN 1
#define AD9833_TRIANGLE 2
#define AD9833_SQUARE 3
#define AD9833_FREQUENCY0_OUT 4
#define AD9833_FREQUENCY1_OUT 5
#define AD9833_PHASE0_OUT 6
#define AD9833_PHASE1_OUT 7
#define AD9833_SLEEP_MODE 9
#define AD9833_POWER_ON 10


//****************************************************************************************************
//    PRIVATE FUNCTIONS (The user should not use it)
//****************************************************************************************************

/**
 * This function write the data to the SPI bus
 *
 * @param data to write to the SPI bus  
 *
 * @return void
 *
 * @note This is a private function, the user should not use it.
 * Change this function to fit other microcontroller models. 
 */
void AD9833_write_data (uint16_t data);


/**
 * This function compose the configuration word to set AD9833 features
 *
 * @param function_type is a type of function to activate
 *
 * @return configuration word to activate the functionality required
 *
 * @note This is a private function, the user should not use it.
 */
uint16_t AD9833_compose_control_word (uint8_t function_type);


//****************************************************************************************************
//    PUBLIC FUNCTIONS
//****************************************************************************************************

/**
 * AD9833 Initialization. This function should be called before using the AD9833
 * RESET command, write 0 into register FREQ0,FREQ1,PHASE0 and PHASE1
 * Set sinusoidal waveform, FREQ0 and PHASEO out
 *
 * @param fmclk is a master frequency [0...25MHz]
 *
 * @return void
 *
 */
void AD9833_initialize (uint8_t fmclk);


/**
 * This function writes the frequency value into FREQ0 or FREQ1 register
 *
 * @param freq_value is a frequency [0...12500000Hz]
 *
 * @param frequency_register is the register to write
 * 					0,REG0-->REGISTER0
 * 					1,REG1-->REGISTER1
 * @return void
 *
 */
void AD9833_set_frequency (uint32_t freq_value, uint8_t frequency_register);


/**
 * This function increases or decreases the LSB register frequency value of a
 * step bit for a tuning frequency
 *
 * @param tuning_step is a step bit [0...255]
 *
 * @param direction is a direction of increases or decreases
 * 					0,DOWN-->decrease 
 * 					1,UP  -->increase
 *
 * @param frequency_register is the register to write
 * 					0,REG0-->REGISTER0
 * 					1,REG1-->REGISTER1
 * @return void
 *
 */
void AD9833_tuning_frequency (uint8_t tuning_step, uint8_t direction, uint8_t frequency_register );


/**
 * This function writes the phase value into PHASE0 or PHASE1 register
 *
 * @param phase_value is a phase [0...3600]
 *
 * @param phase_register is the register to write
 * 					0,REG0-->REGISTER0
 * 					1,REG1-->REGISTER1
 * @return void
 *
 */
void AD9833_set_phase (uint32_t phase_value, uint8_t phase_register);


/**
 * This function activates the AD9833 waveform generator
 *
 * @param function_type is the function to activate
 * 			1,SIN-->sinusoidal waveform
 * 			2,TRIANGLE-->triangular waveform
 * 			3,SQUARE-->square waveform
 * 			4,FREQUENCY0_OUT-->FREQ0 register is used in the phase accumulator
 * 			5,FREQUENCY1_OUT-->FREQ1 register is used in the phase accumulator
 * 			6,PHASE0_OUT-->PHASE0 register data is added to the output of the phase accumulator
 * 			7,PHASE1_OUT-->PHASE1 register data is added to the output of the phase accumulator
 * 			9,SLEEP_MODE-->DAC powered down and internal clock disabled
 * 		   10,POWER_ON-->no power down
 * 
 * @return void
 *
 */
void AD9833_set_function (uint8_t function_type);

#endif

