/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Created on Date : 11/03/2023
Last update     : 11/03/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************


This library contains all the functions that are handy for controlling 			
a GLCD display based on ST7565 controller.						
 	

*******************************************************************************/

#ifndef LTLIB_GLCD_ST7565_H
#define LTLIB_GLCD_ST7565_H

#define GLCD_DRIVER_H

#ifdef COMPILER_XC
    #include <xc.h>    
#endif


#ifdef COMPILER_XC16
    #define NOP_DELAY Nop();
#endif

#ifdef COMPILER_XC32
    #define NOP_DELAY delay_us (5);
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The GLCD_ST7565 Library requires the "delay" library to be included.
#endif

#ifndef LTLIB_UTILITIES_H
    #error The GLCD_ST7565 Library requires the "LTlib_utilities" library to be included.
#endif

#ifndef LTLIB_SPI_MODULE_H
    #error The GLCD_ST7565 Library requires the "module_SPI" library to be included.
#endif

//**************************************************
// HW Initialization
// This section must be properly initialized according
// to the GLCD connections
//**************************************************

//**************************************************
// Device pin Settings CS1B, RST, A0 
//**************************************************

#ifdef COMPILER_XC16
   
    //Control bit declaration
    #define DISPLAY_CS1B LATAbits.LATA4
    #define DISPLAY_RST  LATCbits.LATC3
    #define DISPLAY_A0   LATCbits.LATC6
    #define DISPLAY_LED  LATBbits.LATB10

    //Control bit direction
    #define DISPLAY_CS1B_DIRECTION TRISAbits.TRISA4
    #define DISPLAY_RST_DIRECTION  TRISCbits.TRISC3
    #define DISPLAY_A0_DIRECTION   TRISCbits.TRISC6
    #define DISPLAY_LED_DIRECTION  TRISBbits.TRISB10
#endif


#ifdef COMPILER_XC32

    //Control bit declaration
    #define DISPLAY_CS1B LATAbits.LATA4
    #define DISPLAY_RST  LATCbits.LATC3
    #define DISPLAY_A0   LATCbits.LATC6
    #define DISPLAY_LED  LATBbits.LATB10

    //Control bit direction
    #define DISPLAY_CS1B_DIRECTION TRISAbits.TRISA4
    #define DISPLAY_RST_DIRECTION  TRISCbits.TRISC3
    #define DISPLAY_A0_DIRECTION   TRISCbits.TRISC6
    #define DISPLAY_LED_DIRECTION  TRISBbits.TRISB10

#endif

//*****************************************
//   SPI Module Switch
//*****************************************
#if defined (ST7565_SPI_MODULE_1) || defined (ST7565_SPI_MODULE_2) || defined (ST7565_SPI_MODULE_3)

#else
    #define ST7565_SPI_MODULE_1
#endif 


#ifdef ST7565_SPI_MODULE_1
    #define ST7565_SPI_open SPI1_open
    #define ST7565_SPI_baudrate SPI1_baudrate
    #define ST7565_SPI_mode SPI1_mode 
    #define ST7565_SPI_write_byte SPI1_write_byte
    #define ST7565_SPI_is_TX_over SPI1_is_TX_over
#endif

#ifdef ST7565_SPI_MODULE_2 
    #define ST7565_SPI_open SPI2_open
    #define ST7565_SPI_baudrate SPI2_baudrate
    #define ST7565_SPI_mode SPI2_mode
    #define ST7565_SPI_write_byte SPI2_write_byte
    #define ST7565_SPI_is_TX_over SPI2_is_TX_over
#endif

#ifdef ST7565_SPI_MODULE_3 
    #define ST7565_SPI_open SPI3_open
    #define ST7565_SPI_baudrate SPI3_baudrate
    #define ST7565_SPI_mode SPI3_mode
    #define ST7565_SPI_write_byte SPI3_write_byte
    #define ST7565_SPI_is_TX_over SPI3_is_TX_over
#endif

//**************************************************
// Font info
//**************************************************
extern const uint8_t font_4x5[96][4];
extern const uint8_t font_5x7[96][5];
extern const uint8_t font_7x7[96][7];


//**************************************************
// Display Controller constants
//**************************************************

#define DISPLAY_INPUT 0xFF
#define DISPLAY_OUTPUT 0x00

#ifndef DISPLAY_NUMBER_OF_PAGES
    #define DISPLAY_NUMBER_OF_PAGES 8
#endif

//The GLCD resolution can be set differently before including the library
#ifndef DISPLAY_X_RESOLUTION
    #define DISPLAY_X_RESOLUTION 128
#endif

#ifndef DISPLAY_Y_RESOLUTION
    #define DISPLAY_Y_RESOLUTION 64
#endif


//**************************************************
// GLCD command constants 
//**************************************************

#define DISPLAY_COMM_SET_PAGE              0xB0    //0b10110000
#define DISPLAY_COMM_SET_COLUMN_H          0x10    //0b00010000
#define DISPLAY_COMM_SET_COLUMN_L          0x00    //0b00000000
#define DISPLAY_COMM_DISPLAY_START_LINE    0x40    //0b00000000
#define DISPLAY_COMM_DISPLAY_ON_OFF        0xAE    //0b10101110

//**************************************************
// Internal buffer
//**************************************************
uint8_t Display_memory [DISPLAY_X_RESOLUTION][DISPLAY_NUMBER_OF_PAGES];


//****************************************************************************************************
//    PUBLIC FUNCTIONS
//****************************************************************************************************

/**
 * GLCD Initialization. This function should be called before using the GLCD.
 * Controller are initialized, backlight is set to OFF, Display is cleared and font is set.
 *
 * @param void
 *
 * @return void
 *
 */
void  Display_initialize (void);


/**
 * This function configure via software the display orientation, enabling 
 * a Nord or South orientation.
 *
 * @param orientation Orientation to be used [GLCD_ORIENTATION_NORTH...GLCD_ORIENTATION_SOUTH]
 *
 * @return void
 *
 */
void  Display_set_orientation (uint8_t orientation);


/**
 * GLCD Reset. The data is not kept. It is an hardware reset via the RST pin.
 *
 * @param void
 *
 * @return void
 *
 */
void  Display_Reset (void);


/**
 * This function clears the display either in black or white color. 
 *
 * @param filling_color Filling color allows to set the display either on white or black colour [GLCD_FILLING_WHITE, GLCD_FILLING_BLACK]
 *
 * @return void
 *
 */
void Display_clear (uint8_t filling_color);


/**
 * This function clears the display either in black or white color. 
 *
 * @param contrast_value Select the contrast [0..100]
 *
 * @return void
 *
 */
void Display_set_contrast (uint8_t contrast_value);


/**
 * This function plots a single dot either in black or white color. 
 * Plot position is set accordingly to x,y.
 * 
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [GLCD_FILLING_WHITE, GLCD_FILLING_BLACK]
 *
 * @return void
 *
 */
void Display_plot_xy (uint8_t x_pos,  uint8_t y_pos, uint8_t filling_color);


/**
 * This function turns ON-OFF the GLCD back light.
 *
 * @param backlight Control the backlight LED.
 *         GLCD_TURN_ON_LED  : Backlight is ON
 *         GLCD_TURN_OFF_LED : Backlight is OFF
 *
 * @return void
 *
 */
void Display_backlight (uint8_t backlight);


/**
 * This function sets the display turning it ON or OFF.
 *
 * @param display_status Turn ON or OFF the display [GLCD_TURN_ON_DISPLAY, GLCD_TURN_OFF_DISPLAY]
 *
 * @return void
 *
 * @note If the display is turned OFF the RAM retains the data.
 */
void Display_set_display (uint8_t display_status);


/**
 * This function sets the font. Currently only one font is supported so is not yet that important.
 * The initialization function is calling it so there is no reason to call it till new fonts will be supported.
 *
 * @param width Font width.
 *
 * @param width Font height.
 *
 * @return void
 *
 */
void Display_set_font (uint8_t width, uint8_t height);


/**
 * This function sets the font color being either black or white
 *
 * @param font_color Font width. Value [GLCD_FILLING_WHITE or GLCD_FILLING_BLACK]
 *
 * @return void
 *
 */
void Display_set_font_color (uint8_t font_color);


/**
 * This function enable/disable the double buffer.
 * If the double buffer is enabled the data is wirtten to the buffer only,
 * not to the GLCD controller. To write the data you have to execute the 
 * GLCD_update_double_buffer (); function
 * 
 * @param set_state State for the Double Buffer [ GLCD_DOUBLE_BUFFER_OFF, GLCD_DOUBLE_BUFFER_ON]
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_set_double_buffer_state (uint8_t set_state);


/**
 * This function writes the double buffer content to the controller. 
 *
 * @param void 
 *
 * @return void
 *
 */
void Display_update_double_buffer (void);


//****************************************************************************************************
//    PRIVATE FUNCTIONS (The user should not use it)
//****************************************************************************************************


/**
 * This function writes a command to the controller. 
 * The available commands are GLCD command constants. 
 * For each command there is a dedicated function. The User should use these rather than sending commands.
 *
 * @param command_to_send Command to be sent to the Controller.  
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */

void Display_write_command (uint8_t command_to_send);


/**
 * This function writes data to the controller. 
 *
 * @param data_to_send Data byte to be sent to the Controller.  
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */

void Display_write_data (uint8_t data_to_send);


/**
 * This function sets the controller page (0 to GLCD_NUMBER_OF_PAGES).
 *
 * @param controller_page Controller page [0..GLCD_NUMBER_OF_PAGES].
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_set_page (uint8_t controller_page);


/**
 * This function sets the controller start line address (0 to 63). 
 *
 * @param controller_start_line Controller line address [0..63]. 
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_set_start_line (uint8_t controller_start_line);


/**
 * This function sets the controller column (0 to 127). 
 *
 * @param controller_address Controller address [0..127]. 
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_set_column (uint8_t controller_column);


/**
 * This function writes a data byte to the controller. 
 *
 * @param data_to_send Data byte to be sent out. 
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_write_data (uint8_t data_to_send);


#endif // GLCD_ST7565_H

