/*******************************************************************************

Author : Mauro Laurenti
Version : 1.6
 
Created on Date : 04/09/2006
Last update     : 09/02/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

These functions allow the user to read/write inside the eeprom 
that requires 2 address bytes such as the 24LC512.

********************************************************************************/

#ifndef LTLIB_I2C_EEPROM_H
#define LTLIB_I2C_EEPROM_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The I2C_EEPROM Library requires the "delay" library to be included.
#endif

#ifndef LTLIB_I2C_MODULE_H
    #error The I2C_EEPROM Library requires the "module_I2C" library to be included.
#endif


//*****************************************
//   I2C Module Switch
//*****************************************

#if defined (EEPROM_I2C_MODULE_1) || defined (EEPROM_I2C_MODULE_2)

#else
    #define EEPROM_I2C_MODULE_1
#endif 


#ifdef EEPROM_I2C_MODULE_1
    #define EEPROM_I2C_open I2C1_open
    #define EEPROM_I2C_baud_rate I2C1_baud_rate  
    #define EEPROM_I2C_wait_bus_IDLE I2C1_wait_bus_IDLE
    #define EEPROM_I2C_start_bit I2C1_start_bit
    #define EEPROM_I2C_stop_bit I2C1_stop_bit
    #define EEPROM_I2C_write_byte I2C1_write_byte
    #define EEPROM_I2C_check_bus_collision I2C1_check_bus_collision
    #define EEPROM_I2C_check_ACK I2C1_check_ACK
    #define EEPROM_I2C_negative_ACK I2C1_negative_ACK
    #define EEPROM_I2C_set_master_as_receiver I2C1_set_master_as_receiver
    #define EEPROM_I2C_read_byte I2C1_read_byte
    #define EEPROM_I2C_restart_communication I2C1_restart_communication
#endif

#ifdef EEPROM_I2C_MODULE_2 
    #define EEPROM_I2C_open I2C2_open
    #define EEPROM_I2C_baud_rate I2C2_baud_rate  
    #define EEPROM_I2C_wait_bus_IDLE I2C2_wait_bus_IDLE
    #define EEPROM_I2C_start_bit I2C2_start_bit
    #define EEPROM_I2C_stop_bit I2C2_stop_bit
    #define EEPROM_I2C_write_byte I2C2_write_byte
    #define EEPROM_I2C_check_bus_collision I2C2_check_bus_collision
    #define EEPROM_I2C_check_ACK I2C2_check_ACK
    #define EEPROM_I2C_negative_ACK I2C2_negative_ACK
    #define EEPROM_I2C_set_master_as_receiver I2C2_set_master_as_receiver
    #define EEPROM_I2C_read_byte I2C2_read_byte
    #define EEPROM_I2C_restart_communication I2C2_restart_communication
#endif


/**
 * This function initialize the I2C module accordingly to the clock and baud rate.
 *
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void I2C_EEPROM_initialize (uint16_t baud_rate_KHz);


/**
 * This function writes a byte inside an I2C external EEPROM. 
 *
 * @param data Byte to write [min: 0, max: 255]
 *
 * @param address Address where the byte must be written [min: 0, max: depends on the EEPROM]
 *
 * @param control Is the address of the EEPROM set with the external address pins and the internal one. The bit R/'W set to 0. (e.g 0xA0)  
 *
 * @return status 0: The byte has been properly written
 *       		 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note This function only supports EEPROM which have two bytes for the address [min EEPROM is 25L32].
 */
int8_t I2C_EEPROM_write_byte (uint8_t control, uint16_t address, uint8_t data );


/**
 * This function writes a byte inside the EEPROM but it reads back the value to make sure that it has been properly
 * written. It's slower than the other one due to this feature.
 *
 * @param data Byte to write [min: 0, max: 255]
 *
 * @param address Address where the byte must be written [min: 0, max: depends on the EEPROM]
 *
 * @param control Is the address of the EEPROM set with the external address pins and the internal one. The bit R/'W set to 0. (e.g 0xA0)  
 *
 * @return status 0: The byte has been properly written
 *       		 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *               -4: Readback error
 *
 * @note This function only supports EEPROM which have two bytes for the address [min EEPROM is 25L32].
 * @note To make this library works you need to include the delay library as well.
 */
int8_t I2C_EEPROM_write_byte_check (uint8_t control, uint16_t address, uint8_t data);


/**
 * This function reads a byte from an external I2C EEPROM.
 *
 *
 *
 * @param control Is the address of the EEPROM set with the external address pins and the internal one. The bit R/'W set to 0. (e.g 0xA0)  
 *
 * @param address Address where the byte must be read out [min: 0, max: depends on the EEPROM]
 *
 * @param *data Address of the variable where the read data will be written into.
 *
 * @return status 0: The byte has been properly written
 *       		 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note This function only supports EEPROM which have two bytes for the address [min EEPROM is 25L32].
 */
int8_t I2C_EEPROM_read_byte (uint8_t control, uint16_t address, uint8_t *data );


#endif