/*******************************************************************************

Author : Mauro Laurenti
Version : 1.8
Created on Date : 09/03/2012
Last update     : 28/03/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************


 This library contains all the functions that are handy for controlling 			
 an LCD with a 44780 Hitachi controller via an I2C interface made with a PCF8574							
 	
Before using the functions you have to properly initialize the I2C module to 100KHz.
hardware connections are (using Eagle CAD nomenclature):

PCF8574 P0 -> LCD D4 
PCF8574 P1 -> LCD D5 
PCF8574 P2 -> LCD D6 
PCF8574 P3 -> LCD D7 
PCF8574 P4 -> LCD E 
PCF8574 P5 -> LCD R/W 
PCF8574 P6 -> LCD RS 
PCF8574 P7 -> LCD LED (use a PNP transistor as buffer to drive the backlight LED).

SDA, SCL must be connected to the I2C buffer with pull-up resistors.
A0-A1-A2 address pins must be set as required.
Set PCF8574_ADDRESS_L (see below)  accordingly to the address pis A0-A1-A2.

More details and pictures can be downloaded from the Brief Note BN0015 (www.LaurTec.com)

**********************************************************************************************/


#ifndef LTLIB_LCD_44780_I2C_H
#define LTLIB_LCD_44780_I2C_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************

#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The LCD_44780_I2C Library requires the "LTlib_delay" library to be included.
#endif

#ifndef LTLIB_UTILITIES_H
    #error The LCD_44780 Library requires the "LTlib_utilities" library to be included.
#endif

#if defined (LTLIB_MCP2300X_H) || defined (LTLIB_PCF8574_H) 
  
#else
        #error The LCD_44780_I2C Library requires the "MCP2300x" or "PCF8574" library to be included.
#endif

//**************************************************
// PCF8574 Address
// PCF8574_ADDRESS_H refers to the embedded address (PCF8574 model)
// PCF8574_ADDRESS_L refers to the A0, A1, A2 pins
//**************************************************	

//#define PCF8574
//#define PCF8574A
//#define MCP23008

#if defined (PCF8574) || defined (PCF8574A) || defined (MCP23008)
  
#else
    #define MCP23008
#endif

//**************************************************
//                 Device Address
// 
//**************************************************

//PCF8574
#ifdef PCF8574
    #define DEVICE_ADDRESS_H 0x40
    #define DEVICE_ADDRESS_L 0x00 
    #define PCF8574_GENERIC
#endif 

//PCF8574A
#ifdef PCF8574A
    #define DEVICE_ADDRESS_H 0x70
    #define DEVICE_ADDRESS_L 0x00 
    #define PCF8574_GENERIC
#endif 

#ifdef MCP23008
    // DEVICE_ADDRESS_H is not needed since the MCP23008 library has it already
    // so it is just set to 0x00 (not to 0x04)
    #define DEVICE_ADDRESS_H 0x00 
    #define DEVICE_ADDRESS_L 0x00 
#endif 

//**************************************************
// LCD constants
// Each bit is connected to the PCF8574/MC23008 
//**************************************************

// Standard LaurTec Definition
#define LCD_D0  1
#define LCD_D1  2
#define LCD_D2  4	
#define LCD_D3  8	
#define LCD_E   16	
#define LCD_RW  32
#define LCD_RS  64	
#define LCD_LED 128

//PCF8574 LCD module adapter layout (may differ from product to product)
//#define LCD_D0  16
//#define LCD_D1  32
//#define LCD_D2  64	
//#define LCD_D3  128	
//#define LCD_E   4	
//#define LCD_RW  2
//#define LCD_RS  1	
//#define LCD_LED 8

//*********************************************************
//                  LCD Back Light driver
//*********************************************************

#define LCD_DRIVER_NPN
//#define LCD_DRIVER_PNP

//*********************************************************
//   Depending on the chip a different baud rate is set
//*********************************************************
#define BUS_DATA_RATE_LCD 100
#define LCD_BUS_DATA_RATE 100       

#ifdef MCP23008

    #undef BUS_DATA_RATE_LCD
    #undef LCD_BUS_DATA_RATE

    #define BUS_DATA_RATE_LCD 100
    #define LCD_BUS_DATA_RATE 100       
#endif


//**************************************************
//               Constant Definitions
//**************************************************

#define LEFT    0x00
#define RIGHT   0x04
#define LCD_LEFT  0x00
#define LCD_RIGHT 0x04

#define TURN_ON_CURSOR      0x02
#define TURN_OFF_CURSOR     0x00
#define LCD_TURN_ON_CURSOR  0x02
#define LCD_TURN_OFF_CURSOR 0x00

#define BLINKING_ON  0x01
#define BLINKING_OFF 0x00
#define LCD_BLINKING_ON  0x01
#define LCD_BLINKING_OFF 0x00

#define ZERO_CLEANING_ON 0x01
#define ZERO_CLEANING_OFF 0x00
#define LCD_ZERO_CLEANING_ON 0x01
#define LCD_ZERO_CLEANING_OFF 0x00

//*********************************************************
//   Depending on the chip a different ON-OFF state is used
//*********************************************************

#ifdef LCD_DRIVER_NPN
    #define LCD_TURN_ON_LED  LCD_LED
    #define LCD_TURN_OFF_LED 0x00  
#endif

#ifdef LCD_DRIVER_PNP
    #define LCD_TURN_ON_LED  0x00
    #define LCD_TURN_OFF_LED LCD_LED
#endif


//**************************************************
//               SHIFT Definitions
//**************************************************
//This number of shift definition, is just to avoid magic
//numbers in the code and avoid warnings 

#define LCD_SHIFT_01 1
#define LCD_SHIFT_02 2
#define LCD_SHIFT_03 3
#define LCD_SHIFT_04 4
#define LCD_SHIFT_05 5
#define LCD_SHIFT_06 6
#define LCD_SHIFT_07 7

#define LCD_BITMASK_BIT_0 1
#define LCD_BITMASK_BIT_1 2
#define LCD_BITMASK_BIT_2 4
#define LCD_BITMASK_BIT_3 8
#define LCD_BITMASK_BIT_4 16
#define LCD_BITMASK_BIT_5 32
#define LCD_BITMASK_BIT_6 64
#define LCD_BITMASK_BIT_7 128


//**************************************************

/**
 * This function generates the Enable pulse.
 *
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void LCD_enable_pulse (void);


/**
 * This function generates the Enable pulse.
 *
 *
 * @param data Data to be sent [0..255]
 *
 * @warning This function is a Private one. It should not be used by the user.
 */
void LCD_send_command (uint8_t data);


/**
 * This function locates the cursor at home location. First line first character.
 *
 */
void LCD_home(void);


/**
 * This function shifts the LCD screen on the left or right.
 *
 * @param shift Specify where the shift should be [LCD_LEFT, LCD_RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780 library.
 */
void LCD_shift (uint8_t shift, uint8_t number_of_shift);


/**
 * This function shifts the LCD cursor on the left or right. The position of the cursor is where the
 * next writting will be performed.
 *
 * @param shift Specify where the shift should be [LCD_LEFT, LCD_RIGHT].
 *
 * @param number_of_shift Specify the number of times the shift is executed.
 *
 * @note Use the constants LEFT, RIGHT to specify the direction
 *       to ensure the compatibility with LCD_44780 library.
 */
void LCD_shift_cursor (uint8_t shift, uint8_t number_of_shift);


/**
 * This function locates the LCD cursor on the selected line. Tested on 20x4 16x2 LCD displays.
 *
 * @param line Specify the number of the line [1,2,3,4]
 *
 * @warning It might not work with all the LCD Displays.
 */
void LCD_goto_line (uint8_t line);


/**
 * This function locates the LCD cursor to an arbitrary X Y location.
 * Y represents the line number from top.
 *
 * @param x It Specifies horizontal position
 *
 * @param y It Specifies vertical position (line number)
 *
 * @warning It might not work with all the LCD Displays. Tested on 20x4 16x2 LCD displays.
 */
void LCD_goto_xy (uint8_t x, uint8_t y);


/**
 * This function writes a char to the LCD display. (e.g LCD_write_char ('a'); )
 *
 * @param value Specify the character to be sent [0-128 ASCII code].
 *
 * @note The function accept ASCII integer or constants 'a'..'z'.
 */
void LCD_write_char (uint8_t value);


/**
 * This function writes a const string to the LCD display, e.g LCD_write_message ("Hello"); .
 *
 * @param buffer Is a const string written in rom (is not an array).
 *
 * @note The function accepts strings written in rom (constant), it does not accept array.
 *       To write arrays refers the write_string_LCD () function.
 *
 */
void LCD_write_message (const char *buffer);


/**
 * This function writes an array of char to the LCD display, terminated with /0 .
 *
 * @param buffer It is an array of char terminated with /0 .
 *
 * @note The function accepts strings  within an array. To use const array within rom use write_message_LCD ().
 *
 */
void LCD_write_string (uint8_t *buffer);


/**
 * This function writes a byte in the CGRAM. This allows setting up a custom font. 
 *
 * @param address CGRAM address to write.
 * 
 * @param data Data byte to write
 *
 * @note The font that is supported in 2x Lines is 5x8 only.
 * @note The function returns home (LCD_home) at the end. This resets the Address 
 *       counter and enables the new fonts to be requested from the DDRAM.
 * 
 */
void LCD_write_data_CGRAM(uint8_t address, uint8_t data);


/**
 * This function writes an integer to the LCD display. The integer is converted to string.
 *
 * @param value It is integer that must be written to the LCD display.
 *
 * @param numeber_of_digits It specifies the number of shown digit [0-5].
 *                    0: Left Justified
 *                    1-5: Right Justified with n digit
 *
 * @param zero_cleaning It specifies whether the left zero must be removed [LCD_ZERO_CLEANING_ON, LCD_ZERO_CLEANING_OFF].
 *
 * @note If you set a number of digit less than required the digit will be lost starting from the less
 *       significant digit. Minus is like a digit.
 *
 */
void LCD_write_integer(int16_t value, uint8_t number_of_digits, uint8_t zero_cleaning);


/**
 * This function writes a variable that contains a real number. 
 *
 * @param value Float value to be written (either positive or negative)
 *
 * @param number_of_digits The number is formatted with a fixed number of digits to ensure a constant string length.
 * 
 * @param number_of_fractional The number of digits representing fractional. No approximation is done.
 *
 * @return void
 *
 */
void LCD_write_float (float value, uint8_t number_of_digits, uint8_t number_of_fractional_digits, uint8_t zero_cleaning);



/**
 * This Function cleans the LCD display.
 *
 */
void LCD_clear (void);


/**
 * This function controls the cursor option (blinking, active)
 *
 * @param active Activate the cursor, showing it or not [LCD_TURN_ON_CURSOR, LCD_TURN_OFF_CURSOR]
 *
 * @param blinking  Let the cursor blink [LCD_BLINK_ON, LCD_BLINK_OFF]
 *
 * @note You must use the constant LCD_TURN_ON_CURSOR, LCD_TURN_OFF_CURSOR, LCD_BLINKING_ON, LCD_BLINKING_OFF
 *       to ensure the compatibility with LCD_44780 library.
 *
 */
void LCD_cursor (uint8_t active, uint8_t blinking);


/**
 * This function controls the backligth LED.
 *
 * @param active Activate the LED backlight [LCD_TURN_ON_LED, LCD_TURN_OFF_LED]
 *
 * @param blinking  Let the cursor blink [LCD_BLINKING_ON, LCD_BLINKING_OFF]
 *
 * @note You must use the constant LCD_TURN_ON_LED, LCD_TURN_OFF_LED
 *       to ensure the compatibility with LCD_44780 library.
 */
void LCD_backlight (uint8_t active);


/**
 * This function initializes the LCD to work in 4 bit modality.
 *
 * @param quartz_frequency Quartz freq. expressed in MHz used to run the CPU.
 *
 *
 * @note You must properly set the microcontroller pins using the TRISx registers
 *       The library requires the delay library to be included.
 */
void  LCD_initialize (uint8_t quartz_frequency);

#endif
