/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
  
Created on Date : 02/10/2017
Last update     : 29/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy for controlling 			
the SPI module
*******************************************************************************/

#ifndef LTLIB_MAX6675_H
#define LTLIB_MAX6675_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The MAX6675 Library requires the "delay" library to be included.
#endif

#ifndef LTLIB_SPI_MODULE_H
    #error The MAX6675 Library requires the "module_SPI" library to be included.
#endif


//*****************************************
//   SPI Module Switch
//*****************************************
#if defined (MAX6675_SPI_MODULE_1) || defined (MAX6675_SPI_MODULE_2)

#else
    #define MAX6675_SPI_MODULE_1
#endif 


#ifdef MAX6675_SPI_MODULE_1
    #define MAX6675_SPI_open SPI1_open
    #define MAX6675_SPI_baudrate SPI1_baudrate
    #define MAX6675_SPI_mode SPI1_mode 
    #define MAX6675_SPI_read_byte SPI1_read_byte
#endif

#ifdef MAX6675_SPI_MODULE_2 
    #define MAX6675_SPI_open SPI2_open
    #define MAX6675_SPI_baudrate SPI2_baudrate
    #define MAX6675_SPI_mode SPI2_mode
    #define MAX6675_SPI_read_byte SPI2_read_byte
#endif

//**************************************************
// Device Settings
//**************************************************
#ifdef COMPILER_XC8
    #ifdef _PIC18
        //CS pin
        #define MAX6675_CS_TRIS     TRISDbits.RD3
        #define MAX6675_CS_PORT     LATDbits.LATD3
    #endif

    #ifndef _PIC18
        //CS pin
        #define MAX6675_CS_TRIS     TRISDbits.TRISD3
        #define MAX6675_CS_PORT     PORTDbits.RD3
    #endif
#endif

#ifdef COMPILER_XC32
    //CS pin
    #define MAX6675_CS_TRIS     TRISBbits.TRISB4
    #define MAX6675_CS_PORT     LATBbits.LATB4
#endif


//**************************************************
// Constants
//**************************************************

#define MAX6675_CLOCK_FOSC_64 SPI_CLK_OSC_64
#define MAX6675_CLOCK_FOSC_16 SPI_CLK_OSC_16
#define MAX6675_CLOCK_FOSC_4 SPI_CLK_OSC_4
#define MAX6675_CLOCK_FOSC_TMR2 SPI_TIMER_OUT_CLK

#define MAX6675_CS_DISABLED 0x01
#define MAX6675_CS_ENABLED 0x00

#define MAX6675_DEVICE_CONNECTED 0x01
#define MAX6675_DEVICE_DISCONNECTED 0x00
#define MAX6675_CONNECTEDED_BIT_MASK 0x0002

#define MAX6675_PROBE_CONNECTED 0x01
#define MAX6675_PROBE_DISCONNECTED 0x00
#define MAX6675_PROBE_BIT_MASK 0x0004


/**
 * This function initializes the MAX6675 and PIC signals. SDO, SS, SCK from the
 * PIC side are automatically set by this function.
 *
 * @param clock It sets the clock source for the SPI module. 
 *              PIC18: Use the following constants [MAX6675_CLOCK_FOSC_64, MAX6675_CLOCK_FOSC_16, MAX6675_CLOCK_FOSC_4, MAX6675_CLOCK_FOSC_TMR2]
 *              PIC32: Write the bus clock in KHz.
 *
 * @return void
 *
 * @warning You must properly set to OFF the Analog inputs which may be muxed with the SPI lines.
 */
void MAX6675_initialize (uint8_t clock);


/**
 * This function reads 16 bits out of the MAX6675. The data must be parsed
 * to get the temperature out
 *
 * @param void
 *
 * @return register 16 bits register value out of the temperature sensor.
 *
 * @note This function is a private one. It should not be used by the user
 */
uint16_t MAX6675_read_register (void);


/**
 * This function reads the temperature out of the sensor. It returns an integer value
 * removing the decimal side of the reading. The function should be called with a period 
 * longer than 22ms, which is the conversion time.
 *
 * @param void
 *
 * @return temperaure Integer value of the temeprature.
 *
 * @note While the temperature sensor could have 0.25C accuracy the decimal part
 * of the reading is discarded via data truncation (without approximation).
 */
uint16_t MAX6675_read_temperature (void);


/**
 * This function checks if the IC is connected on the bus. 
 * Indeed the IC has a defined bit ID that can be monitored to validate the 
 * communication in the bus.
 *
 * @param void
 *
 * @return value It returns if the IC is connected [MAX6675_DEVICE_CONNECTED, MAX6675_DEVICE_DISCONNECTED]
 *
 * @note If there is an error, the temmperature value should be discarded.
 */
uint8_t MAX6675_is_connected (void);


/**
 * This function checks if the probe is connected to the IC. 
 * Indeed the IC has a defined bit ID that can be monitored to validate the 
 * probe connection.
 *
 * @param void
 *
 * @return value It returns if the probe is connected [MAX6675_PROBE_CONNECTED, MAX6675_PROBE_DISCONNECTED]
 *
 * @note If there is an error, the temmperature value should be discarded.
 */
uint8_t MAX6675_is_probe_connected (void);

#endif
