/*******************************************************************************

Author : Mauro Laurenti
Version : 1.3
Created on Date : 07/02/2014
Last update     : 31/01/2019
 
CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

These functions allow the user to read/write to the data
port of the I2C I/O expander MCP2300x device.

*******************************************************************************/

#ifndef LTLIB_MCP2300X_H
#define LTLIB_MCP2300X_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The MCP2300X Library requires the "module_I2C" library to be included.
#endif

//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (MCP2300X_I2C_MODULE_1) || defined (MCP2300X_I2C_MODULE_2)

#else
    #define MCP2300X_I2C_MODULE_1
#endif 


#ifdef MCP2300X_I2C_MODULE_1
    #define MCP2300X_I2C_open I2C1_open
    #define MCP2300X_I2C_baud_rate I2C1_baud_rate  
    #define MCP2300X_I2C_write_byte_to_external_device I2C1_write_byte_to_external_device
    #define MCP2300X_I2C_read_byte_from_external_device I2C1_read_byte_from_external_device
#endif

#ifdef MCP2300X_I2C_MODULE_2 
    #define MCP2300X_I2C_open I2C2_open
    #define MCP2300X_I2C_baud_rate I2C2_baud_rate 
    #define MCP2300X_I2C_write_byte_to_external_device I2C2_write_byte_to_external_device
    #define MCP2300X_I2C_read_byte_from_external_device I2C2_read_byte_from_external_device
#endif

//*****************************************
//   Device settings
//*****************************************

#define MCP2300X_INTERNAL_ADDRESS 0x40

//*****************************************
//   Constant definitions
//*****************************************

#define MCP2300x_IODIR   0x00
#define MCP2300x_IPOL    0x01
#define MCP2300x_GPINTEN 0x02
#define MCP2300x_DEFVAL  0x03
#define MCP2300x_INTCON  0x04
#define MCP2300x_IOCON   0x05
#define MCP2300x_GPPU    0x06
#define MCP2300x_INTF    0x07
#define MCP2300x_INTCAP  0x08
#define MCP2300x_GPIO    0x09
#define MCP2300x_OLAT    0x0A


#define MCP2300x_SEQOP       0b00100000
#define MCP2300x_DISSLW      0b00010000
#define MCP2300x_HAEN        0b00001000 // Not implemented by I2C interface
#define MCP2300x_ODR         0b00000100
#define MCP2300x_INTPOL      0b00000010

#define MCP2300x_CLEAR_REG   0b00000000


/**
 * This function initialize the I2C module accordingly to the clock and baud rate.
 *
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several call from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void MCP2300x_initialize (uint16_t baud_rate_KHz);

 
/**
 * This function writes into any register of the MCP2300x device.
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param register_add Address of the register that must be written. 
 *
 * @param data Data that must be written into the register. 
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    For the register address you can use the name of the register as in the data sheet. 
 * The same name, upper case, are defined in the header file.
 *
 */
int8_t MCP2300x_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data );


/**
 * This function reads from any register of the MCP2300x device.
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param register_add Address of the register that must be written. 
 *
 * @return Content out of the pointed register.
 *
 * @note    For the register address you can use the name of the register as in the data sheet. 
 * The same name, upper case, are defined in the header file.
 *
 */
uint8_t MCP2300x_get_register (uint8_t device_address, uint8_t register_add);


/**
 * This function writes into IODIR register, defining the direction of the pins.
 * A 1 in a bit is equivalent to set the corresponding pin as input, while a 0 sets it to Output.
 *
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param direction Value that defines the direction of each pin. 
 *
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 */
int8_t MCP2300x_set_port_direction (uint8_t device_address, uint8_t direction);


/**
 * This function reads from the GPIO register, which is equivalent to read the value of the external pins.
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @return It returns the values of the external pins.
 *
 *
 */
uint8_t MCP2300x_get_port_value (uint8_t device_address);


/**
 * This function writes into the GPIO register, which is equivalent to write to the output port.
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param value Value that must be written to the output port. 
 *
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note Writing to the output port affects only the pins that are set as output.
 *
 */
int8_t MCP2300x_set_port_value (uint8_t device_address, uint8_t value);


/**
 * This function writes into IPOL register, which defines the polarity of the input pins.
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param value The value sets the corresponding pin with an inverting value (input only). 
 *
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 */
int8_t MCP2300x_set_port_polarity (uint8_t device_address, uint8_t value);


/**
 * This function writes into GPPU register, which is equivalent to activate the pull-up resistor.
 * A bit set to 1 enables the pull-up resistor to the corresponding pin. This option affect only
 * the pins that are set as inputs. 
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param value Value to enable the pins where the pull-up resistor must be activated. 
 *
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 */
int8_t MCP2300x_set_port_pull_up_resistor (uint8_t device_address, uint8_t value);


/**
 * This function writes into GPINTEN register, which is equivalent to activate the interrupts.
 * A bit set to 1 enables the interrupt to the corresponding pin. This option affects only
 * pins that are set as inputs.
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param value Value to enable the pins where the interrupt must be activated. 
 *
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @warning The interrupt of the MCP2300x device must be enabled before the Microcontroller 
 * General Interrupt Enable.
 * A dummy read of the GPIO register is internally performed to clear any pending interrupt.
 */
int8_t MCP2300x_set_port_interrupt (uint8_t device_address, uint8_t value);


/**
 * This function writes into IOCON register, enabling some internal configurations.
 * SEQOP, DISSLW, ODR, INTPOL
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param value Value that reflects the pins where the comparison must be enabled. 
 *
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note Refer to the data sheet for more details on available options. Constants are defined in the
 * to enable and easy use of the settings. 
 */
int8_t MCP2300x_set_port_configuration (uint8_t device_address, uint8_t value);


/**
 * This function writes into DEFVAL register, which is equivalent to set the value for the interrupt
 * comparison. If bit 2 is set to 1 a 0 into pin 2 will trigger the interrupt.
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param value Byte that reflects the value of interest to be used for the comparison. 
 *
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note Refer to the data sheet for more details on the interrupt on change and on comparison with DEFVAL register.
 */
int8_t MCP2300x_set_interrupt_compare_value (uint8_t device_address, uint8_t value);


/**
 * This function writes into INTCON register, which is equivalent to select the interrupt modality.
 * A zero sets an interrupt on pin change, while a 1 enable the comparison with the DEFVAL Register.
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @param value Value that enables the pins where the comparison must be activated.
 *
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note Refer to the data sheet for more details on the interrupt on change and on comparison with DEFVAL register
 */
int8_t MCP2300x_set_interrupt_compare_enable (uint8_t device_address, uint8_t value);


/**
 * This function reads from the INTF register, enabling the read of the pin flags that caused the interrupt.
 *
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @return Content of the interrupt flag register.
 *
 * @note   This register is read only. A bit set to 1 it means that the input has generated an interrupt. 
 *
 */
uint8_t MCP2300x_get_port_interrupt_flag (uint8_t device_address);


/**
 * This function reads from the INTCAP register, enabling the read of the pin flags that caused the 
 * interrupt. In particular INTCAP register it captures the value at the time of the interrupt 
 * so it may differ from the value of INTF register.
 *
 *
 * @param device_address Hardware Address of the device (A0, A1, A2 only) [min: 0, max: 7].
 *
 * @return Content of the interrupt flag register captured at the time of the interrupt event.
 *
 * @note   This register is read only. A bit set to 1 it means that the input has generated an interrupt. 
 *
 */
uint8_t MCP2300x_get_port_interrupt_capture (uint8_t device_address);

#endif
