/*******************************************************************************

Author : Mauro Laurenti
Version : 1.4

Created on Date : 08/09/2014
Last update     : 28/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

 This library allows the usage of the MCP4822 DAC.

*******************************************************************************/

#ifndef LTLIB_MCP4822_H
#define LTLIB_MCP4822_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_SPI_MODULE_H
    #error The MCP4822 Library requires the "module_SPI" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The MCP4822 Library requires the "LTlib_delay" library to be included.
#endif


//*****************************************
//   SPI Module Switch
//*****************************************
#if defined (MCP4822_SPI_MODULE_1) || defined (MCP4822_SPI_MODULE_2)

#else
    #define MCP4822_SPI_MODULE_1
#endif 


#ifdef MCP4822_SPI_MODULE_1
    #define MCP4822_SPI_open SPI1_open
    #define MCP4822_SPI_baudrate SPI1_baudrate
    #define MCP4822_SPI_mode SPI1_mode 
    #define MCP4822_SPI_write_byte SPI1_write_byte
    #define MCP4822_SPI_read_byte SPI1_read_byte
#endif

#ifdef MCP4822_SPI_MODULE_2 
    #define MCP4822_SPI_open SPI2_open
    #define MCP4822_SPI_baudrate SPI2_baudrate
    #define MCP4822_SPI_mode SPI2_mode
    #define MCP4822_SPI_write_byte SPI2_write_byte
    #define MCP4822_SPI_read_byte SPI2_read_byte
#endif

//**************************************************
// Device pin Settings CS, LAT
//**************************************************

#ifdef COMPILER_XC8
    #ifdef _PIC18
        //CS pin
        #define MCP4822_CS_TRIS     TRISAbits.RA5
        #define MCP4822_CS_PORT     LATAbits.LATA5

        //LAT pin
        #define MCP4822_LDAC_TRIS   TRISEbits.RE0
        #define MCP4822_LDAC_PORT   LATEbits.LATE0
    #endif

    #ifndef _PIC18
        //CS pin
        #define MCP4822_CS_TRIS     TRISAbits.TRISA5
        #define MCP4822_CS_PORT     PORTAbits.RA5

        //LAT pin
        #define MCP4822_LDAC_TRIS   TRISEbits.TRISE0
        #define MCP4822_LDAC_PORT   PORTEbits.RE0
    #endif
#endif

#ifdef COMPILER_XC32
        //CS pin
        #define MCP4822_CS_TRIS     TRISBbits.TRISB4
        #define MCP4822_CS_PORT     PORTBbits.RB4

        //LAT pin
        #define MCP4822_LDAC_TRIS   TRISAbits.TRISA0
        #define MCP4822_LDAC_PORT   PORTAbits.RA0
#endif

//**************************************************
// Constants
//**************************************************

#define MCP4822_DAC_A 0x00
#define MCP4822_DAC_B 0x80

#define MCP4822_GAIN_1 0x20
#define MCP4822_GAIN_2 0x00

#define MCP4822_OUT_DEFAULT_VALUE 0x00

//When shutdown is ON the device is powered down.
#define MCP4822_SHUTDOWN_ON  0x00
#define MCP4822_SHUTDOWN_OFF 0x10

#define MCP4822_CLOCK_FOSC_64 SPI_CLK_OSC_64
#define MCP4822_CLOCK_FOSC_16 SPI_CLK_OSC_16
#define MCP4822_CLOCK_FOSC_4 SPI_CLK_OSC_4
#define MCP4822_CLOCK_FOSC_TMR2 SPI_TIMER_OUT_CLK

#define MCP4822_CS_DISABLED 0x01
#define MCP4822_CS_ENABLED 0x00

#define MCP4822_LDAC_DISABLED 0x01
#define MCP4822_LDAC_ENABLED 0x00

/**
 * This function initializes the MCP4822 and PIC signals. SDO, SS, SCK from the
 * PIC side are automatically set by this function.
 *
 * @param clock It sets the clock source for the SPI module. Use the following
 *              constants [MCP4822_CLOCK_FOSC_64, MCP4822_CLOCK_FOSC_16, MCP4822_CLOCK_FOSC_4, MCP4822_CLOCK_FOSC_TMR2]
 *
 * @return void
 *
 * @note By default the DAC outputs are set to high impedance.
 *
 * @warning You must properly set the Analog inputs which are muxed with the SPI lines.
 */
void MCP4822_initialize (uint8_t clock);


/**
 * This function sets the output value of the DAC. By Gain 2 the value is expressed in mV,
 * while by Gain 1 is 1/2 mV.
 *
 * @param value It sets the output value of the DAC [12 bits].
 *
 * @param channel It sets the channel to which the value refers to [MCP4822_DAC_A, MCP4822_DAC_B].
 *
 * @return void
 *
 * @note This function does not generate the LATCH pulse to load the DAC values.
 *       This it means that the pulse latch pulse must be generated separately.
 *       This option is used in case both DAC must be set before loading the
 *       output value.
 */
void MCP4822_set_value (uint16_t value, uint8_t channel);


/**
 * This function sets the output value of the DAC. By Gain 2 the value is expressed in mV,
 * while by Gain 1 is 1/2 mV.
 *
 * @param value It sets the output value of the DAC [12 bits].
 *
 * @param channel It sets the channel to which the value refers to [MCP4822_DAC_A, MCP4822_DAC_B].
 *
 * @return void
 *
 * @note This function does generate the LATCH pulse to load the DAC value right after
 *       the SPI writing.
 *
 */
void MCP4822_set_amplitude (uint16_t value, uint8_t channel);


/**
 * This function generates the LATCC pulse required to load the value to the DAC.
 * It is required only in case the MSP4822_set_value is used. The function
 * MSP4822_set_amplitude already include a call to this function.
 *
 * @param void
 *
 * @return void
 * 
 */
void MCP4822_latch_pulse (void);


/**
 * This function sets the DAC Options. The options which are set in the function,
 * take place after the next writing to the corresponding DAC Channel.
 *
 * @param gain It sets the output gain [MCP4822_GAIN_1, MCP4822_GAIN_2]
 *
 * @param power It sets the channel power mode [MCP4822_SHUTDOWN_ON, MCP4822_SHUTDOWN_OFF]
 *
 * @param channel It sets the channel to which the options refer to [MCP4822_DAC_A, MCP4822_DAC_B]
 *
 * @return void
 *
 * @note This function does NOT generate the LATCH pulse to load the DAC value right after
 *       the writing. The settings take place by the next amplitude writing. 
 *
 */
void MCP4822_set_options (uint8_t gain, uint8_t power, uint8_t channel);

#endif