/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Created on Date : 18/02/2024
Last update     : 18/02/2024

CopyRight 2006-2024 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************


This library contains all the functions that are handy for controlling 			
an OLED display based on SSD1306 controller.						
 	

*******************************************************************************/

#ifndef LTLIB_OLED_SSD1306_H
#define LTLIB_OLED_SSD1306_H

//If included, this enables the display_utility library
#define OLED_DRIVER_H

#ifdef COMPILER_XC
    #include <xc.h>    
#endif


#ifdef COMPILER_XC16
    #define NOP_DELAY Nop();
#endif

#ifdef COMPILER_XC32
    #define NOP_DELAY delay_us (5);
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The OLED_SSD1306 Library requires the "delay" library to be included.
#endif

#ifndef LTLIB_UTILITIES_H
    #error The OLED_SSD1306 Library requires the "LTlib_utilities" library to be included.
#endif

#ifndef LTLIB_I2C_MODULE_H
    #error The OLED_SSD1306 Library requires the "module_I2C" library to be included.
#endif


//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (SSD1306_I2C_MODULE_1) || defined (SSD1306_I2C_MODULE_2)

#else
    #define SSD1306_I2C_MODULE_1
#endif 

#ifdef SSD1306_I2C_MODULE_1
    #define SSD1306_I2C_open I2C1_open
    #define SSD1306_I2C_baud_rate I2C1_baud_rate  
    #define SSD1306_I2C_read_byte_from_external_device I2C1_read_byte_from_external_device
    #define SSD1306_I2C_write_array_to_external_device I2C1_write_array_to_external_device

#endif

#ifdef SSD1306_I2C_MODULE_2 
    #define SSD1306_I2C_open I2C2_open
    #define SSD1306_I2C_baud_rate I2C2_baud_rate 
    #define SSD1306_I2C_read_byte_from_external_device I2C2_read_byte_from_external_device
    #define SSD1306_I2C_write_array_to_external_device I2C2_write_array_to_external_device
#endif

//*****************************************
//   Device Address options
//*****************************************
//7 bit definition
#define SSD1306_DEVICE_ADDRESS_0 0x3C
#define SSD1306_DEVICE_ADDRESS_1 0x3D

#ifndef  SSD1306_ADDRESS
    #define SSD1306_ADDRESS SSD1306_DEVICE_ADDRESS_0
#endif  

//*****************************************
//   I2C Device baudrate
//*****************************************


#ifndef  SSD1306_BIT_RATE
    #define SSD1306_BIT_RATE 400
#endif  

//**************************************************
// HW Initialization
// This section must be properly initialized according
// to the display connections
//**************************************************

//**************************************************
// Device pin Settings 
//**************************************************

//Note OLED displays do not need back-light LED.

#ifdef COMPILER_XC16

    //Add PIC specific settings here

#endif


#ifdef COMPILER_XC32

    //Add PIC specific settings here

#endif

//**************************************************
// Font info
//**************************************************
extern const uint8_t font_4x5[96][4];
extern const uint8_t font_5x7[96][5];
extern const uint8_t font_7x7[96][7];


//**************************************************
// Display Controller constants
//**************************************************

#define DISPLAY_INPUT 0xFF
#define DISPLAY_OUTPUT 0x00

#ifndef DISPLAY_NUMBER_OF_PAGES
    #define DISPLAY_NUMBER_OF_PAGES 8
#endif

//The DISPLAY resolution can be set differently before including the library
#ifndef DISPLAY_X_RESOLUTION
    #define DISPLAY_X_RESOLUTION 128
#endif

#ifndef DISPLAY_Y_RESOLUTION
    #define DISPLAY_Y_RESOLUTION 64
#endif


//**************************************************
// Display command constants 
//**************************************************

#define DISPLAY_COMM_SET_PAGE              0xB0    //0b10110000
#define DISPLAY_COMM_SET_COLUMN_H          0x10    //0b00010000
#define DISPLAY_COMM_SET_COLUMN_L          0x00    //0b00000000
#define DISPLAY_COMM_DISPLAY_START_LINE    0x40    //0b01000000
#define DISPLAY_COMM_DISPLAY_ON_OFF        0xAE    //0b10101110


//**************************************************
// Display general purpose constants 
//**************************************************
#define DISPLAY_INVERT_ON   0x01
#define DISPLAY_INVERT_OFF  0x00
        
        
        
//**************************************************
// Internal buffer
//**************************************************
uint8_t Display_memory [DISPLAY_X_RESOLUTION][DISPLAY_NUMBER_OF_PAGES];


//****************************************************************************************************
//    PUBLIC FUNCTIONS
//****************************************************************************************************

/**
 * Display Initialization. This function should be called before using the Display.
 * Controller are initialized, backlight is set to OFF, Display is cleared and font is set.
 *
 * @param void
 *
 * @return void
 *
 */
void  Display_initialize (void);


/**
 * This function configures via software the display orientation, enabling 
 * a Nord or South orientation.
 *
 * @param orientation Orientation to be used [DISPLAY_ORIENTATION_NORTH...DISPLAY_ORIENTATION_SOUTH]
 *
 * @return void
 *
 */
void  Display_set_orientation (uint8_t orientation);


/**
 * This function adds an offset to the Y axes, creating a shift effect of the display
 *
 * @param display_offset Offset value [0...255]
 *
 * @return void
 *
 */
void  Display_set_offset (uint8_t display_offset);



/**
 * Display Reset. The data is not kept. It is an hardware reset via the RST pin.
 *
 * @param void
 *
 * @return void
 *
 */
void  Display_Reset (void);



/**
 * This function clears the display either in black or white color. 
 *
 * @param filling_color Filling color allows to set the display either on white or black colour [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 */
void Display_clear (uint8_t filling_color);


/**
 * This function invert the ON OFF perspective, without changing the RAM content.
 * If ON the picture/Text is off and background is ON.  
 *
 * @param invert_status Define if the invert is ON or OFF [DISPLAY_INVERT_ON, DISPLAY_INVERT_OFF]
 *
 * @return void
 *
 */
void Display_set_invert (uint8_t invert_status);



/**
 * This function clears the display either in black or white color. 
 *
 * @param contrast_value Select the contrast [0..100]
 *
 * @return void
 *
 */
void Display_set_contrast (uint8_t contrast_value);



/**
 * This function plots a single dot either in black or white color. 
 * Plot position is set accordingly to x,y.
 * 
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 */
void Display_plot_xy (uint8_t x_pos,  uint8_t y_pos, uint8_t filling_color);


/**
 * This function turns ON-OFF the Display back light.
 *
 * @param backlight Control the backlight LED.
 *         DISPLAY_TURN_ON_LED  : Backlight is ON
 *         DISPLAY_TURN_OFF_LED : Backlight is OFF
 *
 * @return void
 *
 */
void Display_backlight (uint8_t backlight);


/**
 * This function sets the display turning it ON or OFF.
 *
 * @param display_status Turn ON or OFF the display [DISPLAY_TURN_ON, DISPLAY_TURN_OFF]
 *
 * @return void
 *
 * @note If the display is turned OFF the RAM retains the data.
 */
void Display_set_display (uint8_t display_status);


/**
 * This function sets the font. Currently only one font is supported so is not yet that important.
 * The initialization function is calling it so there is no reason to call it till new fonts will be supported.
 *
 * @param width Font width.
 *
 * @param width Font height.
 *
 * @return void
 *
 */
void Display_set_font (uint8_t width, uint8_t height);


/**
 * This function sets the font color being either black or white
 *
 * @param font_color Font width. Value [DISPLAY_FILLING_WHITE or DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 */
void Display_set_font_color (uint8_t font_color);


/**
 * This function enable/disable the double buffer.
 * If the double buffer is enabled the data is written to the buffer only,
 * not to the Display controller. To write the data you have to execute the 
 * Display_update_double_buffer (); function
 * 
 * @param set_state State for the Double Buffer [ DISPLAY_DOUBLE_BUFFER_OFF, DISPLAY_DOUBLE_BUFFER_ON]
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_set_double_buffer_state (uint8_t set_state);


/**
 * This function writes the double buffer content to the controller. 
 *
 * @param void 
 *
 * @return void
 *
 */
void Display_update_double_buffer (void);


//****************************************************************************************************
//    PRIVATE FUNCTIONS (The user should not use it)
//****************************************************************************************************


/**
 * This function writes a command to the controller. 
 * The available commands are Display command constants. 
 * For each command there is a dedicated function. The User should use these rather than sending commands.
 *
 * @param command_to_send Array of command to be sent to the Controller.  
 * @param length Command data length.
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */

void Display_write_command (uint8_t *command_to_send, uint8_t length);


/**
 * This function writes data to the controller. 
 *
 * @param data_to_send Data byte to be sent to the Controller.  
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */

void Display_write_data (uint8_t data_to_send);


/**
 * This function sets the controller page (0 to DISPLAY_NUMBER_OF_PAGES).
 *
 * @param controller_page Controller page [0..DISPLAY_NUMBER_OF_PAGES].
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_set_page (uint8_t controller_page);


/**
 * This function sets the controller start line address (0 to 63). 
 *
 * @param controller_start_line Controller line address [0..63]. 
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_set_start_line (uint8_t controller_start_line);


/**
 * This function sets the controller column (0 to 127). 
 *
 * @param controller_address Controller address [0..127]. 
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_set_column (uint8_t controller_column);


/**
 * This function writes a data byte to the controller. 
 *
 * @param data_to_send Data byte to be sent out. 
 *
 * @return void
 *
 * @note This is a private function. The user should not use it.
 */
void Display_write_data (uint8_t data_to_send);

#endif 

