/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 20/10/2019
Last update     : 20/10/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library supports the light sensor OPT3001

*******************************************************************************/
#ifndef LTLIB_OPT3001_H
#define LTLIB_OPT3001_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The OPT3001 Library requires the "module_I2C" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The OPT3001 Library requires the "LTlib_delay" library to be included.
#endif


//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (OPT3001_I2C_MODULE_1) || defined (OPT3001_I2C_MODULE_2)

#else
    #define OPT3001_I2C_MODULE_1
#endif 

#ifdef OPT3001_I2C_MODULE_1
    #define OPT3001_I2C_open I2C1_open
    #define OPT3001_I2C_baud_rate I2C1_baud_rate  
    #define OPT3001_I2C_write_word_to_external_device I2C1_write_word_to_external_device
    #define OPT3001_I2C_read_word_from_external_device I2C1_read_word_from_external_device
#endif

#ifdef OPT3001_I2C_MODULE_2 
    #define OPT3001_I2C_open I2C2_open
    #define OPT3001_I2C_baud_rate I2C2_baud_rate 
    #define OPT3001_I2C_write_word_to_external_device I2C2_write_word_to_external_device
    #define OPT3001_I2C_read_word_from_external_device I2C2_read_word_from_external_device
#endif


//*****************************************
//   Register settings
//*****************************************

#define OPT3001_REG_RESULT          0x00
#define OPT3001_REG_CONFIGURATION	0x01
#define OPT3001_REG_LOW_LIMIT		0x02
#define OPT3001_REG_HIGH_LIMIT		0x03
#define OPT3001_REG_MANUFACTURER_ID	0x7E
#define OPT3001_REG_DEVICE_ID		0x7F

#define OPT3001_MANUFACTURER_ID 	0x5449
#define OPT3001_DEVICE_ID			0x3001

#define OPT3001_ADDRESS_1   0x44
#define OPT3001_ADDRESS_2   0x45
#define OPT3001_ADDRESS_3   0x46
#define OPT3001_ADDRESS_4   0x47

//Full scale values
#define OPT3001_FULL_SCALES_NR 12U
uint32_t OPT3001_full_scales[OPT3001_FULL_SCALES_NR] = {40, 81, 163, 327, 655, 1310, 2620, 5241, 10483, 20966, 41932, 83865};


//By each device configuration you have to OR one value for each group
//In this way the configuration register will have a valid value

#define OTP3001_CONF_AUTO_FULL_SCALE       	 	0xC000

#define OTP3001_CONF_CONVERSION_TIME_100ms		0x0000
#define OTP3001_CONF_CONVERSION_TIME_800ms		0x0100

#define OTP3001_CONF_MODE_SHUTDOWN				0x0000
#define OTP3001_CONF_MODE_SINGLE_SHOT			0x0200
#define OTP3001_CONF_MODE_CONTINUOUS_SAMPLING	0x0400

#define OTP3001_CONF_INT_PIN_AS_COMPARATOR		0x0000
#define OTP3001_CONF_INT_PIN_AS_INTERRUPT		0x0010

#define OTP3001_CONF_INT_POLARITY_ACTIVE_LOW	0x0000
#define OTP3001_CONF_INT_POLARITY_ACTIVE_HIGH	0x0008

#define OTP3001_CONF_FAULT_COUNTER_1				0x0000
#define OTP3001_CONF_FAULT_COUNTER_2				0x0001
#define OTP3001_CONF_FAULT_COUNTER_4				0x0002
#define OTP3001_CONF_FAULT_COUNTER_8				0x0003


//******************************************************************************


/**
 * This function initializes the I2C bus and OPT3001 sensor.
 *
 * @param bit_rate_KHZ Bit rate for the I2C bus used for the OPT3001. 
 *                     The value is the frequency in KHz [range: 100, 400]
 *
 * @return none
 *
 */
void OPT3001_initialize (uint16_t baud_rate_KHz);
 

/**
 * This function reads the light intensity back from the sensor.
 * The value is expressed already in Lux
 * 
 * @param device_address Device address written in 7bit format.
 *
 * @return Lux read out as float number.
 *
 */
int8_t OPT3001_get_lux (uint8_t device_address, float * lux);


/**
 * This function sets the lux high limit. The light intensity must be set 
 * according to the datasheet data format (exponent and mantissa).
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param lux Lux limit high limit.
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t OPT3001_set_high_limit (uint8_t device_address, uint32_t lux);


/**
 * This function sets the lux low limit. The light intensity must be set 
 * according to the datasheet data format (exponent and mantissa).
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param lux Lux low limit.
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t TMP1075_set_low_limit (uint8_t device_address, uint32_t lux);


/**
 * This function writes a generic register inside the OPT3001.
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param register_address Register inside the OPT3001 to be written.
 * 
 * @param value Value to be written inside the register. 
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t OPT3001_set_register (uint8_t device_address, uint8_t register_address, uint16_t value);


/**
 * This function reads a generic register inside the OPT3001.
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param register_address Register inside the OPT3001 to be read.
 * 
 * @param readback_value Value readback from the register. 
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t OPT3001_get_register (uint8_t device_address, uint8_t register_address, uint16_t * readback_value);


/**
 * This function reads the Device ID out of the OPT3001 device.
 *
 * @param device_address Device address in 7 bits format.
 *
 * @param device_ID Pointer to a uint16_t variable that will contain the Device ID. 
 *
 * @return status 1: The byte has been properly written
 *       	     -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 */
int8_t OPT3001_get_device_ID (uint8_t device_address, uint16_t * device_ID);


/**
 * This function writes the configuration register inside the OPT3001.
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param configuration Configuration to be written inside the TMP1075. 
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t OPT3001_configuration (uint8_t device_address, uint16_t configuration);


/**
 * This function shell be considered a private one.
 * It converts the value in lux in the 16 bits format accepted by the OPT3001
 * registers. The 16 bits OPT3001 register are made of exponent and mantissa.  
 *
 * @param lux Light intensity expressed in lux as integer value.
 * 
 * @return converted 16 bit format made of exponent and mantissa as specified
 *         by the OPT3001 datasheed.
 *
 */
uint16_t OPT3001_lux_converter (uint32_t lux);
        

#endif

