/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 18/10/2019
Last update     : 18/10/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library supports the temperature sensor TMP1075.

*******************************************************************************/
#ifndef LTLIB_TMP1075_H
#define LTLIB_TMP1075_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_I2C_MODULE_H
    #error The TMP1075 Library requires the "module_I2C" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The TMP1075 Library requires the "LTlib_delay" library to be included.
#endif


//*****************************************
//   I2C Module Switch
//*****************************************
#if defined (TMP1075_I2C_MODULE_1) || defined (TMP1075_I2C_MODULE_2)

#else
    #define TMP1075_I2C_MODULE_1
#endif 

#ifdef TMP1075_I2C_MODULE_1
    #define TMP1075_I2C_open I2C1_open
    #define TMP1075_I2C_baud_rate I2C1_baud_rate  
    #define TMP1075_I2C_write_word_to_external_device I2C1_write_word_to_external_device
    #define TMP1075_I2C_read_word_from_external_device I2C1_read_word_from_external_device
#endif

#ifdef TMP1075_I2C_MODULE_2 
    #define TMP1075_I2C_open I2C2_open
    #define TMP1075_I2C_baud_rate I2C2_baud_rate 
    #define TMP1075_I2C_write_word_to_external_device I2C2_write_word_to_external_device
    #define TMP1075_I2C_read_word_from_external_device I2C2_read_word_from_external_device
#endif


//*****************************************
//   Register settings
//*****************************************

#define TMP1075_REG_TEMPERATURE_RESULTS 0x00
#define TMP1075_REG_CONFIGURATION       0x01
#define TMP1075_REG_LOW_LIMIT           0x02
#define TMP1075_REG_HIGH_LIMIT          0x03
#define TMP1075_REG_DEVICE_ID           0x0F

#define TMP1075_DEVICE_ID 0x7500

//By each device configuration you have to OR one value for each group
//In this way the configuration register will have a valid value
#define TMP1075_CONF_ONESHOT_ON             0x8000
#define TMP1075_CONF_ONESHOT_OFF            0x0000

#define TMP1075_CONF_COVERTION_TIME_27ms    0x0000
#define TMP1075_CONF_COVERTION_TIME_55ms    0x2000
#define TMP1075_CONF_COVERTION_TIME_110ms   0x4000
#define TMP1075_CONF_COVERTION_TIME_220ms   0x6000

#define TMP1075_CONF_FAULT_COUNTER_1        0x0000
#define TMP1075_CONF_FAULT_COUNTER_2        0x0800
#define TMP1075_CONF_FAULT_COUNTER_3        0x1000
#define TMP1075_CONF_FAULT_COUNTER_4        0x1800

#define TMP1075_CONF_ALERT_ACTIVE_LOW       0x0000
#define TMP1075_CONF_ALERT_ACTIVE_HIGH      0x0400

#define TMP1075_CONF_ALERT_AS_INTERRUPT     0x0200
#define TMP1075_CONF_ALERT_AS_COMPARATOR    0x0000

#define TMP1075_CONF_SHUTDOWN_ENABLED       0x0100
#define TMP1075_CONF_SHUTDOWN_DISABLED      0x0000

//******************************************************************************


/**
 * This function initializes the I2C bus and TMP1075 sensor.
 *
 * @param bit_rate_KHZ Bit rate for the I2C bus used for the TMP1075. 
 *                     The value is the frequency in KHz [range: 100, 400]
 *
 * @return none
 *
 */
void TMP1075_initialize (uint16_t baud_rate_KHz);
 

/**
 * This function reads the temperature back from the sensor.
 *
 * @param device_address Device address written in 7bit format.
 *
 * @return Temperature read out (signed int).
 *
 */
int8_t TMP1075_get_temperature (uint8_t device_address, uint16_t * temperature);


/**
 * This function sets the temperature high limit. The temperature is set in 
 * Celsius without decimal point.
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param temperature Temperature limit in celsius degree.
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t TMP1075_set_high_limit (uint8_t device_address, int16_t temperature);


/**
 * This function sets the temperature low limit. The temperature is set in 
 * Celsius without decimal point.
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param temperature Temperature limit in celsius degree.
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t TMP1075_set_low_limit (uint8_t device_address, int16_t temperature);


/**
 * This function writes a generic register inside the TMP1075.
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param register_address Register inside the TMP1075 to be written.
 * 
 * @param value Value to be written inside the register. 
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t TMP1075_set_register (uint8_t device_address, uint8_t register_address, uint16_t value);


/**
 * This function reads a generic register inside the TMP1075.
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param register_address Register inside the TMP1075 to be read.
 * 
 * @param readback_value Value readback from the register. 
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t TMP1075_get_register (uint8_t device_address, uint8_t register_address, uint16_t * readback_value);


/**
 * This function writes the configuration register inside the TMP1075.
 *
 * @param device_address Device address written in 7bit format.
 * 
 * @param configuration Configuration to be written inside the TMP1075. 
 *
 * @return error out of the I2C module functions.
 *
 */
int8_t TMP1075_configuration (uint8_t device_address, uint16_t configuration);

#endif

