/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 14/04/2023
Last update     : 14/04/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy to use a touch panel.
Currently 4 wires resestive is supported.	

*******************************************************************************/


#ifndef LTLIB_TOUCH_H
#define LTLIB_TOUCH_H


#ifdef COMPILER_XC
    #include <xc.h>
#endif


//*****************************************
//   Library Test
//*****************************************

#include "LTlib.h"

#ifndef LTLIB_ADC_MODULE_H
    #error The Touch Library requires the "module_ADC" library to be included.
#endif

#ifndef LTLIB_IO_MODULE_H
    #error The Touch Library requires the "module_IO" library to be included.
#endif

#ifndef LTLIB_DELAY_H
    #error The Touch Library requires the "delay" library to be included.
#endif

//**************************************************
//            Point Definition
//**************************************************
typedef struct{ 
	uint16_t X;
	uint16_t Y;
} point;

typedef struct{ 
	float gain;
	uint16_t offset;
} calibration;

//**************************************************
//            Pin configuration
//**************************************************
#ifdef COMPILER_XC16

    #define TOUCH_BUZZER_PIN      LATAbits.LATA10
    #define TOUCH_BUZZER_PIN_DIR  TRISAbits.TRISA10

    #define TOUCH_TOP_PIN           LATBbits.LATB15
    #define TOUCH_TOP_PIN_DIR       TRISBbits.TRISB15

    #define TOUCH_BOTTOM_PIN        LATBbits.LATB13
    #define TOUCH_BOTTOM_PIN_DIR    TRISBbits.TRISB13

    #define TOUCH_LEFT_PIN          LATBbits.LATB12
    #define TOUCH_LEFT_PIN_DIR      TRISBbits.TRISB12

    #define TOUCH_RIGHT_PIN         LATBbits.LATB14
    #define TOUCH_RIGHT_PIN_DIR     TRISBbits.TRISB14

    #define TOUCH_RIGHT_PULL_UP_R   CNPU1bits.CN12PUE
    #define TOUCH_BOTTOM_PULL_UP_R  CNPU1bits.CN13PUE

    #define BOTTOM_ADC_CH ADC_CH7
    #define RIGHT_ADC_CH ADC_CH6
#endif

#ifdef COMPILER_XC32

    #define TOUCH_BUZZER_PIN      LATAbits.LATA10
    #define TOUCH_BUZZER_PIN_DIR  TRISAbits.TRISA10

    #define TOUCH_TOP_PIN           LATBbits.LATB4
    #define TOUCH_TOP_PIN_DIR       TRISBbits.TRISB4

    #define TOUCH_BOTTOM_PIN        LATCbits.LATC2
    #define TOUCH_BOTTOM_PIN_DIR    TRISCbits.TRISC2

    #define TOUCH_LEFT_PIN          LATBbits.LATB7
    #define TOUCH_LEFT_PIN_DIR      TRISBbits.TRISB7

    #define TOUCH_RIGHT_PIN         LATCbits.LATC1
    #define TOUCH_RIGHT_PIN_DIR     TRISCbits.TRISC1

    #define TOUCH_RIGHT_PULL_UP_R   CNPUCbits.CNPUC1
    #define TOUCH_BOTTOM_PULL_UP_R  CNPUCbits.CNPUC2

    #define BOTTOM_ADC_CH ADC_CH8
    #define RIGHT_ADC_CH ADC_CH7
#endif

//**************************************************
//            Standard Constants
//**************************************************
#define TOUCH_X_MAX 127
#define TOUCH_Y_MAX 63

#define TOUCH_IS_PRESSED 1
#define TOUCH_IS_NOT_PRESSED 0

#define TOUCH_TOP_RIGHT 1
#define TOUCH_TOP_LEFT 2
#define TOUCH_BOTTOM_RIGHT 3
#define TOUCH_BOTTOM_LEFT 4

#define TOUCH_AVERAGING_2 2
#define TOUCH_AVERAGING_4 4
#define TOUCH_AVERAGING_8 8
#define TOUCH_AVERAGING_16 16
#define TOUCH_AVERAGING_32 32
#define TOUCH_AVERAGING_64 64


#define TOUCH_DEBOUNCE_TIME_ms 10

#define TOUCH_BUZZER_PULSES     100
#define TOUCH_BUZZER_T_ON_us    500
#define TOUCH_BUZZER_T_OFF_us   500


#define TOUCH_INPUT_PIN 0x01
#define TOUCH_OUTPUT_PIN 0x00

#define TOUCH_PULL_UP_ON 0x01
#define TOUCH_PULL_UP_OFF 0x00

#define TOUCH_PIN_TO_GND 0x00
#define TOUCH_PIN_TO_VCC 0x01

// Distance from the corner where the calibration point is placed (unit %).
#define MARGIN 10

//The threshold refers to a full scale 12bits
//#define TOUCH_X_THRESHOLD 3500
//#define TOUCH_Y_THRESHOLD 3200

//The threshold refers to a full scale 10bits
#define TOUCH_X_THRESHOLD 900
#define TOUCH_Y_THRESHOLD 900



//**************************************************
//               Functions prototipes
//**************************************************
/**
 * This function initializes the touch panel with the basic settings
 *
 *
 * @param x_size Display X size in pixel 
 * 
 * @param y_size Display Y size in pixel 
 * 
 *
 * @return void
 *
 *
 */
void Touch_initialize (uint16_t x_size, uint16_t y_size );


/**
 * This function starts the calibration sequence
 *
 *
 * @param x_size Display X size in pixel 
 * 
 * @param y_size Display Y size in pixel 
 * 
 *
 * @return void
 *
 *
 */
void Touch_calibrate (void);


/**
 * This function makes a simple beep by using the Buzzer
 *
 *
 * @param count Number of beeps 
 *
 * @return void
 *
 */
void Touch_beep (uint8_t count);


/**
 * This function is called internally after a measure.
 * It sets back all the Touch panel pins as input, to limit the current.
 * *
 * @param void
 *
 * @return void
 *
 */
void Touch_reset_pins (void);


/**
 * This function makes a simple beep by using the Buzzer
 *
 *
 * @param point_to_calibrate define the point under calibration [TOUCH_TOP_RIGHT, TOUCH_TOP_LEFT, TOUCH_BOTTOM_RIGHT, TOUCH_BOTTOM_LEFT] 
 *
 * @return status [1 : completed   0: error]
 *
 */
uint8_t Touch_calibration_point (uint8_t point_to_calibrate);


/**
 * This function checks if the touch display has been touched/pressed. 
 *
 * @param void
 *
 * @return status [1 : completed   0: error]
 *
 */
uint8_t Touch_is_pressed (void);


/**
 * This function measures the ADC raw data that represent the X and Y position of the 
 * pressed position. A number of averaging can be set to reduce the noise.
 *
 * @param avaraging Number of averaging per measure [TOUCH_AVERAGING_2, TOUCH_AVERAGING_4, TOUCH_AVERAGING_8,TOUCH_AVERAGING_16 ]
 *
 * @return point Raw data from the ADC, representing the X and Y
 *
 */
point Touch_measure_raw (uint8_t averaging);


/**
 * This function measures the ADC raw and compensate it with the calibration data.
 * A number of averaging can be set to reduce the noise.
 *
 * @param avaraging Number of averaging per measure [TOUCH_AVERAGING_2, TOUCH_AVERAGING_4, TOUCH_AVERAGING_8,TOUCH_AVERAGING_16 ]
 *
 * @return Point The measured point contains point.Y and point.Y values 
 *
 */
point Touch_measure (uint8_t averaging);


/**
 * This function creates the corrections factors and offset for the touch points. 
 * It uses the calibration points coordinate to compute it.
 *
 * @param void
 *
 * @return status [1: valid 0 : not valid]
 *
 */
uint8_t Touch_correction (void);


/**
 * This function provides the raw data info that can be used to export calibration data.
 * The raw data are given back as parameters. It is user responsibility to save the data. 
 * 
 * @param Top_Right Point containing the XY info of the Top Right touch 
 * 
 * @param Top_Left Point containing the XY info of the Top Left touch 
 * 
 * @param Bottom_Right Point containing the XY info of the Bottom Right touch 
 * 
 * @param Bottom_Left Point containing the XY info of the Bottom Left touch 
 *
 * @return void
 *
 */
void Touch_export_calibration (point *Top_Right, point *Top_Left, point *Bottom_Right, point *Bottom_Left);


/**
 * This function imports the raw calibration data and compute the calibration factors
 * that the touch functions will use. 
 * 
 * @param Top_Right Point containing the XY info of the Top Right touch 
 * 
 * @param Top_Left Point containing the XY info of the Top Left touch 
 * 
 * @param Bottom_Right Point containing the XY info of the Bottom Right touch 
 * 
 * @param Bottom_Left Point containing the XY info of the Bottom Left touch 
 *
 * @return void
 *
 */
void Touch_import_calibration (point Top_Right, point Top_Left, point Bottom_Right, point Bottom_Left);


#endif

