/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1

Created on Date : 15/11/2018
Last update     : 25/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

This library allows controlling the RGB LEDs WS2812B.
The library is time sensitive and it has been testeed only with 20MHz crystal.
Other frequencies will require time adjustments. 
The PORT assigned to the RGB LEDs cannot be used for other purposes. 

Compiled with MPLAB X v5.05 and XC8 v1.44

*******************************************************************************/

#ifndef LTLIB_WS2812B_H
#define LTLIB_WS2812B_H

#ifdef COMPILER_XC
    #include <xc.h>
    //#define _XTAL_FREQ 20000000
#endif


//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The WS2812B Library requires the "delay" library to be included.
#endif

//**************************************************
// HW Initialization
// This section must be properly initialized according
// to the LEDs groups and numbers
//**************************************************
#ifdef COMPILER_XC8
    #ifdef _PIC18
        //Data PORT
        #define LED_PORT LATD

        // TRIS Register setting
        #define LED_GROUP_TRIS TRISD  

    #endif

    #ifndef _PIC18
        //Data PORT
        #define LED_PORT PORTD

        //  TRIS Register settings
        #define LED_GROUP_TRIS TRISD
    #endif
#endif	

#ifdef COMPILER_XC32
    //Data PORT
    #define LED_PORT LATB

    // TRIS Register setting
    #define LED_GROUP_TRIS TRISB 
#endif	

//**************************************************
//             Constant Definitions
//**************************************************
    #define LED_GROUP_1 0x01   
    #define LED_GROUP_2 0x02
    #define LED_GROUP_3 0x04
    #define LED_GROUP_4 0x08
    #define LED_GROUP_5 0x10   
    #define LED_GROUP_6 0x20
    #define LED_GROUP_7 0x40
    #define LED_GROUP_8 0x80


    #ifndef LED_GROUP_1_SIZE
        #define LED_GROUP_1_SIZE 12
    #endif

    #ifndef LED_GROUP_2_SIZE
        #define LED_GROUP_2_SIZE 12
    #endif

    #ifndef LED_GROUP_3_SIZE
        #define LED_GROUP_3_SIZE 12
    #endif

    #ifndef LED_GROUP_4_SIZE
        #define LED_GROUP_4_SIZE 12
    #endif

    #ifndef LED_GROUP_5_SIZE
        #define LED_GROUP_5_SIZE 12
    #endif

    #ifndef LED_GROUP_6_SIZE
        #define LED_GROUP_6_SIZE 12
    #endif

    #ifndef LED_GROUP_7_SIZE
        #define LED_GROUP_7_SIZE 12
    #endif

    #ifndef LED_GROUP_8_SIZE
        #define LED_GROUP_8_SIZE 12
    #endif

    #define LED_FILTER_RESET 0x00

uint8_t group_size[8] = {LED_GROUP_1_SIZE, LED_GROUP_2_SIZE, LED_GROUP_3_SIZE, LED_GROUP_4_SIZE, LED_GROUP_5_SIZE, LED_GROUP_6_SIZE, LED_GROUP_7_SIZE, LED_GROUP_8_SIZE};

//**************************************************

#define COLOR_DEPTH 8  // 8bits each color
#define COLOR_COUNT 3  // 3 Colors RGB


//**************************************************


/**
 * This function initialize the port where the WS2812B are connected and Reset it.
 * 
 * @note This function should be called beofore anyother one.
 * 
 * @warning The PORT assigned to the RGB LEDs cannot be used for other purposes. 
 */
void WS2812_initialize ();


/**
 * This function reset the group of WS2812B. It should be called only one time 
 * before calling multiple times the WS2812_turn_on_LED. Calling it let the 
 * WS2812_turn_on_LED start from the first LED again in case there is a dasychain. 
 *
 * @param group It identifies the group (port pin) where the LED or string is connected to. [min = 0, max = 7]  
 *
 * @return void
 *
 * @note Calling it reset the LED string (group).
 */
void WS2812_reset_LED (uint8_t group);


/**
 * This function set a color for the selected LED in the selected group. 
 * If the group (string) is made of 3 LEDs you have to call the function 3 times.  
 *
 * @param group It identifies the group (port pin) where the LED or string is connected to. [min = 0, max = 7]  
 *
 * @param red_color Select the RED color intensity 0 is min, while 255 is the maximum intensity. [min = 0, max = 255] 
 * 
 * @param greed_color Select the GREEN color intensity 0 is min, while 255 is the maximum intensity. [min = 0, max = 255] 
 * 
 * @param blue_color Select the BLUE color intensity 0 is min, while 255 is the maximum intensity. [min = 0, max = 255] 
 * 
 * @return void
 *
 * @note Setting all the R, G, B to the same value makes the white.
 */
void WS2812_turn_on_LED (uint8_t group, uint8_t red_color, uint8_t green_color, uint8_t blue_color);


/**
 * This function set the entire selected LED groupt (string) to one color.
 *
 * @param group It identifies the group (port pin) where the LED or string is connected to. [min = 0, max = 7]  
 *
 * @param red_color Select the RED color intensity 0 is min, while 255 is the maximum intensity. [min = 0, max = 255] 
 * 
 * @param greed_color Select the GREEN color intensity 0 is min, while 255 is the maximum intensity. [min = 0, max = 255] 
 * 
 * @param blue_color Select the BLUE color intensity 0 is min, while 255 is the maximum intensity. [min = 0, max = 255] 
 * 
 * @return void
 *
 * @note Setting all the R, G, B to the same value makes the white.
 */
void WS2812_turn_on_group (uint8_t group, uint8_t red_color, uint8_t green_color, uint8_t blue_color);


/**
 * This function turn off the entire LED group (string).
 *
 * @param group It identifies the group (port pin) where the LED or string is connected to. [min = 0, max = 7]  
 *
 * @return void
 *
 */
void WS2812_turn_off_group (uint8_t group);

#endif
