/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 11/03/2023
Last update     : 23/02/2024

CopyRight 2006-2024 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************


This library contains all the functions that supports GLCD dysplays.
The functions are on top of any GLCD controller that get used in the project.						
 	

*******************************************************************************/

#ifndef LTLIB_DISPLAY_UTILITY_H
#define LTLIB_DISPLAY_UTILITY_H

#ifdef COMPILER_XC
    #include <xc.h>    
#endif



//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef LTLIB_UTILITIES_H
    #error The Display_utilities Library requires the "LTlib_utilities" library to be included.
#endif


#if defined (GLCD_DRIVER_H) || defined (OLED_DRIVER_H)

#else
    #error The Display_utilities Library requires a chipset GLCD_DRIVER or OLED_DRIVER library to be included first.
#endif


//**************************************************
// Font Tables and Pictures 
//**************************************************
//All fonts and icons are activated by default.
//Comment what is not needed or undefine it. 

#define DISPLAY_ENABLE_FONT_4x5
#define DISPLAY_ENABLE_FONT_5x7
#define DISPLAY_ENABLE_FONT_7x7

#define DISPLAY_ENABLE_LOGO_1
#define DISPLAY_ENABLE_PICTURE_1

#define DISPLAY_ENABLE_NUMBERS

//**************************************************
// Display Font Size and Colors
//**************************************************

#define DISPLAY_FONT_WIDTH_4   4
#define DISPLAY_FONT_WIDTH_5   5
#define DISPLAY_FONT_WIDTH_7   7

#define DISPLAY_FONT_HEIGTH_5  5
#define DISPLAY_FONT_HEIGTH_7  7

#define DISPLAY_FONT_SPACE 1

//**************************************************
// General Purpose constants 
//**************************************************
#define DISPLAY_ZERO_CLEANING_ON 0x01
#define DISPLAY_ZERO_CLEANING_OFF 0x00

#define DISPLAY_ENABLE         0x01    //0b00000001
#define DISPLAY_DISABLE        0x00    //0b00000000

#define DISPLAY_FILLING_WHITE 0x00
#define DISPLAY_FILLING_BLACK 0xFF

#define DISPLAY_TURN_ON_LED    0x01    //0b00000001
#define DISPLAY_TURN_OFF_LED   0x00    //0b00000000

#define DISPLAY_TURN_ON     0x01    //0b00000001
#define DISPLAY_TURN_OFF    0x00    //0b00000000

#define DISPLAY_ENABLE_RESET   0x00    //0b00000000
#define DISPLAY_DISABLE_RESET  0x01    //0b00000001

#define DISPLAY_DOUBLE_BUFFER_ON 0x01
#define DISPLAY_DOUBLE_BUFFER_OFF 0x00

#define DISPLAY_ORIENTATION_NORTH 0x01  //Orientation 12:00
#define DISPLAY_ORIENTATION_SOUTH 0x00  //Orientation 6:00

#define DISPLAY_HIGH           0x01    //0b00000001
#define DISPLAY_LOW            0x00    //0b00000000

#define DISPLAY_ON_STATE           0x01    //0b00000001
#define DISPLAY_OFF_STATE          0x00    //0b00000000



//****************************************************************************************************
//    PUBLIC FUNCTIONS
//****************************************************************************************************

/**
 * This function turns ON-OFF the GLCD back light.
 *
 * @param backlight Control the backlight LED.
 *         DISPLAY_TURN_ON_LED  : Backlight is ON
 *         DISPLAY_TURN_OFF_LED : Backlight is OFF
 *
 * @return void
 *
 */
void Display_backlight (uint8_t backlight);


/**
 * This function sets the display turning it ON or OFF.
 *
 * @param display_status Turn ON or OFF the display [DISPLAY_TURN_ON_DISPLAY, DISPLAY_TURN_OFF_DISPLAY]
 *
 * @return void
 *
 * @note If the display is turned OFF the RAM retains the data.
 */
void Display_set_display (uint8_t display_status);


/**
 * This function plots a single dot either in black or white color. 
 * Plot position is set accordingly to x,y.
 * 
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 */
void Display_plot_xy (uint8_t x_pos,  uint8_t y_pos, uint8_t filling_color);


/**
 * This function plots an horizontal line either in black or white color. 
 * Origin and length can be set by the user.
 * 
 * @param x_origin Line X origin [0..127].
 *
 * @param y_origin Line Y origin [0..63].
 *
 * @param length Line length [0..127].
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the line length is valid, depending on x_origin, y_origin.
 *
 */
void Display_draw_horizontal_line (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t filling_color);


/**
 * This function plots a vertical line either in black or white color. 
 * Origin and length can be set by the user.
 * 
 * @param x_origin Line X origin [0..128].
 *
 * @param y_origin Line Y origin [0..63].
 *
 * @param length Line length [0..63].
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the line length is valid, depending on x_origin, y_origin.
 *
 */
void Display_draw_vertical_line (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t filling_color);


/**
 * This function plots a vertical line either in black or white color. 
 * Origin and length can be set by the user.
 * 
 * @param x_origin Line X origin [0..128].
 *
 * @param y_origin Line Y origin [0..63].
 * 
 * @param x_end Line X origin [0..128].
 *
 * @param y_end Line Y origin [0..63].
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the line length is valid, depending on x_origin, y_origin.
 *
 */
void Display_draw_line (uint8_t x_origin, uint8_t y_origin, uint8_t x_end, uint8_t y_end, uint8_t filling_color);


/**
 * This function plots an empty window either with black or white color borders. 
 * Origin and length and height can be set by the user.
 * 
 * @param x_origin Window X origin [0..127].
 *
 * @param y_origin Window Y origin [0..63].
 *
 * @param length Window length [0..127].
 *
 * @param length Window height [0..63].
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the window length and height are valid, depending on x_origin, y_origin.
 *
 */
void Display_draw_window (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t height, uint8_t filling_color);


/**
 * This function draws a colored box either in black or white color. 
 * Origin and length and height can be set by the user.
 * 
 * @param x_origin Box X origin [0..127]
 *
 * @param y_origin Box Y origin [0..63]
 *
 * @param length Box length.
 *
 * @param height Box height.
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the window length and height are valid, depending on x_origin, y_origin.
 *
 */
void Display_draw_box (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t height, uint8_t filling_color);


/**
 * This function draws a colored button either in black or white color. 
 * Origin and length and height can be set by the user. The font used for the name 
 * Is the active one.
 * 
 * @param x_origin Box X origin [0..127]
 *
 * @param y_origin Box Y origin [0..63]
 *
 * @param length Box length.
 *
 * @param height Box height.
 * 
 * @param text Text written in the bottom
 *
 * @param filling_color Filling color allows to set the dot either on white or black color [DISPLAY_FILLING_WHITE, DISPLAY_FILLING_BLACK]
 *
 * @return void
 *
 * @warning It is user responsibility to make sure that the window length and height are valid, depending on x_origin, y_origin.
 *
 */
void Display_draw_button (uint8_t x_origin, uint8_t y_origin, uint8_t length, uint8_t height, const char *text, uint8_t filling_color);


/**
 * This function draws a picture with 128x64 resolution.
 *
 * @param picture_table Pointer to the table containing the picture (stored in flash). 
 *
 * @return void
 *
 */
void Display_draw_picture (const uint8_t * picture_table);


/**
 * This function draws an icon of fixed size.
 *
 * @param x_origin Box X origin [0..127]
 *
 * @param y_origin Box Y origin [0..63]
 *
 * @param x_size Icon length.
 *
 * @param y_size Icon height.
 * 
 * @param icon_table Pointer to the table containing the picture (stored in flash). 
 *
 * @return void
 *
 */
void Display_draw_icon (uint8_t x_pos, uint8_t y_pos,uint8_t x_size, uint8_t y_size, const uint8_t * icon_table);


/**
 * This function writes an ASCII character.
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param character ASCII character either as number (unsigned char) or character. Example: 97, 'a'. 
 *
 * @return void
 *
 */
void Display_write_char (uint8_t x_pos, uint8_t y_pos, uint8_t character);


/**
 * This function writes an entire string of char. The string must be terminated with /0.
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param character_array Pointer to the array that contains the string. 
 *
 * @return void
 *
 * @warning The array of unsigned char must be terminated with the /0 character.
 *
 */
void Display_write_string (uint8_t x_pos, uint8_t y_pos, uint8_t * character_array);


/**
 * This function writes a constant string of char. 
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param character_array Pointer to the array that contains the string (stored in flash). 
 *
 * @return void
 *
 */
void Display_write_message (uint8_t x_pos, uint8_t y_pos, const char * character);


/**
 * This function writes variable that contains an integer number. 
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param value Integer value to be written (either positive or negative)
 *
 * @param number_of_digits The number is formatted with a fixed number of digits to ensure a constant string length.
 *
 * @return void
 *
 */
void Display_write_integer (uint8_t x_pos, uint8_t y_pos, int16_t value, uint8_t number_of_digits, uint8_t zero_cleaning);


/**
 * This function writes a variable that contains a real number. 
 *
 * @param x_pos Position X [0..127].
 *
 * @param y_pos Position Y [0..63].
 *
 * @param value Float value to be written (either positive or negative)
 *
 * @param number_of_digits The number is formatted with a fixed number of digits to ensure a constant string length.
 * 
 * @param number_of_fractional The number of digits representing fractional. No approximation is done.
 *
 * @return void
 *
 */
void Display_write_float (uint8_t x_pos, uint8_t y_pos, float value, uint8_t number_of_digits, uint8_t number_of_fractional_digits, uint8_t zero_cleaning);


//************************************************************************************************
//                             Font Tables  FONT_5x7
//************************************************************************************************

#ifdef DISPLAY_ENABLE_FONT_5x7

const uint8_t font_5x7[96][5] =  {              
	{0x00,0x00,0x00,0x00,0x00}, // empty space
	{0x00,0x00,0x5f,0x00,0x00}, // !
 	{0x00,0x03,0x00,0x03,0x00}, // "
	{0x14,0x7f,0x14,0x7f,0x14}, // #
   	{0x24,0x2a,0x7f,0x2a,0x12}, // $
	{0x23,0x13,0x08,0x64,0x62}, // %
	{0x36,0x49,0x55,0x22,0x50}, // &
	{0x00,0x05,0x03,0x00,0x00}, // '
 	{0x00,0x1c,0x22,0x41,0x00}, // (
	{0x00,0x41,0x22,0x1c,0x00}, // )
    {0x14,0x08,0x3e,0x08,0x14}, // *
	{0x08,0x08,0x3e,0x08,0x08}, // +
    {0x00,0x50,0x30,0x00,0x00}, // ,
	{0x08,0x08,0x08,0x08,0x08}, // -
    {0x00,0x60,0x60,0x00,0x00}, // .
	{0x20,0x10,0x08,0x04,0x02}, // /
    {0x3e,0x51,0x49,0x45,0x3e}, // 0
	{0x00,0x42,0x7f,0x40,0x00}, // 1
    {0x42,0x61,0x51,0x49,0x46}, // 2
	{0x21,0x41,0x45,0x4b,0x31}, // 3
    {0x18,0x14,0x12,0x7f,0x10}, // 4
	{0x27,0x45,0x45,0x45,0x39}, // 5
    {0x3c,0x4a,0x49,0x49,0x30}, // 6
	{0x01,0x71,0x09,0x05,0x03}, // 7
    {0x36,0x49,0x49,0x49,0x36}, // 8
	{0x06,0x49,0x49,0x29,0x1e}, // 9
    {0x00,0x36,0x36,0x00,0x00}, // :
	{0x00,0x56,0x36,0x00,0x00}, // ;
    {0x08,0x14,0x22,0x41,0x00}, // <
	{0x14,0x14,0x14,0x14,0x14}, // =
    {0x00,0x41,0x22,0x14,0x08}, // >
	{0x02,0x01,0x51,0x09,0x06}, // ?
    {0x32,0x49,0x79,0x41,0x3e}, // @
	{0x7e,0x11,0x11,0x11,0x7e}, // A
    {0x7f,0x49,0x49,0x49,0x36}, // B
	{0x3e,0x41,0x41,0x41,0x22}, // C
    {0x7f,0x41,0x41,0x22,0x1c}, // D
	{0x7f,0x49,0x49,0x49,0x41}, // E
    {0x7f,0x09,0x09,0x09,0x01}, // F
	{0x3e,0x41,0x49,0x49,0x7a}, // G
    {0x7f,0x08,0x08,0x08,0x7f}, // H
	{0x00,0x41,0x7f,0x41,0x00}, // I
    {0x20,0x40,0x41,0x3f,0x01}, // J
	{0x7f,0x08,0x14,0x22,0x41}, // K
    {0x7f,0x40,0x40,0x40,0x40}, // L
	{0x7f,0x02,0x0c,0x02,0x7f}, // M
    {0x7f,0x04,0x08,0x10,0x7f}, // N
	{0x3e,0x41,0x41,0x41,0x3e}, // O
    {0x7f,0x09,0x09,0x09,0x06}, // P
	{0x3e,0x41,0x51,0x21,0x5e}, // Q
    {0x7f,0x09,0x19,0x29,0x46}, // R
	{0x46,0x49,0x49,0x49,0x31}, // S
    {0x01,0x01,0x7f,0x01,0x01}, // T
	{0x3f,0x40,0x40,0x40,0x3f}, // U
    {0x1f,0x20,0x40,0x20,0x1f}, // V
	{0x3f,0x40,0x38,0x40,0x3f}, // W
    {0x63,0x14,0x08,0x14,0x63}, // X
	{0x07,0x08,0x70,0x08,0x07}, // Y
    {0x61,0x51,0x49,0x45,0x43}, // Z
	{0x00,0x7f,0x41,0x41,0x00}, // [
    {0x02,0x04,0x08,0x10,0x20}, // \]
	{0x00,0x41,0x41,0x7f,0x00}, // ]
    {0x04,0x02,0x01,0x02,0x04}, // ^
	{0x40,0x40,0x40,0x40,0x40}, // _
    {0x00,0x01,0x02,0x04,0x00}, // `
	{0x20,0x54,0x54,0x54,0x78}, // a
    {0x7f,0x48,0x44,0x44,0x38}, // b
	{0x38,0x44,0x44,0x44,0x00}, // c
    {0x38,0x44,0x44,0x48,0x7f}, // d
	{0x38,0x54,0x54,0x54,0x18}, // e
    {0x08,0x7e,0x09,0x01,0x02}, // f
	{0x0c,0x52,0x52,0x52,0x3e}, // g
    {0x7f,0x08,0x04,0x04,0x78}, // h
	{0x00,0x44,0x7d,0x40,0x00}, // i
    {0x20,0x40,0x44,0x3d,0x00}, // j
	{0x7f,0x10,0x28,0x44,0x00}, // k
    {0x00,0x41,0x7f,0x40,0x00}, // l
	{0x7c,0x04,0x18,0x04,0x78}, // m
    {0x7c,0x08,0x04,0x04,0x78}, // n
	{0x38,0x44,0x44,0x44,0x38}, // o
    {0x7c,0x14,0x14,0x14,0x08}, // p
	{0x08,0x14,0x14,0x18,0x7c}, // q
    {0x7c,0x08,0x04,0x04,0x08}, // r
	{0x48,0x54,0x54,0x54,0x20}, // s
    {0x04,0x3f,0x44,0x40,0x20}, // t
	{0x3c,0x40,0x40,0x20,0x7c}, // u
    {0x1c,0x20,0x40,0x20,0x1c}, // v
	{0x3c,0x40,0x30,0x40,0x3c}, // w
    {0x44,0x28,0x10,0x28,0x44}, // x
	{0x0c,0x50,0x50,0x50,0x3c}, // y
    {0x44,0x64,0x54,0x4c,0x44},// z
	{0x00,0x08,0x36,0x41,0x00}, // {
    {0x00,0x00,0x7f,0x00,0x00}, // |
	{0x00,0x41,0x36,0x08,0x00}, // }
    {0x10,0x08,0x08,0x10,0x08}, // ^
	{0x00,0x00,0x02,0x05,0x02}}; // degree char 127

#endif // ENABLE_FONT_5x7

//************************************************************************************************
//                                     Font Tables  FONT_7x7
//************************************************************************************************

#ifdef DISPLAY_ENABLE_FONT_7x7

const uint8_t font_7x7[96][7] = {
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00}, // ' '
  {0x00, 0x06, 0x5F, 0x5F, 0x06, 0x00, 0x00}, // '!'
  {0x00, 0x07, 0x07, 0x00, 0x07, 0x07, 0x00}, // '"'
  {0x14, 0x7F, 0x7F, 0x14, 0x7F, 0x7F, 0x14}, // '#'
  {0x24, 0x2E, 0x6B, 0x6B, 0x3A, 0x12, 0x00}, // '$'
  {0x46, 0x66, 0x30, 0x18, 0x0C, 0x66, 0x62}, // '%'
  {0x30, 0x7A, 0x4F, 0x5D, 0x37, 0x7A, 0x48}, // '&'
  {0x04, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00}, // '''
  {0x00, 0x1C, 0x3E, 0x63, 0x41, 0x00, 0x00}, // '('
  {0x00, 0x41, 0x63, 0x3E, 0x1C, 0x00, 0x00}, // ')'
  {0x00, 0x2A, 0x3E, 0x1C, 0x1C, 0x3E, 0x2A}, // '*'
  {0x08, 0x08, 0x3E, 0x3E, 0x08, 0x08, 0x00}, // '+'
  {0x00, 0x80, 0xE0, 0x60, 0x00, 0x00, 0x00}, // ','
  {0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x00}, // '-'
  {0x00, 0x00, 0x60, 0x60, 0x00, 0x00, 0x00}, // '.'
  {0x60, 0x30, 0x18, 0x0C, 0x06, 0x03, 0x01}, // '/'
  {0x3E, 0x7F, 0x71, 0x59, 0x4D, 0x7F, 0x3E}, // '0'
  {0x40, 0x42, 0x7F, 0x7F, 0x40, 0x40, 0x00}, // '1'
  {0x62, 0x73, 0x59, 0x49, 0x6F, 0x66, 0x00}, // '2'
  {0x22, 0x63, 0x49, 0x49, 0x7F, 0x36, 0x00}, // '3'
  {0x18, 0x1C, 0x16, 0x53, 0x7F, 0x7F, 0x50}, // '4'
  {0x27, 0x67, 0x45, 0x45, 0x7D, 0x39, 0x00}, // '5'
  {0x3C, 0x7E, 0x4B, 0x49, 0x79, 0x30, 0x00}, // '6'
  {0x03, 0x03, 0x71, 0x79, 0x0F, 0x07, 0x00}, // '7'
  {0x36, 0x7F, 0x49, 0x49, 0x7F, 0x36, 0x00}, // '8'
  {0x06, 0x4F, 0x49, 0x69, 0x3F, 0x1E, 0x00}, // '9'
  {0x00, 0x00, 0x66, 0x66, 0x00, 0x00, 0x00}, // ':'
  {0x00, 0x80, 0xE6, 0x66, 0x00, 0x00, 0x00}, // ';'
  {0x08, 0x1C, 0x36, 0x63, 0x41, 0x00, 0x00}, // '<'
  {0x24, 0x24, 0x24, 0x24, 0x24, 0x24, 0x00}, // '='
  {0x00, 0x41, 0x63, 0x36, 0x1C, 0x08, 0x00}, // '>'
  {0x02, 0x03, 0x51, 0x59, 0x0F, 0x06, 0x00}, // '?'
  {0x3E, 0x7F, 0x41, 0x5D, 0x5D, 0x1F, 0x1E}, // '@'
  {0x7C, 0x7E, 0x13, 0x13, 0x7E, 0x7C, 0x00}, // 'A'
  {0x41, 0x7F, 0x7F, 0x49, 0x49, 0x7F, 0x36}, // 'B'
  {0x1C, 0x3E, 0x63, 0x41, 0x41, 0x63, 0x22}, // 'C'
  {0x41, 0x7F, 0x7F, 0x41, 0x63, 0x3E, 0x1C}, // 'D'
  {0x41, 0x7F, 0x7F, 0x49, 0x5D, 0x41, 0x63}, // 'E'
  {0x41, 0x7F, 0x7F, 0x49, 0x1D, 0x01, 0x03}, // 'F'
  {0x1C, 0x3E, 0x63, 0x41, 0x51, 0x73, 0x72}, // 'G'
  {0x7F, 0x7F, 0x08, 0x08, 0x7F, 0x7F, 0x00}, // 'H'
  {0x00, 0x41, 0x7F, 0x7F, 0x41, 0x00, 0x00}, // 'I'
  {0x30, 0x70, 0x40, 0x41, 0x7F, 0x3F, 0x01}, // 'J'
  {0x41, 0x7F, 0x7F, 0x08, 0x1C, 0x77, 0x63}, // 'K'
  {0x41, 0x7F, 0x7F, 0x41, 0x40, 0x60, 0x70}, // 'L'
  {0x7F, 0x7F, 0x0E, 0x1C, 0x0E, 0x7F, 0x7F}, // 'M'
  {0x7F, 0x7F, 0x06, 0x0C, 0x18, 0x7F, 0x7F}, // 'N'
  {0x1C, 0x3E, 0x63, 0x41, 0x63, 0x3E, 0x1C}, // 'O'
  {0x41, 0x7F, 0x7F, 0x49, 0x09, 0x0F, 0x06}, // 'P'
  {0x1E, 0x3F, 0x21, 0x71, 0x7F, 0x5E, 0x00}, // 'Q'
  {0x41, 0x7F, 0x7F, 0x09, 0x19, 0x7F, 0x66}, // 'R'
  {0x26, 0x6F, 0x4D, 0x59, 0x73, 0x32, 0x00}, // 'S'
  {0x03, 0x41, 0x7F, 0x7F, 0x41, 0x03, 0x00}, // 'T'
  {0x7F, 0x7F, 0x40, 0x40, 0x7F, 0x7F, 0x00}, // 'U'
  {0x1F, 0x3F, 0x60, 0x60, 0x3F, 0x1F, 0x00}, // 'V'
  {0x7F, 0x7F, 0x30, 0x18, 0x30, 0x7F, 0x7F}, // 'W'
  {0x43, 0x67, 0x3C, 0x18, 0x3C, 0x67, 0x43}, // 'X'
  {0x07, 0x4F, 0x78, 0x78, 0x4F, 0x07, 0x00}, // 'Y'
  {0x47, 0x63, 0x71, 0x59, 0x4D, 0x67, 0x73}, // 'Z'
  {0x00, 0x7F, 0x7F, 0x41, 0x41, 0x00, 0x00}, // '['
  {0x01, 0x03, 0x06, 0x0C, 0x18, 0x30, 0x60}, // backslash
  {0x00, 0x41, 0x41, 0x7F, 0x7F, 0x00, 0x00}, // ']'
  {0x08, 0x0C, 0x06, 0x03, 0x06, 0x0C, 0x08}, // '^'
  {0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80}, // '_'
  {0x00, 0x00, 0x03, 0x07, 0x04, 0x00, 0x00}, // '`'
  {0x20, 0x74, 0x54, 0x54, 0x3C, 0x78, 0x40}, // 'a'
  {0x41, 0x7F, 0x3F, 0x48, 0x48, 0x78, 0x30}, // 'b'
  {0x38, 0x7C, 0x44, 0x44, 0x6C, 0x28, 0x00}, // 'c'
  {0x30, 0x78, 0x48, 0x49, 0x3F, 0x7F, 0x40}, // 'd'
  {0x38, 0x7C, 0x54, 0x54, 0x5C, 0x18, 0x00}, // 'e'
  {0x48, 0x7E, 0x7F, 0x49, 0x03, 0x02, 0x00}, // 'f'
  {0x98, 0xBC, 0xA4, 0xA4, 0xF8, 0x7C, 0x04}, // 'g'
  {0x41, 0x7F, 0x7F, 0x08, 0x04, 0x7C, 0x78}, // 'h'
  {0x00, 0x44, 0x7D, 0x7D, 0x40, 0x00, 0x00}, // 'i'
  {0x60, 0xE0, 0x80, 0x80, 0xFD, 0x7D, 0x00}, // 'j'
  {0x41, 0x7F, 0x7F, 0x10, 0x38, 0x6C, 0x44}, // 'k'
  {0x00, 0x41, 0x7F, 0x7F, 0x40, 0x00, 0x00}, // 'l'
  {0x7C, 0x7C, 0x18, 0x38, 0x1C, 0x7C, 0x78}, // 'm'
  {0x7C, 0x7C, 0x04, 0x04, 0x7C, 0x78, 0x00}, // 'n'
  {0x38, 0x7C, 0x44, 0x44, 0x7C, 0x38, 0x00}, // 'o'
  {0x84, 0xFC, 0xF8, 0xA4, 0x24, 0x3C, 0x18}, // 'p'
  {0x18, 0x3C, 0x24, 0xA4, 0xF8, 0xFC, 0x84}, // 'q'
  {0x44, 0x7C, 0x78, 0x4C, 0x04, 0x1C, 0x18}, // 'r'
  {0x48, 0x5C, 0x54, 0x54, 0x74, 0x24, 0x00}, // 's'
  {0x00, 0x04, 0x3E, 0x7F, 0x44, 0x24, 0x00}, // 't'
  {0x3C, 0x7C, 0x40, 0x40, 0x3C, 0x7C, 0x40}, // 'u'
  {0x1C, 0x3C, 0x60, 0x60, 0x3C, 0x1C, 0x00}, // 'v'
  {0x3C, 0x7C, 0x70, 0x38, 0x70, 0x7C, 0x3C}, // 'w'
  {0x44, 0x6C, 0x38, 0x10, 0x38, 0x6C, 0x44}, // 'x'
  {0x9C, 0xBC, 0xA0, 0xA0, 0xFC, 0x7C, 0x00}, // 'y'
  {0x4C, 0x64, 0x74, 0x5C, 0x4C, 0x64, 0x00}, // 'z'
  {0x08, 0x08, 0x3E, 0x77, 0x41, 0x41, 0x00}, // '{'
  {0x00, 0x00, 0x00, 0x77, 0x77, 0x00, 0x00}, // '|'
  {0x41, 0x41, 0x77, 0x3E, 0x08, 0x08, 0x00}, // '}'
  {0x02, 0x03, 0x01, 0x03, 0x02, 0x03, 0x01}, // '~'
  {0x00, 0x00, 0x07, 0x05, 0x07, 0x00, 0x00}}; // degrees char 127

#endif // ENABLE_FONT_7x7


//************************************************************************************************
//                                     Font Tables  FONT_4x5
//************************************************************************************************

#ifdef DISPLAY_ENABLE_FONT_4x5

const uint8_t font_4x5[96][4] = {                
     {0x00, 0x00, 0x00, 0x00}, //space
     {0x00, 0x00, 0x17, 0x00}, // !
     {0x00, 0x03, 0x00, 0x03}, // "
     {0x00, 0x1f, 0x0a, 0x1f}, // #
     {0x00, 0x14, 0x1f, 0x0a}, // $
     {0x00, 0x09, 0x04, 0x12}, // %
     {0x00, 0x1a, 0x15, 0x1e}, // &
     {0x00, 0x03, 0x01, 0x00}, // '
     {0x00, 0x0e, 0x11, 0x00}, // (
     {0x00, 0x11, 0x0e, 0x00}, // )
     {0x00, 0x15, 0x0e, 0x15}, // *
     {0x00, 0x04, 0x0e, 0x04}, // +
     {0x00, 0x10, 0x10, 0x00}, // ,
     {0x00, 0x04, 0x04, 0x04}, // -
     {0x00, 0x00, 0x10, 0x00}, // .
     {0x00, 0x18, 0x04, 0x03}, // /
     {0x00, 0x1e, 0x11, 0x0f}, // 0
     {0x00, 0x12, 0x1f, 0x10}, // 1
     {0x00, 0x19, 0x15, 0x12}, // 2
     {0x00, 0x11, 0x15, 0x0a}, // 3
     {0x00, 0x06, 0x04, 0x1f}, // 4
     {0x00, 0x17, 0x15, 0x09}, // 5
     {0x00, 0x0e, 0x15, 0x08}, // 6
     {0x00, 0x01, 0x1d, 0x07}, // 7
     {0x00, 0x0a, 0x15, 0x0a}, // 8
     {0x00, 0x02, 0x15, 0x0e}, // 9
     {0x00, 0x00, 0x14, 0x00}, // :
     {0x00, 0x10, 0x14, 0x00}, // ;
     {0x00, 0x04, 0x0a, 0x11}, // <
     {0x00, 0x14, 0x14, 0x14}, // =
     {0x00, 0x11, 0x0a, 0x04}, // >
     {0x00, 0x01, 0x15, 0x02}, // ?
     {0x00, 0x1f, 0x11, 0x17}, // @
     {0x00, 0x1e, 0x05, 0x1e}, // A
     {0x00, 0x1f, 0x15, 0x0a}, // B
     {0x00, 0x0e, 0x11, 0x11}, // C
     {0x00, 0x1f, 0x11, 0x0e}, // D
     {0x00, 0x1f, 0x15, 0x11}, // E
     {0x00, 0x1f, 0x05, 0x01}, // F
     {0x00, 0x0e, 0x11, 0x1d}, // G
     {0x00, 0x1f, 0x04, 0x1f}, // H
     {0x00, 0x11, 0x1f, 0x11}, // I
     {0x00, 0x08, 0x10, 0x0f}, // J
     {0x00, 0x1f, 0x04, 0x1b}, // K
     {0x00, 0x1f, 0x10, 0x10}, // L
     {0x00, 0x1f, 0x06, 0x1f}, // M
     {0x00, 0x1f, 0x02, 0x1f}, // N
     {0x00, 0x0e, 0x11, 0x0e}, // O
     {0x00, 0x1f, 0x05, 0x02}, // P
     {0x00, 0x0e, 0x19, 0x1e}, // Q
     {0x00, 0x1f, 0x05, 0x1a}, // R
     {0x00, 0x16, 0x15, 0x0d}, // S
     {0x00, 0x01, 0x1f, 0x01}, // T
     {0x00, 0x1f, 0x10, 0x1f}, // U
     {0x00, 0x0f, 0x10, 0x0f}, // V
     {0x00, 0x1f, 0x0c, 0x1f}, // W
     {0x00, 0x1b, 0x04, 0x1b}, // X
     {0x00, 0x03, 0x1c, 0x03}, // Y
     {0x00, 0x19, 0x15, 0x13}, // Z
     {0x00, 0x1f, 0x11, 0x00}, // [
     {0x00, 0x03, 0x04, 0x18}, //
     {0x00, 0x11, 0x1f, 0x00}, // ]
     {0x00, 0x02, 0x01, 0x02}, // ^
     {0x10, 0x10, 0x10, 0x10}, // _
     {0x00, 0x01, 0x03, 0x00}, // `
     {0x00, 0x18, 0x14, 0x1c}, // a
     {0x00, 0x1f, 0x14, 0x08}, // b
     {0x00, 0x08, 0x14, 0x14}, // c
     {0x00, 0x08, 0x14, 0x1f}, // d
     {0x00, 0x0c, 0x1c, 0x14}, // e
     {0x00, 0x04, 0x1e, 0x05}, // f
     {0x00, 0x0c, 0x04, 0x1c}, // g
     {0x00, 0x1f, 0x04, 0x18}, // h
     {0x00, 0x00, 0x1d, 0x00}, // i
     {0x00, 0x00, 0x1d, 0x00}, // j
     {0x00, 0x1f, 0x08, 0x14}, // k
     {0x00, 0x00, 0x1f, 0x00}, // l
     {0x00, 0x1c, 0x0c, 0x1c}, // m
     {0x00, 0x1c, 0x04, 0x18}, // n
     {0x00, 0x08, 0x14, 0x08}, // o
     {0x00, 0x1c, 0x14, 0x08}, // p
     {0x00, 0x08, 0x14, 0x1c}, // q
     {0x00, 0x1c, 0x04, 0x00}, // r
     {0x00, 0x10, 0x1c, 0x04}, // s
     {0x00, 0x04, 0x1e, 0x14}, // t
     {0x00, 0x1c, 0x10, 0x1c}, // u
     {0x00, 0x0c, 0x10, 0x0c}, // v
     {0x00, 0x1c, 0x18, 0x1c}, // w
     {0x00, 0x14, 0x08, 0x14}, // x
     {0x00, 0x0c, 0x10, 0x0c}, // y
     {0x00, 0x04, 0x1c, 0x10}, // z
     {0x00, 0x04, 0x1f, 0x11}, // {
     {0x00, 0x00, 0x1f, 0x00}, // |
     {0x00, 0x11, 0x1f, 0x04}, // }
     {0x02, 0x01, 0x02, 0x01}, // ~
     {0x00, 0x03, 0x03, 0x00}};  //Char degrees 127

#endif // ENABLE_FONT_4x5


//************************************************************************************************
//                                       Pictures and Logos
//************************************************************************************************

#ifdef DISPLAY_ENABLE_LOGO_1
    

const uint8_t logo_1 [] = {

0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x30, 0xF0, 0xF0, 0xF0, 0xF0, 0x30, 0x30, 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xE0, 0xF0, 0x70, 0x30, 0x30,
0xF0, 0xF0, 0xF0, 0xF0, 0x30, 0x30, 0x70, 0xF0, 0xF0, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0,
0xFC, 0xFF, 0xFF, 0x3F, 0x03, 0x00, 0x00, 0x00, 0x00, 0x80, 0xC0, 0x40, 0x00, 0xE0, 0xF0, 0xF8,
0xFC, 0x3C, 0x0E, 0x06, 0x06, 0x86, 0xEE, 0xFC, 0xFE, 0xFE, 0x3E, 0x06, 0x08, 0x0C, 0xCC, 0xFC,
0xFC, 0xFE, 0x3E, 0x06, 0x00, 0xC0, 0xF8, 0xFE, 0xFE, 0x7E, 0x06, 0x00, 0x00, 0x0C, 0x8C, 0xFC,
0xFC, 0xFE, 0x3E, 0x0E, 0x04, 0x06, 0x1E, 0x1E, 0x0C, 0x00, 0x03, 0x03, 0x00, 0x00, 0xC0, 0xFC,
0xFF, 0xFF, 0x3F, 0x03, 0x00, 0x00, 0x00, 0x00, 0x03, 0x03, 0xE3, 0xF0, 0xF8, 0xFC, 0xFC, 0xDE,
0xC6, 0xE6, 0x7E, 0x7E, 0x7C, 0x3C, 0x00, 0xE0, 0xF0, 0xF8, 0xFC, 0x3C, 0x0E, 0x06, 0x06, 0x1E,
0x3E, 0x3C, 0x38, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x0C, 0x0C, 0x0F,
0x0F, 0x0F, 0x0F, 0x0C, 0x0C, 0x0C, 0x0C, 0x0E, 0x0F, 0x07, 0x01, 0x00, 0x00, 0x03, 0x07, 0x0F,
0x0F, 0x0C, 0x0C, 0x0C, 0x06, 0x07, 0x0F, 0x0F, 0x0F, 0x0D, 0x0C, 0x06, 0x04, 0x02, 0x07, 0x0F,
0x0F, 0x0F, 0x0C, 0x04, 0x03, 0x07, 0x0F, 0x0F, 0x0F, 0x0C, 0x0C, 0x08, 0x00, 0x0C, 0x0F, 0x0F,
0x0F, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x0C, 0x0C, 0x0F, 0x0F,
0x0F, 0x0F, 0x0C, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x07, 0x07, 0x0F, 0x0E, 0x0C,
0x0C, 0x0C, 0x06, 0x06, 0x02, 0x00, 0x00, 0x03, 0x07, 0x07, 0x0F, 0x0E, 0x0C, 0x0C, 0x0C, 0x06,
0x06, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02,
0xFE, 0x02, 0x02, 0x02, 0x02, 0x84, 0x78, 0x00, 0xE0, 0x50, 0x50, 0x60, 0x00, 0x20, 0x50, 0x50,
0x90, 0x00, 0x10, 0xF4, 0x00, 0x60, 0xD0, 0x50, 0x20, 0x20, 0x00, 0x10, 0xF0, 0x20, 0x10, 0xF0,
0x00, 0x00, 0x00, 0x00, 0x10, 0xF0, 0x20, 0x10, 0xF0, 0x20, 0x10, 0xF0, 0x00, 0x00, 0xA0, 0x50,
0x50, 0xF0, 0x00, 0x00, 0xE0, 0x10, 0x10, 0x12, 0xFE, 0x00, 0x00, 0xE0, 0x50, 0x50, 0x60, 0x00,
0x00, 0x00, 0x00, 0xE0, 0x50, 0x50, 0x60, 0x00, 0xA0, 0x50, 0x50, 0xF0, 0x00, 0x00, 0x20, 0x50,
0x50, 0x90, 0x00, 0x10, 0xF0, 0x00, 0xF0, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01,
0x00, 0x00, 0x01, 0x01, 0x01, 0x06, 0x09, 0x09, 0x09, 0x06, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01,
0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01, 0x00, 0x01, 0x01,
0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00,
0x00, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x01,
0x01, 0x00, 0x00, 0x08, 0x04, 0x03, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00,
0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

#endif // ENABLE_LOGO_1



#ifdef DISPLAY_ENABLE_PICTURE_1

//Icon Size 26x32
const uint8_t music_note [] = {
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xC0, 0xC0, 0xC0, 0xE0, 0x60, 0x60, 0x70, 0x30,
0x38, 0x18, 0x18, 0x1C, 0x0C, 0x0E, 0x06, 0xFE, 0xFE, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0xFF, 0xFF, 0xFF, 0x70, 0x30, 0x38, 0x18, 0x18, 0x1C, 0x0C, 0x0C, 0x0E, 0x06, 0x07, 0x03,
0x03, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x20, 0xF8, 0xF8, 0xFC, 0xFC, 0xFC, 0xFC, 0xFF, 0xFF, 0x3F, 0x00, 0x3C,
0x7E, 0x7E, 0xFF, 0xFF, 0x7E, 0x7F, 0x3F, 0x1F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 
};

#endif // GLCD_ENABLE_PICTURE_1


//************************************************************************************************
//                                      Numbers 25 x 32
//************************************************************************************************

#ifdef DISPLAY_ENABLE_NUMBERS

const uint8_t number_0 [] = {
0x00, 0x00, 0x80, 0xE0, 0xF0, 0xF8, 0xFC, 0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0x7E, 0xFE, 0xFE, 0xFE,
0xFC, 0xFC, 0xF8, 0xF8, 0xF0, 0xE0, 0x80, 0x00, 0x00, 0x00, 0xF8, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
0xFF, 0xFF, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
0x00, 0x00, 0x00, 0x3F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x80, 0x00, 0x00, 0x00, 0x00,
0x00, 0x80, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F,
0x1F, 0x3F, 0x3F, 0x7F, 0x7F, 0x7F, 0x7E, 0x7E, 0x7E, 0x7F, 0x7F, 0x3F, 0x3F, 0x3F, 0x1F, 0x0F,
0x07, 0x01, 0x00, 0x00, 
};

const uint8_t number_1 [] = {
0x00, 0x00, 0x00, 0x00, 0x30, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xFC, 0xF8,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x1C, 0x3E, 0x3F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x3F, 0x3F, 0x1E, 0x00,
0x00, 0x00, 0x00, 0x00, 
};

const uint8_t number_2 [] = {
0x00, 0x00, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFC,
0xFC, 0xF8, 0xF8, 0xF0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0x7F, 0xFF, 0xFF, 0xFF, 0xFF,
0xFF, 0x7F, 0x00, 0x00, 0x80, 0xC0, 0xE1, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x3F, 0x0F, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xC1, 0xE1, 0xF8, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0x7F,
0x1F, 0x0F, 0x07, 0xC3, 0xE1, 0xF0, 0xF0, 0xE0, 0xE0, 0x80, 0x00, 0x00, 0x18, 0x3E, 0x3F, 0x7F,
0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F,
0x7F, 0x3F, 0x0F, 0x00, 
};

const uint8_t number_3 [] = {
0x00, 0x00, 0xF8, 0xFC, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE,
0xFC, 0xFC, 0xFC, 0xF8, 0xF0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x07, 0x0F, 0x8F, 0xCF,
0xE7, 0xE1, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xE0, 0xF1, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x1F, 0x00,
0x00, 0x00, 0x00, 0x00, 0x80, 0xE0, 0xE0, 0xE1, 0xE7, 0xE7, 0xC7, 0x87, 0x07, 0x07, 0x07, 0x07,
0x07, 0x8F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0xFC, 0x70, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F,
0x1F, 0x1F, 0x3F, 0x3F, 0x3F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x3F, 0x3F, 0x3F, 0x1F, 0x1F, 0x0F,
0x07, 0x03, 0x00, 0x00, 
};

const uint8_t number_4 [] = {
0x00, 0x00, 0x00, 0x00, 0xC0, 0xF0, 0xFC, 0xFC, 0xFE, 0xFE, 0xFC, 0x7C, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF0, 0xFC, 0xFF, 0xFF, 0xFF, 0xFF,
0xFF, 0xBF, 0x07, 0x01, 0x00, 0x00, 0xFC, 0xFE, 0xFF, 0xFF, 0xFF, 0xFE, 0xFE, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x03, 0x0F, 0x1F, 0x1F, 0x3F, 0x3F, 0x3F, 0x3F, 0x3F, 0x3F, 0x3F, 0x3F, 0xFF,
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x3F, 0x3F, 0x3F, 0x1E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x3C, 0x7E, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F,
0x7F, 0x7E, 0x3C, 0x00, 
};

const uint8_t number_5 [] = {
0x00, 0x00, 0x00, 0xF8, 0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE,
0xFE, 0xFE, 0xFE, 0xFE, 0xFC, 0xFC, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF,
0xFF, 0xFF, 0xFF, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF3, 0xE7, 0xE7, 0xC7, 0x87, 0x03, 0x01,
0x00, 0x00, 0x00, 0x80, 0xE0, 0xE1, 0xF3, 0xF7, 0xE7, 0xC7, 0x87, 0x07, 0x07, 0x07, 0x07, 0x07,
0x07, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x03, 0x07, 0x0F,
0x1F, 0x3F, 0x3F, 0x3F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7F, 0x3F, 0x3F, 0x3F, 0x1F, 0x0F,
0x07, 0x01, 0x00, 0x00, 
};

const uint8_t number_6 [] = {
0x00, 0x00, 0x80, 0xC0, 0xF0, 0xF8, 0xF8, 0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE,
0xFE, 0xFC, 0xFC, 0xF8, 0xF8, 0xF0, 0xC0, 0x00, 0x00, 0x00, 0xF8, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
0xFF, 0xFF, 0xF1, 0xF0, 0xF0, 0xF8, 0xF8, 0xF8, 0xF8, 0xF1, 0xF3, 0xF3, 0xE3, 0xC3, 0x81, 0x00,
0x00, 0x00, 0x00, 0x1F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x87, 0x03, 0x03, 0x03, 0x03,
0x03, 0x87, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0x00, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F,
0x1F, 0x1F, 0x3F, 0x3F, 0x7F, 0x7F, 0x7E, 0x7E, 0x7E, 0x7F, 0x7F, 0x3F, 0x3F, 0x1F, 0x1F, 0x0F,
0x07, 0x01, 0x00, 0x00, 
};

const uint8_t number_7 [] = {
0x00, 0xF8, 0xFC, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE,
0xFE, 0xFE, 0xFE, 0xFE, 0xFC, 0xFC, 0xF8, 0xE0, 0x00, 0x00, 0x07, 0x0F, 0x0F, 0x0F, 0x0F, 0x07,
0x03, 0x00, 0x00, 0x00, 0x80, 0xC0, 0xF0, 0xFD, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x7F, 0x1F, 0x07,
0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC0, 0xE0, 0xF8, 0xFE, 0xFF, 0xFF, 0xFF,
0xFF, 0x7F, 0x1F, 0x0F, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x1C, 0x3F, 0x7F, 0x7F, 0x7F, 0x7F, 0x3F, 0x0F, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 
};

const uint8_t number_8 [] = {
0x00, 0x00, 0x00, 0x00, 0xE0, 0xF0, 0xF8, 0xFC, 0xFC, 0xFE, 0xFE, 0x7E, 0x7E, 0x7E, 0xFE, 0xFE,
0xFC, 0xFC, 0xF8, 0xF0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0F, 0xBF, 0xFF, 0xFF,
0xFF, 0xFF, 0xFF, 0xF8, 0xF0, 0xF0, 0xF0, 0xF8, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xBF, 0x0F, 0x00,
0x00, 0x00, 0x00, 0x78, 0xFE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x87, 0x03, 0x03, 0x03, 0x03,
0x03, 0x87, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFE, 0x70, 0x00, 0x00, 0x00, 0x01, 0x07, 0x0F,
0x1F, 0x3F, 0x3F, 0x7F, 0x7F, 0x7F, 0x7E, 0x7E, 0x7F, 0x7F, 0x7F, 0x3F, 0x3F, 0x3F, 0x1F, 0x0F,
0x07, 0x01, 0x00, 0x00, 
};

const uint8_t number_9 [] = {
0x00, 0x00, 0x80, 0xC0, 0xF0, 0xF8, 0xF8, 0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0x7E, 0xFE, 0xFE, 0xFE,
0xFC, 0xFC, 0xF8, 0xF0, 0xE0, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x1C, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
0xFF, 0xFF, 0xE1, 0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0xE1, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
0x00, 0x00, 0x00, 0x00, 0x00, 0x81, 0xC3, 0xC7, 0xCF, 0x8F, 0x9F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F,
0x0F, 0x8F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x07, 0x0F, 0x1F,
0x1F, 0x3F, 0x3F, 0x7F, 0x7F, 0x7F, 0x7F, 0x7E, 0x7F, 0x7F, 0x7F, 0x3F, 0x3F, 0x1F, 0x1F, 0x0F,
0x03, 0x01, 0x00, 0x00, 
};

#endif   
   

#endif 

