/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 06/10/2017
Last update     : 28/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library contains all the functions that are handy for controlling 			
matrix keypads with either 4x3 or 4x4 formats. 						
 	

*******************************************************************************/
#ifndef LTLIB_KEYPAD_H
#define LTLIB_KEYPAD_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************

#include "LTlib.h"

#ifndef LTLIB_DELAY_H
    #error The Keypad Library requires the "LTlib_delay" library to be included.
#endif

#ifndef LTLIB_IO_MODULE_H
    #error The Keypad Library requires the "module_IO" library to be included.
#endif

//*****************************************
//   Keypad type
//*****************************************
//You need to define one of the supported keypad matrix before you call the header file
// or remove the comment below. By default the KEYPAD_4x4 gets activated.

//#define KEYPAD_4x4
//#define KEYPAD_4x3
//The keypad type must be defined
#if defined (KEYPAD_4x3) || defined (KEYPAD_4x4) 
  
#else
    #define KEYPAD_4x4
#endif

//*****************************************
//   Library constants
//*****************************************

//Once you call the read button you can decide if debaunce is needed or not.
//This is useful depending on the applicaiton.
//Within a polling function you can use the DEBAUNCE, while within an ISR it
//is not reccomended to use it.
#define DEBAUNCE_ON 0x00
#define DEBAUNCE_OFF 0x01

//You must define either define eother PULL_UP_RESISTOR_ON 0x01 or PULL_UP_RESISTOR_OFF 0x00
//If you do not do so PULL_UP_RESISTOR_ON 0x01 will be defined.
#if defined (PULL_UP_RESISTOR_ON) || defined (PULL_UP_RESISTOR_OFF)

#else
    #define PULL_UP_RESISTOR_ON 0x01
#endif 

//The PORT used for the key pad must be named accordingly to the IO_module library.
#ifndef KEYPAD_PORT 
    #define KEYPAD_PORT IO_PORTB
#endif

//*****************************************
//   Keypad layout
//*****************************************
#ifdef KEYPAD_4x3
    #define KEYPAD_COLUMNS 3
    #define KEYPAD_KEYS 12
#endif 

#ifdef KEYPAD_4x4
    #define KEYPAD_COLUMNS 4
    #define KEYPAD_KEYS 16
#endif 

#define KEYPAD_KEY_0 '0'
#define KEYPAD_KEY_1 '1'
#define KEYPAD_KEY_2 '2'
#define KEYPAD_KEY_3 '3'
#define KEYPAD_KEY_4 '4'
#define KEYPAD_KEY_5 '5'
#define KEYPAD_KEY_6 '6'
#define KEYPAD_KEY_7 '7'
#define KEYPAD_KEY_8 '8'
#define KEYPAD_KEY_9 '9'
#define KEYPAD_KEY_STAR '*'
#define KEYPAD_KEY_POUND '#'
#define KEYPAD_KEY_A 'A'
#define KEYPAD_KEY_B 'B'
#define KEYPAD_KEY_C 'C'
#define KEYPAD_KEY_D 'D'
#define KEYPAD_KEY_NOT_PRESSED 0x00

const uint8_t keypad_values[] = {KEYPAD_KEY_0, KEYPAD_KEY_1, KEYPAD_KEY_2, KEYPAD_KEY_3, KEYPAD_KEY_4,
                                       KEYPAD_KEY_5, KEYPAD_KEY_6, KEYPAD_KEY_7, KEYPAD_KEY_8, KEYPAD_KEY_9, KEYPAD_KEY_STAR, KEYPAD_KEY_POUND, 
                                       KEYPAD_KEY_A, KEYPAD_KEY_B, KEYPAD_KEY_C, KEYPAD_KEY_D
                                      };

//*****************************************
//   Keypad raw patterns
//*****************************************
#define KEYPAD_RAW_0 0xEB   //0b11101011
#define KEYPAD_RAW_1 0x77   //0b01110111
#define KEYPAD_RAW_2 0x7B   //0b01111011
#define KEYPAD_RAW_3 0x7D   //0b01111101
#define KEYPAD_RAW_4 0xB7   //0b10110111
#define KEYPAD_RAW_5 0x0BB  //0b10111011
#define KEYPAD_RAW_6 0xBD   //0b10111101
#define KEYPAD_RAW_7 0xD7   //0b11010111
#define KEYPAD_RAW_8 0xDB   //0b11011011
#define KEYPAD_RAW_9 0xDD   //0b11011101
#define KEYPAD_RAW_STAR 0xE7    //0b11100111
#define KEYPAD_RAW_POUND 0xED   //0b11101101
#define KEYPAD_RAW_A 0x7E       //0b01111110
#define KEYPAD_RAW_B 0xBE       //0b10111110
#define KEYPAD_RAW_C 0xDE       //0b11011110
#define KEYPAD_RAW_D 0xEE       //0b11101110

const uint8_t keypad_raw_values[] = {KEYPAD_RAW_0, KEYPAD_RAW_1, KEYPAD_RAW_2, KEYPAD_RAW_3, KEYPAD_RAW_4,
                                       KEYPAD_RAW_5, KEYPAD_RAW_6, KEYPAD_RAW_7, KEYPAD_RAW_8, KEYPAD_RAW_9, KEYPAD_RAW_STAR, KEYPAD_RAW_POUND, 
                                       KEYPAD_RAW_A, KEYPAD_RAW_B, KEYPAD_RAW_C, KEYPAD_RAW_D
                                      };

/**
 * This function initialize the keypad
 *
 * @param none
 *
 * @return pressed button
 *
 * @note In case of 4x3 Keybad, bit 0 is set as input and it is not used.
 * 
 */
void keypad_initialization ( );


/**
 * This function reads a button back from the keypad. The debaunce feature is 
 * a blocking one, so it may not be always suitable for all applications. 
 *
 * @param debaunce It sets if the button value is debaunced or not [value: DEBAUNCE_ON, DEBAUNCE_OFF]
 *
 * @return pressed button as ASCII code [value 0 it means that no button has been pressed].
 *
 * @note If the button is not debaunced it may be affected by spikes. It is not reccomended
 * to use the debaunce option within an ISR, while it is common use to debaunce it within a polling application.
 * While it is not reccomended to debaunce the button within the ISR, you must debaunce it within the 
 * main program.
 * 
 */
uint8_t keypad_read (uint8_t debaunce);

#endif 
