/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.3
 
Created on Date : 4/11/2007
Last update     : 26/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
********************************************************
PURPOSES
********************************************************

This library is supposed to be used for CAN bus  
applications. All the basic functions are included.
Standard and Extended messages are supported

*******************************************************************************/

#ifndef LTLIB_CAN_MODULE_H	
#define LTLIB_CAN_MODULE_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef CAN_LIBRARY_SUPPORTED
    #error (LTlib) The MCU you have selected is not supported by module_CAN library.
#endif


//*****************************************

typedef uint8_t BYTE;


typedef struct{ 
	uint32_t identifier;
	BYTE data[8];
	BYTE type;			//1 = IDE	0=standard
	BYTE length;		// data length
	BYTE RTR;			//Remote flag. 1 it means remote frame

} CANmessage;


//************************************************************************
// constants used for CANOperationMode function 
//************************************************************************

enum CAN_OP_MODE{
	CAN_OP_MODE_NORMAL = 0b00000000,
	CAN_OP_MODE_SLEEP = 0b00100000,
	CAN_OP_MODE_LOOP = 0b01000000,
	CAN_OP_MODE_LISTEN = 0b01100000,
	CAN_OP_MODE_CONFIG = 0b10000000
};

//************************************************************************
// constants used for CANInitialize function 
//************************************************************************

enum CAN_CONFIG_FLAGS {
    CAN_CONFIG_DEFAULT          = 0b11111111,   // 11111111

    CAN_CONFIG_PHSEG2_PRG_BIT   = 0b00000001,
    CAN_CONFIG_PHSEG2_PRG_ON    = 0b11111111,   // XXXXXXX1
    CAN_CONFIG_PHSEG2_PRG_OFF   = 0b11111110,   // XXXXXXX0

    CAN_CONFIG_LINE_FILTER_BIT  = 0b00000010,
    CAN_CONFIG_LINE_FILTER_ON   = 0b11111111,   // XXXXXX1X
    CAN_CONFIG_LINE_FILTER_OFF  = 0b11111101,   // XXXXXX0X

    CAN_CONFIG_SAMPLE_BIT       = 0b00000100,
    CAN_CONFIG_SAMPLE_ONCE      = 0b11111111,   // XXXXX1XX
    CAN_CONFIG_SAMPLE_THRICE    = 0b11111011,   // XXXXX0XX

    CAN_CONFIG_MSG_TYPE_BIT     = 0b00001000,
    CAN_CONFIG_STD_MSG          = 0b11111111,   // XXXX1XXX
    CAN_CONFIG_XTD_MSG          = 0b11110111,   // XXXX0XXX

    CAN_CONFIG_DBL_BUFFER_BIT   = 0b00010000,
    CAN_CONFIG_DBL_BUFFER_ON    = 0b11111111,   // XXX1XXXX
    CAN_CONFIG_DBL_BUFFER_OFF   = 0b11101111,   // XXX0XXXX

    CAN_CONFIG_MSG_BITS         = 0b01100000,
    CAN_CONFIG_ALL_MSG          = 0b11111111,   // X11XXXXX
    CAN_CONFIG_VALID_XTD_MSG    = 0b11011111,   // X10XXXXX
    CAN_CONFIG_VALID_STD_MSG    = 0b10111111,   // X01XXXXX
    CAN_CONFIG_ALL_VALID_MSG    = 0b10011111    // X00XXXXX
};


//************************************************************************
// constants used for CAN_send_message function 
//************************************************************************

 enum CAN_TX_MSG_FLAGS {
	CAN_TX_PRIORITY_MASK= 0b00000011,   // bit mask
    CAN_TX_PRIORITY_0   = 0b11111100,   // XXXXXX00
    CAN_TX_PRIORITY_1   = 0b11111101,   // XXXXXX01
    CAN_TX_PRIORITY_2   = 0b11111110,   // XXXXXX10
    CAN_TX_PRIORITY_3   = 0b11111111,   // XXXXXX11

    CAN_TX_FRAME_MASK   = 0b00001000,   // bit mask
    CAN_TX_STD_FRAME    = 0b11110111,   // XXXX0XXX
    CAN_TX_XTD_FRAME    = 0b11111111,   // XXXX1XXX

	CAN_TX_RTR_MASK     = 0b01000000,    // bit mask
    CAN_REMOTE_TX_FRAME = 0b11111111,	 // X1XXXXXX
    CAN_NORMAL_TX_FRAME = 0b10111111     // X0XXXXXX    
};

//************************************************************************
// constants used for MASK function 
//************************************************************************

enum CAN_MASK {
    CAN_MASK_B1 = 1,
    CAN_MASK_B2 = 2,
};

//************************************************************************
// constants used for Filter function 
//************************************************************************

enum CAN_FILTER {
    CAN_FILTER_B1_F1,
    CAN_FILTER_B1_F2,
    CAN_FILTER_B2_F1,
    CAN_FILTER_B2_F2,
    CAN_FILTER_B2_F3,
    CAN_FILTER_B2_F4
};

//************************************************************************
// constants used for RX errors
//************************************************************************

enum CAN_RX_ERRORS {
    CAN_RX_BUFFER_1_OVFL = 0b00000001,
	CAN_RX_BUFFER_2_OVFL = 0b00000010
};

//************************************************************************
// Functions Prototypes
//************************************************************************

/**
 * This function is used to initialize the module. It should be the first function to be calles.
 *
 * @param propSeg Propagation Segment, PROP_SEG as per CAN specifications [value: 1...8]
 * @param phaseSeg1 PHASE_SEG1 as per CAN specifications [value: 1...8]   
 * @param phaseSeg2 PHASE_SEG2 as per CAN specifications [value: 1...8]  
 * @param SJW Synchronized Jump Width as per CAN specifications [value: 1...4] 
 * @param  BRP Baud Rate Prescaler [value: 0...63] 
 * @param  flags CAN Module configuration [use the CAN_CONFIG_FLAGS values]
 *     
 * @return void
 * 
 * @note
 * Refer to the MCU datasheet and CAN specifications for more details.
 *
 */
void CAN_open (BYTE propSeg, BYTE phaseSeg1, BYTE phaseSeg2, BYTE SJW, BYTE BRP, enum CAN_CONFIG_FLAGS flags);


/**
 * This function disable the CAN module setting it in SLEEP mode.
 *
 * @param void
 *     
 * @return void
 * 
 */
void CAN_close (void);


/**
 * This function writes a message within the TX buffer.
 *
 * @param identifier Message Identifier.
 * @param data Data Byte poiter to an array of BYTE [max. 8 Bytes]   
 * @param dataLength Number of bytes to be sent [1...8]
 * @param flags MEssage configuration (Use the CAN_TX_MSG_FLAGS values)
 *     
 * @return void
 * 
 * @note
 * Refer to the MCU datasheet and CAN specifications for more details.
 *
 */
void CAN_write_message (uint32_t identifier, BYTE *data, BYTE dataLength, enum CAN_TX_MSG_FLAGS flags);


/**
 * This function reads a message from the RX buffer. The data is stored within 
 * a CANmessage structure.
 *
 * @param msg Pointer to a CANmessage structure.
 *     
 * @return error Reprots the error out of the COMSTAT register
 * 
 * @note
 * Refer to the MCU datasheet and CAN specifications for more details.
 *
 */
uint8_t CAN_read_message (CANmessage *msg);


/**
 * This function sets the MASK used to filter the incoming messages.
 *
 * @param mask_number select the mask number [CAN_MASK_B1, CAN_MASK_B2]
 * @param val Mask value.
 * @param type Mask type, either Standard or Extended [CAN_CONFIG_STD_MSG, CAN_CONFIG_XTD_MSG]
 *     
 * @return void
 * 
 * @note
 * Refer to the MCU datasheet and CAN specifications for more details.
 *
 */
void CAN_set_mask (enum CAN_MASK mask_number, uint32_t val, enum CAN_CONFIG_FLAGS type);


/**
 * This function sets filters used to filter the incoming messages.
 *
 * @param filter_number select the filter number [CAN_FILTER_B1_F1, CAN_FILTER_B1_F2,CAN_FILTER_B2_F1...CAN_FILTER_B2_F4]
 * @param val Filter value
 * @param type Filter type, either Standard or Extended [CAN_CONFIG_STD_MSG, CAN_CONFIG_XTD_MSG]
 *     
 * @return void
 * 
 * @note
 * Refer to the MCU datasheet and CAN specifications for more details.
 *
 */
void CAN_set_filter (enum CAN_FILTER filter_number, uint32_t val, enum CAN_CONFIG_FLAGS type);


/**
 * This function changes the CAN module operational mode.
 *
 * @param mode Mode to be set for the CAN module [
 *
 * @return void
 *
 * @Note for the mode you have to use one of the folowing constants: CAN_OP_MODE_NORMAL
 *       CAN_OP_MODE_SLEEP, CAN_OP_MODE_LOOP, CAN_OP_MODE_LISTEN, CAN_OP_MODE_CONFIG
 * 
 */
void CAN_operation_mode (enum CAN_OP_MODE mode);


/**
 * This function let the CAN module discard all the TX messages.
 *
 * @param none
 *
 * @return none
 *
 */
void CAN_abort_messages (void);


/**
 * This function checks if there is any free slot within the TX buffer.
 *
 * @param none
 *
 * @return status 1: One or more message slot are available
 *                0: TX buffer is full.
 *
 */
BYTE CAN_is_TX_ready (void);


/**
 * This function checks if there is any data within the RX buffer.
 *
 * @param none
 *
 * @return status 1: One or more messages are in the buffer
 *                0: RX buffer is empty 
 *
 */
BYTE CAN_is_RX_ready (void);


/**
 * This function returns the TX BUS passive status. The device gets in passive mode
 * once the error counter has exceeded the error limit of 127.
 *
 * @param none
 *
 * @return status 1: CAN Module is passive
 *                0: Normal Operation
 *
 */
BYTE CAN_is_TX_passive (void);


/**
 * This function returns the RX BUS passive status. The device gets in passive mode
 * once the error counter has exceeded the error limit of 127.
 *
 * @param none
 *
 * @return status 1: CAN Module is passive
 *                0: Normal Operation
 *
 */
BYTE CAN_is_RX_passive (void);


/**
 * This function returns TX warning status. It gets ON once the error counter 
 * reaches the value 96.
 * While the warning is ON the CAN modules still operates properly.
 *
 * @param none
 *
 * @return status 1: TX Warning is ON
 *                0: Normal Operation
 *
 */
BYTE CAN_is_TX_warning_ON (void);


/**
 * This function returns RX warning status.It gets ON once the error counter 
 * reaches the value 96.
 * While the warning is ON the CAN modules still operates properly.
 *
 * @param none
 *
 * @return status 1: RX Warning is ON
 *                0: Normal Operation
 *
 */
BYTE CAN_is_RX_warning_ON (void);


/**
 * This function returns the number of TX errors. 
 *
 * @param none
 *
 * @return error Number of errors
 *
 */
BYTE CAN_get_TX_error_count (void);


/**
 * This function returns the number of RX errors. 
 *
 * @param none
 *
 * @return error Number of errors
 *
 */
BYTE CAN_get_RX_error_count (void);


/**
 * This function is used to check if the mdoule is in BUS OFF. The module gets to 
 * BUSS OFF once the error counter reaches 255.
 *
 * @param none
 *
 * @return status 1: BUS is off
 *                0: Normal Operation
 *
 */
BYTE CAN_is_BUS_OFF (void);


#endif