/*******************************************************************************
 
Author : Mauro Laurenti
Version : 1.1
 
Created on Date : 06/11/2018
Last update     : 31/01/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library is written to support reading and writing the internal Flash
 
*******************************************************************************/

#ifndef LTLIB_FLASH_MODULE_H
#define LTLIB_FLASH_MODULE_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef FLASH_LIBRARY_SUPPORTED
    #error (LTlib) The MCU you have selected is not supported by module_FLASH library.
#endif


/**
 * This function reads a block of data from the flash memory as specified within the parameters. 
 *
 * @param memory_address Start address from which read the bytes out.
 *
 * @param block_size Number of bytes to read out [1..64K]
 * 
 * @param buffer Array pointer where to write the data.
 *
 * @return void
 * 
 * @warning: For PIC32 the read functions require the virtual memory address. 
 *
 */
#ifdef COMPILER_XC8
    void FLASH_read_memory (uint32_t memory_address, uint16_t block_size, uint8_t *buffer);
#endif
    
#ifdef COMPILER_XC32
    void FLASH_read_memory (uint32_t memory_address, uint32_t block_size, uint32_t *buffer);
#endif    

    
/**
 * This function writes a block of data to the flash memory as specified within the parameters. 
 *
 * @param memory_address Start address to which write the bytes out
 *
 * @param block_size Number of bytes to write out [u_int16]
 * 
 * @param buffer Array pointer from which read the data.
 *
 * @return error  0 : No error
 *               -1 : An error occured
 *               -2 : Array is too big to be written
 *
 * @warning The block size while it supports up to 64K, must not exceed the write block size of 
 * the specific MCU that is used. 
 * 
 * @warning The bootloaders do not typically use the interrupts.
 *          This function assumes that the interrupts are not enabled.
 *          If not, disable interrupts before starting a write sequence.
 *
 */
#ifdef COMPILER_XC8
    int8_t FLASH_write_memory (uint32_t memory_address, uint16_t block_size, uint8_t *buffer);
#endif
    
#ifdef COMPILER_XC32
    int8_t FLASH_write_memory (uint32_t memory_address, uint16_t block_size, uint32_t *buffer);
#endif   
    
    
#ifdef COMPILER_XC32
/**
 * This function writes a single data word within the pointed address value
 *
 * @param memory_address Start address to which write the bytes out
 * 
 * @param buffer Data value to write.
 *
 * @return error  0 : No error
 *               -1 : An error occured
 *
 * @warning The bootloaders do not typically use the interrupts.
 *          This function assumes that the interrupts are not enabled.
 *          If not, disable interrupts before starting a write sequence.
 */
int8_t FLASH_write_word (uint32_t memory_address, uint32_t data);
#endif        
    

/**
 * This function erased a block of the flash memory as specified by the address. 
 *
 * @param memory_address Start address of the block to erase.
 *
 * @return error  0 : No error
 *               -1 : An error occured
 *
 * @warning Each address should be a multiple of the erase block size 
 * of the specific MCU that is used. 
 * If not, only the block that contains the specified address will be erased.
 *
 */
int8_t FLASH_erase_memory (uint32_t memory_address);


/**
 * This function returns the Device ID as written in the datasheet. 
 *
 * @param void.
 *
 * @return Device ID
 * 
 * @warning: For PIC32 the read functions require the virtual memory address. 
 *
 */
uint32_t FLASH_read_device_ID (void);


/**
 * This function returns the Device silicon revision. 
 *
 * @param void.
 *
 * @return Device revision
 *
 * @note The specific value may vary from device to device, even if they have 
 * the same Device ID. Indeed the silicon may be improved over time. 
 * 
 * @warning: For PIC32 the read functions require the virtual memory address. 
 */
uint32_t FLASH_read_device_revision (void);

#endif
