/*******************************************************************************
 
Autore : Mauro Laurenti
Versione : 1.6
 
Created on Date : 02/10/2010
Last update     : 08/04/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 
*******************************************************************************/

#ifndef LTLIB_I2C_MODULE_H
#define LTLIB_I2C_MODULE_H

#ifdef COMPILER_XC
    #include <xc.h>
#endif

//*****************************************
//   Library Test
//*****************************************
#include "LTlib.h"

#ifndef I2C_LIBRARY_SUPPORTED
    #error (LTlib) The MCU you have selected is not supported by module_I2C library.
#endif


//****************************************************

#define   I2C_SLAVE_7_BIT_ADDRESS   	0x06     	
#define   I2C_SLAVE_10_BIT_ADDRESS  	0x07    	
#define   I2C_MASTER                    0x08     	

//Slewrate should be on for 400Kb/s
#define   I2C_SLEW_ON                   0x00 

//Slewrate should be on for 100Kb/s
#define   I2C_SLEW_OFF                  0x80  	



//****************************************************

/**
 * This function Opens the I2C module enabling SCL and SDA lines. 
 *
 * @param device_type It specifies if the device is a Master or a Slave [I2C_MASTER, I2C_SLAVE_7_BIT_ADDRESS, I2C_SLAVE_10_BIT_ADDRESS] 
 *
 *
 * @return void
 *
 * @Note
 * You must use the constants I2C_MASTER, I2C_SLAVE_7_BIT_ADDRESS, I2C_SLAVE_10_BIT_ADDRESS.
 * By default the slew rate is turned on to support communications up to 400KHz.
 *
 */
void I2C1_open (uint8_t device_type);

#ifdef I2C2_AVAILABLE
    void I2C2_open (uint8_t device_type);
#endif

    
 /**
 * This function Closes the I2C module. 
 *
 * @param none
 *
 * @return void
 *
 */
void I2C1_close (void); 

#ifdef I2C2_AVAILABLE 
    void I2C2_close (void);
#endif
    
    
 /**
 * This function sets the bus baudrate in case the device is a Master device. 
 *
 * @param bus_baud_rate_KHZ Baud Rate expressed in KHz [min: 1, max:400]
 *
 * @return void
 * 
 * @Warning
 * You must not set the baudrate for a slave device. Doing so may change the slave address. 
 *
 */  
    
void I2C1_baud_rate (uint16_t bus_baud_rate_KHZ);

#ifdef I2C2_AVAILABLE 
    void I2C2_baud_rate (uint16_t bus_baud_rate_KHZ);
#endif
    

/**
 * This function sets I2C slave address. 
 *
 * @param device_address Slave Address 
 *
 * @return void
 * 
 *
 */     
void I2C1_set_slave_address (uint16_t device_address);

#ifdef I2C2_AVAILABLE 
    void I2C2_set_slave_address (uint16_t device_address);
#endif
    
    
 /**
 * This macro reads the input buffer.
 *
 * @param void
 *
 * @return Input buffer value
 *
 */  
#ifdef COMPILER_XC8
    #define I2C1_read_byte() (SSPBUF)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_read_byte() (SSP2BUF)
    #endif
#endif

#ifdef COMPILER_XC16
    #define I2C1_read_byte() (I2C1RCV)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_read_byte() (I2C2RCV)
    #endif
#endif    
    
    
#ifdef COMPILER_XC32
    #define I2C1_read_byte() (I2C1RCV)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_read_byte() (I2C2RCV)
    #endif
#endif
    
        
 /**
 * This function writes a byte to the output buffer and performs several error tests.
 *
 * @param byte_to_send Byte to be sent out
 *
 * @return error 0: ACK received for the byte sent out
 *              -1: Bus Conflict occurs  
 *              -2: NACK has been received
 *              -3: Unkown Error
 *
 */    
int8_t I2C1_write_byte (uint8_t byte_to_send);

#ifdef I2C2_AVAILABLE 
    int8_t I2C2_write_byte (uint8_t byte_to_send);
#endif    
    
   
    
 /**
 * This function waits for the bus to be Idle before continuing. 
 *
 * @param void
 *
 * @return void

 */  
#ifdef COMPILER_XC8    
    #define I2C1_wait_bus_IDLE() while((SSPCON2 & 0x1F) | (SSPSTATbits.R_W))

    #ifdef I2C2_AVAILABLE 
        #define I2C2_wait_bus_IDLE() while((SSP2CON2 & 0x1F) | (SSP2STATbits.R_W))
    #endif    
#endif  
    
#ifdef COMPILER_XC16    
    void I2C1_wait_bus_IDLE(void);

    #ifdef I2C2_AVAILABLE 
        void I2C2_wait_bus_IDLE(void);
    #endif    
#endif      
    
    
#ifdef COMPILER_XC32    
    void I2C1_wait_bus_IDLE(void);

    #ifdef I2C2_AVAILABLE 
        void I2C2_wait_bus_IDLE(void);
    #endif    
#endif      
    
    
    
 /**
 * This function releases the start bit and waits until Start bit is out (idle). 
 *
 * @param void
 *
 * @return void
 *
 * @Warning
 * This is a blocking function. 
 */     
void I2C1_start_bit (void);
 
#ifdef I2C2_AVAILABLE 
    void I2C2_start_bit (void);
#endif   
    
    
 /**
 * This function releases the stop bit and waits until Stop bit is out (idle). 
 *
 * @param void
 *
 * @return void
 *
 * @Warning
 * This is a blocking function. 
 */     
void I2C1_stop_bit (void);
 
#ifdef I2C2_AVAILABLE 
    void I2C2_stop_bit (void);
#endif   
    
    
 /**
 * This function resets the write collision bit.
 *
 * @param void
 *
 * @return void

 */     
void I2C1_reset_write_collision_flag (void);
 
#ifdef I2C2_AVAILABLE 
    void I2C2_reset_write_collision_flag (void);
#endif   

    
/**
 * This function write a byte to an external device made of 1 control byte, 1 addres byte and 1 byte data.
 *
 * @param control_byte Control byte of the external device
 *        register_address Address to be written within the external device
 *        data Data to be written within the pointed address                  
 *
 * @return error Communication error
 *               0: No communication error
 *              -1: Bus Collision error 
 *              -2: Not ACK Error
 *              -3: Write Collision
 */     
signed char I2C1_write_byte_to_external_device (uint8_t control_byte, uint8_t register_address, uint8_t data);
 
#ifdef I2C2_AVAILABLE 
    signed char  I2C2_write_byte_to_external_device (uint8_t control_byte, uint8_t register_address, uint8_t data);
#endif   
    
    
/**
 * This function read a byte from an external device made of 1 control byte, 1 addres byte.
 *
 * @param control_byte Control byte of the external device
 *        register_address Address to be written within the external device
 *        data pointer to the variable where the read data will be written into. 
 * 
 * @return error Communication error or data byte
 *               0: No communication error
 *              -1: Bus Collision error 
 *              -2: Not ACK Error
 *              -3: Write Collision               
 *
 * @return void

 */     
int8_t I2C1_read_byte_from_external_device (uint8_t control_byte, uint8_t register_address, uint8_t *data);
 
#ifdef I2C2_AVAILABLE 
    int8_t  I2C2_read_byte_from_external_device (uint8_t control_byte, uint8_t register_address, uint8_t *data);
#endif   
   

/**
 * This function write a word (16bits) to an external device made of 1 control byte, 1 address byte and 2 bytes of data.
 *
 * @param control_byte Control byte of the external device
 *        register_address Address to be written within the external device
 *        data Data to be written within the pointed address                  
 *
 * @return error Communication error
 *               0: No communication error
 *              -1: Bus Collision error 
 *              -2: Not ACK Error
 *              -3: Write Collision
 */     
signed char I2C1_write_word_to_external_device (uint8_t control_byte, uint8_t register_address, uint16_t data);
 
#ifdef I2C2_AVAILABLE 
    signed char  I2C2_write_word_to_external_device (uint8_t control_byte, uint8_t register_address, uint16_t data);
#endif   
    
    
/**
 * This function read a word (16 bits) from an external device made of 1 control byte, 1 addres byte.
 *
 * @param control_byte Control byte of the external device
 *        register_address Address to be written within the external device
 *        data pointer to the variable where the read data will be written into. 
 * 
 * @return error Communication error or data byte
 *               0: No communication error
 *              -1: Bus Collision error 
 *              -2: Not ACK Error
 *              -3: Write Collision               
 *
 * @return void

 */     
int8_t I2C1_read_word_from_external_device (uint8_t control_byte, uint8_t register_address, uint16_t *data);
 
#ifdef I2C2_AVAILABLE 
    int8_t  I2C2_read_word_from_external_device (uint8_t control_byte, uint8_t register_address, uint16_t *data);
#endif    
    
    
/**
 * This function write an array of bytes to an external device made of 1 control byte, 1 address byte.
 *
 * @param control_byte Control byte of the external device
 *        data Data array pointer to be written within the pointed address  
 *        array_size size of the array [1...255]        
 *
 * @return error Communication error
 *               0: No communication error
 *              -1: Bus Collision error 
 *              -2: Not ACK Error
 *              -3: Write Collision
 */     
signed char I2C1_write_array_to_external_device (uint8_t control_byte, uint8_t *data, uint8_t array_size);
 
#ifdef I2C2_AVAILABLE 
    signed char  I2C2_write_array_to_external_device (uint8_t control_byte, uint8_t *data, uint8_t array_size);
#endif       
    
     
 /**
 * This macro checks if there is any data ready in the RX buffer.
 *
 * @param void
 *
 * @return flag It returns data ready flag. 
 *              0: Buffer is empty
 *              1: Buffer is full
 */     
#ifdef COMPILER_XC8
    #define I2C1_check_data_ready() (SSPSTATbits.BF)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_data_ready() (SSP2STATbits.BF)
    #endif    
#endif 

#ifdef COMPILER_XC16
    #define I2C1_check_data_ready() (I2C1STATbits.RBF)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_data_ready() (I2C2STATbits.RBF)
    #endif    
#endif      
    
    
#ifdef COMPILER_XC32
    #define I2C1_check_data_ready() (I2C1STATbits.RBF)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_data_ready() (I2C2STATbits.RBF)
    #endif    
#endif     
  
    
 /**
 * This macro checks if received command is a read or write operation.
 *
 * @param void
 *
 * @return flag It returns RW flag. 
 *              0: Write Command
 *              1: Read Command

 */     
#ifdef COMPILER_XC8
    #define I2C1_check_read_write_operation() (SSPSTATbits.R_W)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_read_write_operation() (SSP2STATbits.R_W)
    #endif      
#endif 

#ifdef COMPILER_XC16
    #define I2C1_check_read_write_operation() (I2C2STATbits.R_W)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_read_write_operation() (SSP2STATbits.R_W)
    #endif      
#endif       
    
    
#ifdef COMPILER_XC32
    #define I2C1_check_read_write_operation() (I2C2STATbits.R_W)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_read_write_operation() (SSP2STATbits.R_W)
    #endif      
#endif     

   
 /**
 * This macro checks if the stop bit has been detected.
 *
 * @param void
 *
 * @return flag It returns Stop bit flag. 
 *              0: Stop bit is not detected
 *              1: Stop bit has been detected

 */   
#ifdef COMPILER_XC8    
    #define I2C1_check_stop_bit() (SSPSTATbits.P)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_stop_bit() (SSP2STATbits.P)
    #endif  
#endif     
  
#ifdef COMPILER_XC16    
    #define I2C1_check_stop_bit() (I2C1STATbits.P)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_stop_bit() (I2C2STATbits.P)
    #endif  
#endif     
    
#ifdef COMPILER_XC32    
    #define I2C1_check_stop_bit() (I2C1STATbits.P)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_stop_bit() (I2C2STATbits.P)
    #endif  
#endif     
    
    
 /**
 * This macro checks if the start bit has been detected.
 *
 * @param void
 *
 * @return flag It returns Start bit flag. 
 *              0: Start bit is not detected
 *              1: Start bit has been detected
 */
#ifdef COMPILER_XC8     
    #define I2C1_check_start_bit() (SSPSTATbits.S)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_start_bit() (SSP2STATbits.S)
    #endif 
#endif

#ifdef COMPILER_XC16     
    #define I2C1_check_start_bit() (I2C1STATbits.S)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_start_bit() (I2C2STATbits.S)
    #endif 
#endif 

    
#ifdef COMPILER_XC32     
    #define I2C1_check_start_bit() (I2C1STATbits.S)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_start_bit() (I2C2STATbits.S)
    #endif 
#endif    
    
    
 /**
 * This macro checks if a bus collision happened.
 *
 * @param void
 *
 * @return flag It returns the bus collision status. 
 *              0: Normal transmission
 *              1: Bus collision has been detected

 */     
#ifdef COMPILER_XC8
    #define I2C1_check_bus_collision() (PIR2bits.BCLIF)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_bus_collision() (PIR3bits.BCL2IF)
    #endif 
#endif

#ifdef COMPILER_XC16
    #define I2C1_check_bus_collision() (I2C1STATbits.BCL)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_bus_collision() (I2C2STATbits.BCL)
    #endif 
#endif  
    
    
#ifdef COMPILER_XC32
    #define I2C1_check_bus_collision() (I2C1STATbits.BCL)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_bus_collision() (I2C2STATbits.BCL)
    #endif 
#endif    

    
 /**
 * This macro checks if the ACK has been received.
 *
 * @param void
 *
 * @return flag It returns the ACK status. 
 *              0: ACK has been received
 *              1: ACK has not been received
 */     
#ifdef COMPILER_XC8
    #define I2C1_check_ACK() (SSPCON2bits.ACKSTAT)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_ACK() (SSP2CON2bits.ACKSTAT)
    #endif 
#endif 

#ifdef COMPILER_XC16
    #define I2C1_check_ACK() (I2C1STATbits.ACKSTAT)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_ACK() (I2C2STATbits.ACKSTAT)
    #endif 
#endif 
    
#ifdef COMPILER_XC32
    #define I2C1_check_ACK() (I2C1STATbits.ACKSTAT)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_check_ACK() (I2C2STATbits.ACKSTAT)
    #endif 
#endif      
    
    
 /**
 * This macro restarts the I2C module
 *
 * @param void
 *
 * @return void
 */     
#ifdef COMPILER_XC8    
    #define I2C1_restart_communication() SSPCON2bits.RSEN=1;while(SSPCON2bits.RSEN)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_restart_communication() SSP2CON2bits.RSEN=1;while(SSP2CON2bits.RSEN)
    #endif       
#endif 

#ifdef COMPILER_XC16    
    #define I2C1_restart_communication() I2C1_CONFIG_REG.RSEN=1;while(I2C1_CONFIG_REG.RSEN)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_restart_communication() I2C2_CONFIG_REG.RSEN=1;while(I2C2_CONFIG_REG.RSEN)
    #endif       
#endif     

    
#ifdef COMPILER_XC32    
    #define I2C1_restart_communication() I2C1CONbits.RSEN=1;while(I2C1CONbits.RSEN)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_restart_communication() I2C2CONbits.RSEN=1;while(I2C2CONbits.RSEN)
    #endif       
#endif     
    

 /**
 * This macro releases an ACK sequence 
 *
 * @param void
 *
 * @return void
 *
 */ 
#ifdef COMPILER_XC8     
    #define I2C1_ACK()  SSPCON2bits.ACKDT=0, SSPCON2bits.ACKEN=1;while(SSPCON2bits.ACKEN)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_ACK()  SSP2CON2bits.ACKDT=0, SSP2CON2bits.ACKEN=1;while(SSP2CON2bits.ACKEN)
    #endif      
#endif  
 
#ifdef COMPILER_XC16     
    void I2C1_ACK(void);

    #ifdef I2C2_AVAILABLE 
        void I2C2_ACK(void);
    #endif      
#endif     
    
#ifdef COMPILER_XC32     
    void I2C1_ACK(void);

    #ifdef I2C2_AVAILABLE 
        void I2C2_ACK(void);
    #endif      
#endif    
    
 /**
 * This macro releases a negative ACK sequence NACK
 *
 * @param void
 *
 * @return void
 *
 */ 
#ifdef COMPILER_XC8     
    #define I2C1_negative_ACK()  SSPCON2bits.ACKDT=1, SSPCON2bits.ACKEN=1;while(SSPCON2bits.ACKEN)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_negative_ACK()  SSP2CON2bits.ACKDT=1, SSP2CON2bits.ACKEN=1;while(SSP2CON2bits.ACKEN)
    #endif      
#endif  
 
#ifdef COMPILER_XC16     
    void I2C1_negative_ACK(void);

    #ifdef I2C2_AVAILABLE 
        void I2C2_negative_ACK(void);
    #endif      
#endif        
        
        
#ifdef COMPILER_XC32     
    void I2C1_negative_ACK(void);

    #ifdef I2C2_AVAILABLE 
        void I2C2_negative_ACK(void);
    #endif      
#endif    
    
    
 /**
 * This macro sets the I2C module as receiver to enable the data read (read sequence).
 *
 * @param void
 *
 * @return void
 *
 */  
#ifdef COMPILER_XC8     
    #define I2C1_set_master_as_receiver()  SSPCON2bits.RCEN = 1;while (SSPCON2bits.RCEN)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_set_master_as_receiver() SSP2CON2bits.RCEN = 1; while (SSP2CON2bits.RCEN)
    #endif    
#endif  

#ifdef COMPILER_XC16     
    #define I2C1_set_master_as_receiver()  I2C1CONLbits.RCEN = 1;while (I2C1CONLbits.RCEN)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_set_master_as_receiver() I2C2CONLbits.RCEN = 1;while (I2C2CONLbits.RCEN)
    #endif    
#endif         
        
#ifdef COMPILER_XC32     
    #define I2C1_set_master_as_receiver()  I2C1CONbits.RCEN = 1;while (I2C1CONbits.RCEN)

    #ifdef I2C2_AVAILABLE 
        #define I2C2_set_master_as_receiver() I2C2CONbits.RCEN = 1;while (I2C2CONbits.RCEN)
    #endif    
#endif     
    
#endif
