/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 01/04/2023
Last update     : 01/04/2023
 
CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "BME280.h"


//************************************************************
//         BME280_initialize 
//************************************************************
void BME280_initialize (uint8_t device_address, uint16_t baud_rate_KHz){

    BME280_I2C_open (I2C_MASTER);
    BME280_I2C_baud_rate (baud_rate_KHz);
    
    BME280_software_reset (device_address);
    BME280_download_calibration_data(device_address);   
        
    BME280_set_humidity_oversampling (device_address, BME280_OVERSAMPLING_8);
    BME280_set_temperature_oversampling (device_address, BME280_OVERSAMPLING_8);
    BME280_set_pressure_oversampling (device_address, BME280_OVERSAMPLING_8);
    
    //As per datasheet, a minimum 2ms must be wait before triggering 
    //the first measure. Some time is passed already during the initialization.
    delay_ms (5);
}


//************************************************************
//         BME280_set_register 
//************************************************************
int8_t BME280_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data){

    uint8_t control_byte = 0;

    control_byte = device_address <<1;

    return (BME280_I2C_write_byte_to_external_device (control_byte,register_add, data));
}


//************************************************************
//           BME280_get_register 
//************************************************************
int8_t BME280_get_register (uint8_t device_address, uint8_t register_address, uint8_t * readback_value){
	uint8_t value = 0x00;
    uint8_t control_byte = 0;
    int8_t error = 0;
    
    control_byte = device_address <<1;

    error = BME280_I2C_read_byte_from_external_device (control_byte, register_address, &value);
    
    *readback_value = value;
            
    return (error);
}


//************************************************************
//           BME280_get_temperature 
//************************************************************
int8_t BME280_get_temperature (uint8_t device_address, int32_t * temperature){
	uint8_t value = 0;
    uint32_t temperature_raw = 0;
    int8_t code_error = 0;
   
    //The data read back from the sensor is the ADC value and is 20 bits unsigned
    code_error = BME280_get_register (device_address, BME280_REG_TEMP_XLSB, &value);
    temperature_raw = ((uint32_t) value ) >> 4;
    
    code_error |= BME280_get_register (device_address, BME280_REG_TEMP_LSB, &value);
    temperature_raw |= ((uint32_t) value) << 4;
    
    code_error |= BME280_get_register (device_address, BME280_REG_TEMP_MSB, &value);
    temperature_raw |= ((uint32_t) value) << 12;
    
    //******************************************************
    // Temperature computation out of the BME280 data sheet
    // Selected algorithm is based on integer values
    // Similar variables naming has been used to easy check.
    // Resolution is 0.01C "5123" equal 51.23C
    //******************************************************
    uint32_t var1 = 0;
    uint32_t var2 = 0;
    
    var1 = ((((temperature_raw >> 3) - ((int32_t) BME280_dig_T1 <<1))) *((int32_t)BME280_dig_T2)) >> 11;
    var2 = (((((temperature_raw >> 4) - ((int32_t) BME280_dig_T1 )) * ((temperature_raw >> 4) - ((int32_t)BME280_dig_T1))) >> 12) * ((int32_t)BME280_dig_T3)) >> 14;
    
    BME280_t_fine =  var1 + var2;
    
    //Divide by 10 to get 0.01 resolution.
    //This is not pointed out by the data sheet so it looks a bug since I get 0.001C
    //resolution if I do not divide by 10
    *temperature = ((BME280_t_fine * 5 + 128) >> 8) / 10;
    
    return (code_error);
}


//************************************************************
//           BME280_get_pressure 
//************************************************************
int8_t BME280_get_humidity (uint8_t device_address, uint32_t * humidity){
    
    uint8_t value = 0;
    uint32_t humidity_raw = 0;
    int8_t code_error = 0;
    uint32_t v_x1 = 0;

   
    //The data read back from the sensor is the ADC value and is 20 bits unsigned
    code_error = BME280_get_register (device_address, BME280_REG_HUM_LSB, &value);
    humidity_raw = ((uint32_t) value );
    
    code_error |= BME280_get_register (device_address, BME280_REG_HUM_MSB, &value);
    humidity_raw |= ((uint32_t) value) << 8;
    
    v_x1 = BME280_t_fine - ((uint32_t) 76800);
    v_x1 = (((((humidity_raw << 14) - (((uint32_t) BME280_dig_H4) << 20) - (((uint32_t) BME280_dig_H5) * v_x1)) + ((uint32_t)16384)) >> 15) \
           * (((((((v_x1 *((uint32_t)BME280_dig_H6)) >> 10) * (((v_x1 * ((uint32_t) BME280_dig_H3)) >> 11) \
           + ((uint32_t) 32768))) >> 10) + ((uint32_t) 2097152)) * ((uint32_t)BME280_dig_H2) + 8192) >> 14)); 
    
    v_x1 = (v_x1 - (((((v_x1 >> 15) * (v_x1 >> 15)) >> 7 ) * ((uint32_t)BME280_dig_H1)) >> 4));

    v_x1 = (v_x1 < 0 ? 0 : v_x1);
    v_x1 = (v_x1 > 419430400 ? 419430400 : v_x1);
        
    *humidity = (uint32_t) (v_x1 >> 12);
    
    return (code_error);
}

//************************************************************
//           BME280_get_pressure 
//************************************************************
int8_t BME280_get_pressure (uint8_t device_address, uint32_t *pressure){
	uint8_t value = 0;
    uint32_t pressure_raw = 0;
    int8_t code_error = 0;
   
    //The data read back from the sensor is the ADC value and is 20 bits unsigned
    code_error = BME280_get_register (device_address, BME280_REG_PRESS_XLSB, &value);
    pressure_raw = ((uint32_t) value ) >> 4;
    
    code_error |= BME280_get_register (device_address, BME280_REG_PRESS_LSB, &value);
    pressure_raw |= ((uint32_t) value) << 4;
    
    code_error |= BME280_get_register (device_address, BME280_REG_PRESS_MSB, &value);
    pressure_raw |= ((uint32_t) value) << 12;
    
    
    //******************************************************
    // Pressure computation out of the BME280 data sheet
    // Selected algorithm is based on integer values
    // Similar variables naming has been used to easy check.
    // Resolution 1 Pa
    //******************************************************
    int64_t var1 = 0;
    int64_t var2 = 0;
    int64_t p = 0;
    
    var1 = ((int64_t)BME280_t_fine) - 128000;
    var2 = var1 * var1 * (int64_t) BME280_dig_P6;
    var2 = var2 + ((var1*(int64_t) BME280_dig_P5) <<17);
    var2 = var2 + (((int64_t) BME280_dig_P4) <<35);
    var1 = ((var1 * var1 * (int64_t) BME280_dig_P3) >>8) + ((var1 * (int64_t) BME280_dig_P2) <<12);
    var1 = (((((int64_t)1)<<47) + var1))*((int64_t) BME280_dig_P1) >>33;
    
    if(var1 == 0){
        // avoid exception caused by division by zero   
        return (0);  
    }
    
    p = 1048576 - pressure_raw;
    p = (((p<<31)- var2) *3125)/var1;
    var1 = (((int64_t) BME280_dig_P9) * (p >> 13) * (p >> 13)) >> 25;
    var2 = (((int64_t) BME280_dig_P8) * p) >> 19;
    p = ((p + var1 + var2) >> 8) + (((int64_t) BME280_dig_P7) <<4);
    
    //The last divide by 256 is made to get Pa instead of Q24.8 format
    *pressure =  (uint32_t) (p >> 8) ;
    
    return (code_error);
}


//************************************************************
//           BME280_get_device_ID 
//************************************************************
int8_t BME280_get_device_ID (uint8_t device_address, uint8_t * device_ID){
   
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    code_error = BME280_get_register (device_address, BME280_REG_ID, &value);
    *device_ID = value;
        
    return (code_error);
}


//************************************************************
//         BME280_get_calibration_data 
//************************************************************
int8_t BME280_download_calibration_data (uint8_t device_address){
       
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    //*******************************
    //Temperature coefficients
    //*******************************
    code_error |= BME280_get_register (device_address, 0x88, &value);
    BME280_dig_T1 = (uint16_t) value;
    code_error |= BME280_get_register (device_address, 0x89, &value);
    BME280_dig_T1 |= ((uint16_t) value) << 8;
        
    code_error |= BME280_get_register (device_address, 0x8A, &value);
    BME280_dig_T2 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x8B, &value);
    BME280_dig_T2 |= ((int16_t) value) << 8;
    
    code_error |= BME280_get_register (device_address, 0x8C, &value);
    BME280_dig_T3 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x8D, &value);
    BME280_dig_T3 |= ((int16_t) value) << 8;    

    //*******************************
    //Pressure coefficients
    //*******************************
    code_error |= BME280_get_register (device_address, 0x8E, &value);
    BME280_dig_P1 = (uint16_t) value;
    code_error |= BME280_get_register (device_address, 0x8F, &value);
    BME280_dig_P1 |= ((uint16_t) value) << 8;

    code_error |= BME280_get_register (device_address, 0x90, &value);
    BME280_dig_P2 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x91, &value);
    BME280_dig_P2 |= ((int16_t) value) << 8;   

    code_error |= BME280_get_register (device_address, 0x92, &value);
    BME280_dig_P3 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x93, &value);
    BME280_dig_P3 |= ((int16_t) value) << 8;   
    
    code_error |= BME280_get_register (device_address, 0x94, &value);
    BME280_dig_P4 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x95, &value);
    BME280_dig_P4 |= ((int16_t) value) << 8;   
    
    code_error |= BME280_get_register (device_address, 0x96, &value);
    BME280_dig_P5 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x97, &value);
    BME280_dig_P5 |= ((int16_t) value) << 8;   

    code_error |= BME280_get_register (device_address, 0x98, &value);
    BME280_dig_P6 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x99, &value);
    BME280_dig_P6 |= ((int16_t) value) << 8; 

    code_error |= BME280_get_register (device_address, 0x9A, &value);
    BME280_dig_P7 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x9B, &value);
    BME280_dig_P7 |= ((int16_t) value) << 8; 

    code_error |= BME280_get_register (device_address, 0x9C, &value);
    BME280_dig_P8 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x9D, &value);
    BME280_dig_P8 |= ((int16_t) value) << 8; 
    
    code_error |= BME280_get_register (device_address, 0x9E, &value);
    BME280_dig_P9 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0x9F, &value);
    BME280_dig_P9 |= ((int16_t) value) << 8; 
     
    //*******************************
    //Humidity coefficients
    //*******************************
    
    code_error |= BME280_get_register (device_address, 0xA1, &value);
    BME280_dig_H1 = (uint8_t) value;
    
    code_error |= BME280_get_register (device_address, 0xE1, &value);
    BME280_dig_H2 = (int16_t) value;
    code_error |= BME280_get_register (device_address, 0xE2, &value);
    BME280_dig_H2 |= ((int16_t) value) << 8; 
    
    code_error |= BME280_get_register (device_address, 0xE3, &value);
    BME280_dig_H3 = (uint8_t) value;

    code_error |= BME280_get_register (device_address, 0xE5, &value);
    BME280_dig_H4 = (int16_t) value;
    BME280_dig_H4 &= 0x0F;
    code_error |= BME280_get_register (device_address, 0xE4, &value);
    BME280_dig_H4 |= ((int16_t) value) << 4;
    
    code_error |= BME280_get_register (device_address, 0xE5, &value);
    BME280_dig_H5 = (int16_t) value >> 4;
    code_error |= BME280_get_register (device_address, 0xE6, &value);
    BME280_dig_H5 |= ((int16_t) value) << 4;
    
    code_error |= BME280_get_register (device_address, 0xE7, &value);
    BME280_dig_H6 = (int8_t) value;
    
    return(code_error); 
}


//************************************************************
//         BME280_software_reset 
//************************************************************
int8_t BME280_software_reset (uint8_t device_address){
    int8_t code_error = 0;
               
    code_error = BME280_set_register (device_address, BME280_REG_RESET, 0xB6);
    
    return(code_error); 
}


//************************************************************
//         BME280_set_temperature_oversampling 
//************************************************************
int8_t BME280_set_temperature_oversampling (uint8_t device_address, uint8_t oversampling){
    
    uint8_t readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = BME280_get_register (device_address, BME280_REG_CTRL_MES, &readback);

    //Reset the oversampling bits associated to the temperature 
    readback = readback & BME280_OVERSAMPLING_TEMP_MASK;
    //Set the new bits values associated to the temperature oversampling
    readback = readback | (oversampling << 5);
    
    code_error |= BME280_set_register (device_address, BME280_REG_CTRL_MES, readback);
        
    return (code_error);
}


//************************************************************
//         BME280_set_pressure_oversampling 
//************************************************************
int8_t BME280_set_pressure_oversampling (uint8_t device_address, uint8_t oversampling){
    uint8_t readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = BME280_get_register (device_address, BME280_REG_CTRL_MES, &readback);

    //Reset the oversampling bits associated to the pressure 
    readback = readback & BME280_OVERSAMPLING_PRESS_MASK;
    //Set the new bits values associated to the pressure oversampling
    readback = readback | (oversampling << 2);
    
    code_error |= BME280_set_register (device_address, BME280_REG_CTRL_MES, readback);
        
    return (code_error);
}


//************************************************************
//         BME280_set_humidity_oversampling 
//************************************************************
int8_t BME280_set_humidity_oversampling (uint8_t device_address, uint8_t oversampling){

    int8_t code_error = 0x00;
       
    code_error = BME280_set_register (device_address, BME280_REG_CTRL_HUM, oversampling);
        
    return (code_error);   
}


//************************************************************
//         BME280_trigger_new_measure 
//************************************************************
int8_t BME280_trigger_new_measure (uint8_t device_address){
        
    return (BME280_set_mode (device_address, BME280_MODE_FORCED));    
}


//************************************************************
//         BME280_set_mode 
//************************************************************
int8_t BME280_set_mode (uint8_t device_address, uint8_t mode){
    uint8_t readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = BME280_get_register (device_address, BME280_REG_CTRL_MES, &readback);

    //Reset the mode bits
    readback = readback & BME280_MODE_MASK;
    //Set the new bits values associated to the mode
    readback = readback | mode;
    
    code_error |= BME280_set_register (device_address, BME280_REG_CTRL_MES, readback);
        
    return (code_error); 
}


//************************************************************
//         BME280_is_conversion_running 
//************************************************************
int8_t BME280_is_conversion_running (uint8_t device_address){
    
    uint8_t readback = 0x00;
   
    BME280_get_register (device_address, BME280_REG_STATUS, &readback);

    //Reset all the bits except the MEASURING one
    readback = readback & BME280_MEASURING_BIT_MASK;
    
    if (readback == 0){
        return (BME280_CONVERSION_COMPLETED);
    } else {
        return (BME280_CONVERSION_RUNNING);
    }
}


//************************************************************
//         BME280_set_time_interval 
//************************************************************
int8_t BME280_set_time_interval (uint8_t device_address, uint8_t time){
    uint8_t readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = BME280_get_register (device_address, BME280_REG_CONFIG, &readback);

    //Reset the mode bits
    readback = readback & BME280_TIME_INTERVAL_MASK;
    //Set the new bits values associated to the mode
    readback = readback | time;
    
    code_error |= BME280_set_register (device_address, BME280_REG_CONFIG, readback);
        
    return (code_error); 
}


//************************************************************
//         BME280_set_time_interval 
//************************************************************
int8_t BME280_set_FIR_coefficients (uint8_t device_address, uint8_t coefficients){
    uint8_t readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = BME280_get_register (device_address, BME280_REG_CONFIG, &readback);

    //Reset the mode bits
    readback = readback & BME280_FIR_COEFFICIENTS_MASK;
    //Set the new bits values associated to the mode
    readback = readback | coefficients;
    
    code_error |= BME280_set_register (device_address, BME280_REG_CONFIG, readback);
        
    return (code_error);  
}

