/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Created on Date : 11/03/2023
Last update     : 11/03/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "GLCD_ST7565.h"
#include "Display_utilities.h"


//*********************************************
//         Private global variables
//*********************************************
uint8_t Display_font_width = 0;
uint8_t Display_font_height = 0;

uint8_t Display_font_color = DISPLAY_FILLING_BLACK;

uint8_t Display_column_pointer = 0;
uint8_t Display_page_pointer = 0;

uint8_t Display_double_buffer = DISPLAY_DOUBLE_BUFFER_OFF;


//*********************************************
//             GLCD_write_command
//*********************************************
void Display_write_command (uint8_t command_to_send) {

	DISPLAY_CS1B = DISPLAY_LOW;
    DISPLAY_A0 = DISPLAY_LOW;
    
    //Nop();    
       
    ST7565_SPI_write_byte (command_to_send);
    
    DISPLAY_CS1B = DISPLAY_HIGH;
    DISPLAY_A0 = DISPLAY_HIGH;
}

//*********************************************
//             GLCD_write_data
//*********************************************
void Display_write_data (uint8_t data_to_send){
    
    DISPLAY_CS1B = DISPLAY_LOW;
    DISPLAY_A0 = DISPLAY_HIGH;
    
    //If the double buffer is ON, the controller is not updated. 
    if (Display_double_buffer == DISPLAY_DOUBLE_BUFFER_OFF){
        ST7565_SPI_write_byte (data_to_send);
    }

    //Update internal memory to keep track of the controller state
    Display_memory [Display_column_pointer] [Display_page_pointer] = data_to_send;
    
    //The column address is auto-incremented within the controller.
    //The pointer must be incremented manually at each data write
    Display_column_pointer++;
   
    DISPLAY_CS1B = DISPLAY_HIGH;
    DISPLAY_A0 = DISPLAY_HIGH;
    
}


//*********************************************
//             GLCD_set_double_buffer_state
//*********************************************
void Display_set_double_buffer_state (uint8_t set_state){
    Display_double_buffer = set_state;
}


//*********************************************
//             GLCD_set_double_buffer_state
//*********************************************
void Display_update_double_buffer (void){
	uint8_t page = 0;
	uint8_t column = 0;
    
    uint8_t Display_column_pointer_backup = Display_column_pointer;
    uint8_t Display_page_pointer_backup = Display_column_pointer;
    
    Display_set_double_buffer_state (DISPLAY_DOUBLE_BUFFER_OFF);
	
	for (page = 0; page < DISPLAY_NUMBER_OF_PAGES; page++) {

            Display_set_column (0);
            Display_set_page (page);

            for (column = 0; column < DISPLAY_X_RESOLUTION; column++) {

                    Display_write_data (Display_memory [column] [page]);
            }
    }
    
    Display_column_pointer = Display_column_pointer_backup;
    Display_column_pointer = Display_page_pointer_backup;
    
    Display_set_double_buffer_state (DISPLAY_DOUBLE_BUFFER_ON);
}


//*********************************************
//             GLCD_set_display
//*********************************************
void Display_set_display (uint8_t display_status){
	
    if (display_status == DISPLAY_TURN_ON){
        Display_write_command (DISPLAY_COMM_DISPLAY_ON_OFF | DISPLAY_ON_STATE);
        Display_write_command (0b10101100); 
        Display_write_command (0b00000000); 
        
        Display_write_command (0b10100100); 
    }
    
    if (display_status == DISPLAY_TURN_OFF){
        Display_write_command (0b10101101); 
        Display_write_command (0b00000000); 
        
        Display_write_command (DISPLAY_COMM_DISPLAY_ON_OFF); 
        Display_write_command (0b10100101); 
    }

}


//*********************************************
//             GLCD_set_page
//*********************************************
void  Display_set_orientation (uint8_t orientation){
    
    if (orientation == DISPLAY_ORIENTATION_NORTH){
        Display_write_command (0b10100000);
        Display_write_command (0b11001000);
    } 
    
    if (orientation == DISPLAY_ORIENTATION_SOUTH){
        Display_write_command (0b10100001);
        Display_write_command (0b11000000);        
    }
       
}


//*********************************************
//             GLCD_set_page
//*********************************************
void Display_set_page (uint8_t controller_page) {
    uint8_t command_to_be_sent = 0x00;
    
    //OR with the command 
    command_to_be_sent |= DISPLAY_COMM_SET_PAGE;
    
    //OR with the selected page 
    command_to_be_sent |= controller_page;
            
	Display_write_command (command_to_be_sent);

    Display_page_pointer = controller_page;
   
}


//*********************************************
//             GLCD_set_column
//*********************************************
void Display_set_column (uint8_t controller_column){
    uint8_t column_4_High_bits = 0x00;
    uint8_t column_4_Low_bits = 0x00;
    
    uint8_t command_to_be_sent = 0x00;
    
    column_4_High_bits = ((controller_column & 0xF0) >> 4);
    column_4_Low_bits = (controller_column & 0x0F);
    
    command_to_be_sent = DISPLAY_COMM_SET_COLUMN_H | column_4_High_bits;
    Display_write_command (command_to_be_sent);
    
    command_to_be_sent = DISPLAY_COMM_SET_COLUMN_L | column_4_Low_bits;
    Display_write_command (command_to_be_sent);
    
    Display_column_pointer = controller_column;
    
}


//*********************************************
//             GLCD_set_start_line
//*********************************************
void Display_set_start_line (uint8_t controller_start_line){

    uint8_t command_to_be_sent = 0x00;
    
    //OR with the command 
    command_to_be_sent |= DISPLAY_COMM_DISPLAY_START_LINE;
    
    //OR with the selected page 
    command_to_be_sent |= controller_start_line;
            
	Display_write_command (command_to_be_sent);
}


//*********************************************
//             GLCD_clear
//*********************************************
void Display_clear (uint8_t filling_color) {
	
	uint8_t page = 0;
	uint8_t column = 0;

	Display_set_column (column);

	for (page = 0; page < DISPLAY_NUMBER_OF_PAGES; page++) {
		
        Display_set_page (page);        
        Display_set_column (0);
        
		for (column = 0; column < DISPLAY_X_RESOLUTION; column++) {

            Display_write_data (filling_color);
            
            if (filling_color == DISPLAY_FILLING_BLACK) {
                Display_memory[Display_column_pointer][Display_page_pointer] = 0xFF;
            } else {
                Display_memory[Display_column_pointer][Display_page_pointer] = 0x00;
            }
		}
	}
}


//*********************************************
//             GLCD_plot_xy
//*********************************************
void Display_plot_xy (uint8_t x_pos,  uint8_t y_pos, uint8_t filling_color){

	uint8_t data_byte;
	uint8_t data_mask;
    
    uint8_t column = x_pos;
    uint8_t page = 0;   

    //Divide by 8 to get the page
    page = y_pos >> 3;
    
    data_byte = Display_memory [column][page];
    
    data_mask = 0x01 << (y_pos & 0x07);	

	if (filling_color == DISPLAY_FILLING_BLACK)
		data_byte = data_byte | data_mask;
	else
		data_byte = data_byte & ~data_mask;
	
    Display_set_column(column);
    Display_set_page(page);
	Display_write_data (data_byte);

}


//*********************************************
//             GLCD_set_font
//*********************************************
void Display_set_font (uint8_t width, uint8_t height){

	Display_font_width = width;
	Display_font_height = height;

}


//*********************************************
//             GLCD_set_font_color
//*********************************************
void Display_set_font_color (uint8_t font_color){
    Display_font_color = font_color;
}


//*********************************************
//             GLCD_backlight
//*********************************************
void Display_backlight(uint8_t backlight) {

	DISPLAY_LED = backlight;
}


//*********************************************
//             GLCD_Reset
//*********************************************
void  Display_Reset (void){
    DISPLAY_RST = DISPLAY_LOW;
    
    delay_ms (1);
    
    DISPLAY_RST = DISPLAY_HIGH;
    
}


//*********************************************
//             GLCD_set_contrast
//*********************************************
void Display_set_contrast (uint8_t contrast_value){
    
    uint8_t contrast_to_set = 0;
    
    //Converter the % value to the 0...63 value)
    contrast_to_set = (uint8_t) ((63 * contrast_value)/100);
    
    Display_write_command (0b00100111);  
    
    Display_write_command (0b10000001);
    Display_write_command (contrast_to_set); 
    
}

//*********************************************
//             GLCD_initialize
//*********************************************
void  Display_initialize (void) {

    //wait for the power to stabilize    
    delay_ms (10);
        
    //Pin initialization
    DISPLAY_CS1B_DIRECTION = PIN_AS_OUTPUT;
    DISPLAY_RST_DIRECTION = PIN_AS_OUTPUT;
    DISPLAY_A0_DIRECTION  = PIN_AS_OUTPUT;
    DISPLAY_LED_DIRECTION = PIN_AS_OUTPUT;
    
    DISPLAY_RST = DISPLAY_HIGH;
    
    Display_Reset ();
    
    ST7565_SPI_baudrate(SPI_CLK_4000_KHZ);
    ST7565_SPI_mode (SPI_MODE_1);
    ST7565_SPI_open (SPI_MASTER_DEVICE);
    
    Display_set_start_line (0);
    Display_set_orientation (DISPLAY_ORIENTATION_SOUTH);
    
#if (DISPLAY_Y_RESOLUTION == 64)
    Display_write_command (0b10100110);
    Display_write_command (0b10100010);
    Display_write_command (0b00101111);
#endif    
    
#if (DISPLAY_Y_RESOLUTION == 32)
    Display_write_command (0b01000000);
    Display_write_command (0b10100001);
    
    Display_write_command (0b11000000);
    Display_write_command (0b10100110);
    Display_write_command (0b10100010);
    
    Display_write_command (0b00101111);
#endif 
    
    Display_write_command (0b11111000);
    Display_write_command (0b00000000);    
    
    Display_set_contrast (30); 
    
    Display_write_command (0b10101100);
    Display_write_command (0b00000000);
    
    Display_write_command (0b10101111);
    Display_column_pointer = 0;
    Display_page_pointer = 0;
    	
	Display_backlight (DISPLAY_TURN_OFF_LED);
    Display_clear (DISPLAY_FILLING_WHITE);
	Display_set_display (DISPLAY_TURN_ON);
	Display_set_font (DISPLAY_FONT_WIDTH_5, DISPLAY_FONT_HEIGTH_7);
    Display_set_font_color (DISPLAY_FILLING_BLACK);

}
