/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Created on Date : 23/10/2019
Last update     : 23/10/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "HDC2080.h"


//************************************************************
//         HDC2080_initialize 
//************************************************************
void HDC2080_initialize (uint8_t device_address, uint16_t baud_rate_KHz){

    HDC2080_I2C_open (I2C_MASTER);
    HDC2080_I2C_baud_rate (baud_rate_KHz);
    
    HDC2080_software_reset (device_address);
    
    HDC2080_heater (device_address, HDC2080_HEATER_OFF);
    
    HDC2080_set_temperature_offset (device_address, 0x00);
    HDC2080_set_humidity_offset (device_address, 0x00);
    
    HDC2080_set_temperature_resolution (device_address, HDC2080_CONF_TEMPERATURE_RES_14_BITS);
    HDC2080_set_humidity_resolution (device_address, HDC2080_CONF_HUMIDITY_RES_14_BITS);
}


//************************************************************
//         HDC2080_set_register 
//************************************************************
int8_t HDC2080_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data){

    uint8_t control_byte = 0;

    control_byte = device_address <<1;

    return (HDC2080_I2C_write_byte_to_external_device (control_byte,register_add,data));
}


//************************************************************
//           HDC2080_get_register 
//************************************************************
int8_t HDC2080_get_register (uint8_t device_address, uint8_t register_address, uint8_t * readback_value){
	uint8_t value = 0x00;
    uint8_t control_byte = 0;
    int8_t error = 0;
    
    control_byte = device_address <<1;

    error = HDC2080_I2C_read_byte_from_external_device (control_byte, register_address, &value);
    
    *readback_value = value;
            
    return (error);
}


//************************************************************
//           HDC2080_get_temperature 
//************************************************************
int8_t HDC2080_get_temperature (uint8_t device_address, float * temperature){
	uint8_t value = 0;
    uint16_t temperature_raw = 0;
    int8_t code_error = 0;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_TEMPERATURE_LOW, &value);
    temperature_raw = value;
    
    code_error |= HDC2080_get_register (device_address, HDC2080_REG_TEMPERATURE_HIGH, &value);
    temperature_raw |= ((uint16_t) value) << 8;
    
    *temperature =  ((float) temperature_raw * 0.002518)  - 40;
    
    return (code_error);
}


//************************************************************
//           HDC2080_get_humidity 
//************************************************************
int8_t HDC2080_get_humidity (uint8_t device_address, float *humidity){
	uint8_t value = 0;
    uint16_t humidity_raw = 0;
    int8_t code_error = 0;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_HUMIDITY_LOW, &value);
    humidity_raw = value;
    
    code_error |= HDC2080_get_register (device_address, HDC2080_REG_HUMIDITY_HIGH, &value);
    humidity_raw |= ((uint16_t) value) << 8;
    
    *humidity = (float) humidity_raw * 0.001526;
    
    return (code_error);
}


//************************************************************
//           HDC2080_get_device_ID 
//************************************************************
int8_t HDC2080_get_device_ID (uint8_t device_address, uint16_t * device_ID){
   
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_DEVICE_ID_LOW, &value);
    *device_ID = value;
    
    code_error |= HDC2080_get_register (device_address, HDC2080_REG_DEVICE_ID_HIGH, &value);
    *device_ID |= ((uint16_t) value) << 8;
    
    return (code_error);
}


//************************************************************
//           HDC2080_get_manufacturer_ID 
//************************************************************
int8_t HDC2080_get_manufacturer_ID (uint8_t device_address, uint16_t * manufacturer_ID){
    
    uint8_t data_readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_MANUFACTURER_ID_LOW, &data_readback);
    *manufacturer_ID = data_readback;
    
    code_error |= HDC2080_get_register (device_address, HDC2080_REG_MANUFACTURER_ID_HIGH, &data_readback);
    *manufacturer_ID |= ((uint16_t) data_readback) << 8;
    
    return (code_error);
}


//************************************************************
//           HDC2080_trigger_new_measure 
//************************************************************
int8_t HDC2080_trigger_new_measure (uint8_t device_address){
    
    uint8_t data_readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_MEASUREMENT_CONF, &data_readback);
    //reset bits
    data_readback &= HDC2080_CONF_MEASUREMENT_TRIGGER_MASK;
    
    //Set the bits
    data_readback |= HDC2080_CONF_MEASUREMENT_TRIGGER;
    
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_MEASUREMENT_CONF, data_readback);
    
    return (code_error);
}


//************************************************************
//           HDC2080_heater 
//************************************************************
int8_t HDC2080_heater (uint8_t device_address, uint8_t setting){
    uint8_t data_readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_RESET_DRDY_INT_CONF, &data_readback);
    //Turn off the heater
    data_readback &= HDC2080_HEATER_MASK;
    
    //Set the heater as needed
    data_readback |= setting;
    
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_RESET_DRDY_INT_CONF, data_readback);
    
    return (code_error);
}


//************************************************************
//           HDC2080_set_temperature_offset implementation
//************************************************************
int8_t HDC2080_set_temperature_offset (uint8_t device_address, uint8_t adjustment){

    int8_t code_error = 0x00;
 
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_TEMP_OFFSET_ADJUST, adjustment);
    
    return (code_error);
}


//************************************************************
//           HDC2080_set_humidity_offset 
//************************************************************
int8_t HDC2080_set_humidity_offset (uint8_t device_address, uint8_t adjustment){
    int8_t code_error = 0x00;
 
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_HUM_OFFSET_ADJUST, adjustment);
    
    return (code_error);
}


//************************************************************
//     HDC2080_set_temperature_resolution 
//************************************************************
int8_t HDC2080_set_temperature_resolution (uint8_t device_address, uint8_t resolution){
    
    uint8_t data_readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_MEASUREMENT_CONF, &data_readback);
    //reset the resolution
    data_readback &= HDC2080_CONF_TEMPERATURE_RES_MASK;
    
    //Set the new resolution
    data_readback |= resolution;
    
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_MEASUREMENT_CONF, data_readback);
    
    return (code_error);
    
}


//************************************************************
//      HDC2080_set_humidity_resolution 
//************************************************************
int8_t HDC2080_set_humidity_resolution (uint8_t device_address, uint8_t resolution){
    
    uint8_t data_readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_MEASUREMENT_CONF, &data_readback);
    //reset the resolution
    data_readback &= HDC2080_CONF_HUMIDITY_RES_MASK;
    
    //Set the new resolution
    data_readback |= resolution;
    
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_MEASUREMENT_CONF, data_readback);
    
    return (code_error);
    
}


//************************************************************
//           HDC2080_set_measurement 
//************************************************************
int8_t HDC2080_set_measurement (uint8_t device_address, uint8_t measurement){
    uint8_t data_readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_MEASUREMENT_CONF, &data_readback);
    //reset the resolution
    data_readback &= HDC2080_CONF_MEASUREMENT_MASK;
    
    //Set the new resolution
    data_readback |= measurement;
    
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_MEASUREMENT_CONF, data_readback);
    
    return (code_error);
}


//************************************************************
//           HDC2080_cleaning_cycle 
//************************************************************
int8_t HDC2080_cleaning_cycle (uint8_t device_address, uint8_t cycles){
    
    int8_t code_error = 0x00;
    uint8_t counter = 0;
    
    for (counter = 0; counter < cycles; counter++){
        code_error = HDC2080_heater (device_address, HDC2080_HEATER_ON);
        delay_ms (10);
        code_error |= HDC2080_heater (device_address, HDC2080_HEATER_OFF);
    }

    return (code_error);    
}
    

//************************************************************
//         HDC2080_set_humidity_limit 
//************************************************************
int8_t HDC2080_set_humidity_limit (uint8_t device_address, uint8_t limit_type, float humidity){
    int8_t code_error = 0;
    uint8_t int_humidity = 0;
    
    int_humidity = (uint8_t) (humidity * 2.56); 
            
    code_error = HDC2080_set_register (device_address, limit_type, int_humidity );
    
    return(code_error);
}


//************************************************************
//         HDC2080_set_temperature_limit 
//************************************************************
int8_t HDC2080_set_temperature_limit (uint8_t device_address,  uint8_t limit_type, float temperature){
    int8_t code_error = 0;
    uint8_t int_temperature = 0;
    
    int_temperature = (uint8_t) ((temperature + 40.0) * 1.55); 
            
    code_error = HDC2080_set_register (device_address, limit_type, int_temperature);
    
    return(code_error);
}


//************************************************************
//         HDC2080_set_interrutp_enable 
//************************************************************
int8_t HDC2080_set_interrupt_enable (uint8_t device_address, uint8_t interrupt_enable){
    int8_t code_error = 0;
               
    code_error = HDC2080_set_register (device_address, HDC2080_REG_INTERRUPT_ENABLE, interrupt_enable);
    
    return(code_error);
}


//************************************************************
//         HDC2080_set_interrupt_modes 
//************************************************************
int8_t HDC2080_set_interrupt_modes (uint8_t device_address, uint8_t interrupt_modes){
    
    uint8_t data_readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_RESET_DRDY_INT_CONF, &data_readback);
    //reset the bits
    data_readback &= HDC2080_INT_MODES_MASK;
    
    //Set the new bits
    data_readback |= interrupt_modes;
    
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_RESET_DRDY_INT_CONF, data_readback);
    
    return (code_error);
    
}


//************************************************************
//         HDC2080_set_interrupt_modes 
//************************************************************
int8_t HDC2080_set_sampling_modes (uint8_t device_address, uint8_t sampling_modes){
 
    uint8_t data_readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_RESET_DRDY_INT_CONF, &data_readback);
    //reset the bits
    data_readback &= HDC2080_SAMPLING_MODES_MASK;
    
    //Set the new bits
    data_readback |= sampling_modes;
    
    code_error |= HDC2080_set_register (device_address, HDC2080_REG_RESET_DRDY_INT_CONF, data_readback);
    
    return (code_error);
    
}

//************************************************************
//         HDC2080_get_interrupt_status 
//************************************************************
int8_t HDC2080_get_interrupt_status (uint8_t device_address, uint8_t * interrupt_status){
    int8_t code_error = 0x00;
   
    code_error = HDC2080_get_register (device_address, HDC2080_REG_INTERRUPT_DRDY, interrupt_status);
    
    return(code_error);
}


//************************************************************
//         HDC2080_software_reset 
//************************************************************
int8_t HDC2080_software_reset (uint8_t device_address){
    int8_t code_error = 0;
               
    code_error = HDC2080_set_register (device_address, HDC2080_REG_RESET_DRDY_INT_CONF, 0x80);
    
    return(code_error); 
}
