/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 21/10/2017
Last update     : 27/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library supports the HMC5883L magnetic sensor from Honeywell

*******************************************************************************/
									
#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "HMC5883.h"


//************************************************************
//           HMC5883_initialize  Implementation
//************************************************************
void HMC5883_initialize (uint8_t bit_rate_KHZ){
        
    HMC5883_I2C_open (I2C_MASTER);
    HMC5883_I2C_bit_rate (bit_rate_KHZ);   
        
    HMC5883_set_average (HMC5883_AVERAGE_1);
    HMC5883_set_measurement_mode (HMC5883_MODE_CONTINUOUS);
    HMC5883_set_sampling_rate (HMC5883_SAMPLING_15);
    HMC5883_set_gain (HMC5883_GAIN_1090);
    
    //wait for the analog circuitry settling time
    delay_ms(100);
}


//************************************************************
//        HMC5883_read_magnetic_field  Implementation
//************************************************************
int16_t HMC5883_read_magnetic_field (uint8_t field_axis){
    
    uint8_t MSB_register = 0x00;
    uint8_t LSB_register = 0x00;
    
    uint8_t MSB_value = 0x00;
    uint8_t LSB_value = 0x00;
    
    int16_t field_value = 0x0000;
    
    if (field_axis == HMC5883_FIELD_X){
        MSB_register = HMC5883_DATA_OUT_X_MSB;
        LSB_register = HMC5883_DATA_OUT_X_LSB;  
    }
    
    if (field_axis == HMC5883_FIELD_Y){
        MSB_register = HMC5883_DATA_OUT_Y_MSB;
        LSB_register = HMC5883_DATA_OUT_Y_LSB;     
    }
    
    if (field_axis == HMC5883_FIELD_Z){
        MSB_register = HMC5883_DATA_OUT_Z_MSB;
        LSB_register = HMC5883_DATA_OUT_Z_LSB;     
    }
    
    MSB_value = HMC5883_read_register (MSB_register);
    LSB_value = HMC5883_read_register (LSB_register);
    
    field_value = (int16_t)(((uint16_t) MSB_value) << 8 ) | ((uint16_t) LSB_value);
    
    field_value = (int16_t)((field_value ) * HMC5883_resolution_table[HMC5883_gain_value]);
    
    return (field_value);
}


//************************************************************
//           HMC5883_initialize  Implementation
//************************************************************
uint8_t HMC5883_read_register (uint8_t register_address){
    
    uint8_t data = 0;
    
    HMC5883_I2C_read_byte_from_external_device (HMC5883_ADDRESS, register_address, &data);
    
    return (data);
}


//************************************************************
//           HMC5883_write_register  Implementation
//************************************************************
int8_t HMC5883_write_register (uint8_t register_address, uint8_t register_value){
    
    return (HMC5883_I2C_write_byte_to_external_device (HMC5883_ADDRESS,register_address,register_value));
}


//************************************************************
//           HMC5883_is_data_ready  Implementation
//************************************************************
uint8_t HMC5883_is_data_ready (void){
    uint8_t data = 0;
    uint8_t value = HMC5883_NOT_CONNECTED;
    
    data = HMC5883_read_register (HMC5883_STATUS_REGISTER);
    
    if ((data & HMC5883_DATA_READY_BIT_MASK) == 0x01){
        value = HMC5883_DATA_READY;
    } else {
        value = HMC5883_DATA_NOT_READY;
    }
    
    return (value);
}


//************************************************************
//         HMC5883_is_device_connected  Implementation
//************************************************************
uint8_t HMC5883_is_device_connected (void){
    uint8_t data = 0;
    uint8_t value = HMC5883_NOT_CONNECTED;
    
    data = HMC5883_read_register (HMC5883_ID_REGISTER_A);
    
    if (data == ID_REG_A_VALUE){
        value = HMC5883_CONNECTED;
    } else {
        value = HMC5883_NOT_CONNECTED;
    }
    
    return (value);
}


//************************************************************
//         HMC5883_set_sampling_rate  Implementation
//************************************************************
void HMC5883_set_sampling_rate (uint8_t sampling_rate){
    
    uint8_t data = 0;
    
    //value readback before the change
    data = HMC5883_read_register (HMC5883_CONFIGURATION_REG_A);
    
    //reset the average bits
    data &= HMC5883_SAMPLING_AND_MASK;
    
    //set new value
    data |= sampling_rate;
    
    HMC5883_write_register (HMC5883_CONFIGURATION_REG_A, data);
}


//************************************************************
//         HMC5883_set_gain  Implementation
//************************************************************
void HMC5883_set_gain (uint8_t amplifier_gain){
    
    HMC5883_write_register (HMC5883_CONFIGURATION_REG_B, amplifier_gain);
    HMC5883_gain_value = amplifier_gain >> 5;
}


//************************************************************
//         HMC5883_set_measurement_mode  Implementation
//************************************************************
void HMC5883_set_measurement_mode (uint8_t mode){
    HMC5883_write_register (HMC5883_MODE_REGISTER, mode);
}


//************************************************************
//         HMC5883_set_average  Implementation
//************************************************************
void HMC5883_set_average (uint8_t averaging_samples){
    uint8_t data = 0;
    
    //value readback before the change
    data = HMC5883_read_register (HMC5883_CONFIGURATION_REG_A);
    
    //reset the average bits
    data &= HMC5883_AVERAGE_AND_MASK;
    
    //set new value
    data |= averaging_samples;
    
    HMC5883_write_register (HMC5883_CONFIGURATION_REG_A, data);    
}
