/*******************************************************************************

Author : Mauro Laurenti
Version : 1.6
 
Created on Date : 04/09/2006
Last update     : 09/02/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


********************************************************
PURPOSES
********************************************************

These functions allow the user to read/write inside the eeprom
that requires 2 address bytes such as the 24LC512.

********************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "I2C_EEPROM.h"

//************************************************************
//         I2C_EEPROM_initialize function implementation
//************************************************************
void I2C_EEPROM_initialize (uint16_t baud_rate_KHz){

    EEPROM_I2C_open (I2C_MASTER);
    EEPROM_I2C_baud_rate (baud_rate_KHz);
}



//************************************************************
//       I2C_EEPROM_write_byte function implementation
//************************************************************

int8_t I2C_EEPROM_write_byte (uint8_t control, uint16_t address, uint8_t data ){
	
	uint8_t addressH;
	uint8_t addressL;	
	
	addressH = (uint8_t) ((address & 0xFF00) >> 8);
	addressL = (uint8_t) (address & 0x00FF);
	
//*****************************
// Start Condition and control
// Byte are sent
//*****************************		

	EEPROM_I2C_wait_bus_IDLE()
    ;                       
	EEPROM_I2C_start_bit();                     
	
	// Check for bus collision
  	if (EEPROM_I2C_check_bus_collision()) {
        return (-1);                 
    }

	// Write control byte - R/W bit should be 0
	if (EEPROM_I2C_write_byte(control)){
        // Return with write Collision error		
        return (-3);             
    }
     
//*****************************
// Address High is sent
//*****************************	
	EEPROM_I2C_wait_bus_IDLE()
    ; 
              
	if (!EEPROM_I2C_check_ACK()){ 
		
        if (EEPROM_I2C_write_byte(addressH)){
            // Return with write Collision error 
            return (-3);           
      }
 	} else {
 		// Return with Not Ack error condition
        return (-2);               
    }
 
//*****************************
// Address Low is sent
//*****************************

	EEPROM_I2C_wait_bus_IDLE()
    ;   
	
	// Check if ACK condition has been received               
	if (!EEPROM_I2C_check_ACK()) {
		
		// Write address low to the EEPROM
		if (EEPROM_I2C_write_byte(addressL)) {
			// Return with write Collision error 
            return (-3);          
        }
    } else {
        // Return with Not Ack error condition
     	return (-2);            
    }

//*****************************
// Data Byte is sent
//*****************************   

    EEPROM_I2C_wait_bus_IDLE()
    ;  
       
    if (!EEPROM_I2C_check_ACK()){
	    
        if (EEPROM_I2C_write_byte(data)) {
            // Return with write Collision error 
            return (-3);       
        }
    } else {
      	// Return with Not Ack error condition
        return (-2);            
    }
    
//*****************************
// Stop command is sent
//*****************************
  
	EEPROM_I2C_wait_bus_IDLE()
    ;  
                    
  	if (!EEPROM_I2C_check_ACK()) {
	 	
	EEPROM_I2C_stop_bit();
    
    } else {
  		// Return with Not Ack error condition
  		return (-2);           
   }

    if (EEPROM_I2C_check_bus_collision()){
        return (-1);                
    }
  
  // Return with no error
  return (0);                   
}


//************************************************************
//         I2C_EEPROM_write_byte_check function implementation
//************************************************************

int8_t I2C_EEPROM_write_byte_check (uint8_t control, uint16_t address, uint8_t data ) {
	
	// Store the returned error from the read and write function call
	int8_t error;
	
	// Store the data read back from the EEPROM
	uint8_t dataReadback;
	
	error = I2C_EEPROM_write_byte(control, address, data);

	if (error < 0)
		return (error);
		
	// wait for the write process to be completed
	delay_ms (6);
	 
	error = I2C_EEPROM_read_byte (control, address, &dataReadback);
	
	if (error < 0)
		return (error);
	
	if (dataReadback == data)
		return (1);
	else 
		return(-4);
}


//************************************************************
//       I2C_EEPROM_read_byte function implementation
//************************************************************

int8_t I2C_EEPROM_read_byte (uint8_t control, uint16_t address, uint8_t *data) {
	
	uint8_t addressH;
	uint8_t addressL;	
	
	addressH = (uint8_t) ((address & 0xFF00) >> 8);
	addressL = (uint8_t) (address & 0x00FF);


//*****************************
// Start Condition and control
// Byte are sent
//*****************************


	EEPROM_I2C_wait_bus_IDLE()
    ;    
               
	EEPROM_I2C_start_bit(); 
 
	if (EEPROM_I2C_check_bus_collision()) {
		return (-1);               
	}
	
	if (EEPROM_I2C_write_byte(control)){
		// Return with write collision error
        return (-3);             
	} 

//*****************************
// Address High is sent
//*****************************	

	EEPROM_I2C_wait_bus_IDLE()
    ;                    
	
	// Check if ACK condition has been received
	if (!EEPROM_I2C_check_ACK()) {
		
      if (EEPROM_I2C_write_byte(addressH)){
	     	// Return with write collision error
			return (-3);            
      }
    }else {
    	// Return with Not Ack error condition
		return (-2);          
    }

//*****************************
// Address Low is sent
//*****************************	
	EEPROM_I2C_wait_bus_IDLE()
    ; 
	
	// Check if ACK condition has been received                   
	if (!EEPROM_I2C_check_ACK()) {

        if (EEPROM_I2C_write_byte(addressL)) {
          // Return with write collision error 
            return (-3);            
        }
    }else {
        // Return with Not Ack error condition
		return (-2);          
    }


//*****************************
// Restart condition
//*****************************	        
	EEPROM_I2C_wait_bus_IDLE()
    ; 
              
	if (!EEPROM_I2C_check_ACK()){
		
		EEPROM_I2C_restart_communication();
 
		if (EEPROM_I2C_check_bus_collision()) {
            return (-1);          
        }
      
        // Write Control byte - This time R/W bit should be 1                    
		if (EEPROM_I2C_write_byte(control+1)) {
			// Return with write collision error
         return (-3);          
        }

//*****************************
// Data is Read
//*****************************

		EEPROM_I2C_wait_bus_IDLE()
        ;               
       
        // Check if ACK condition has been received  
		if (!EEPROM_I2C_check_ACK()){
			
            EEPROM_I2C_set_master_as_receiver()

            EEPROM_I2C_negative_ACK(); 

            EEPROM_I2C_stop_bit(); 

             // Check if Bus collision happened 
            if (EEPROM_I2C_check_bus_collision()) {
                return (-1);        
            }

		} else {
            // Return with Not Ack error
            EEPROM_I2C_stop_bit(); 
            return (-2);          
		}

	} else {
		// Return with Not Ack error
        EEPROM_I2C_stop_bit(); 
		return (-2);            
     }   
	
	// Data is read from the buffer	
	*data = EEPROM_I2C_read_byte();
 
	// No error occurred  
	return (0);
}
