/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 01/12/2019
Last update     : 01/12/2019
 
CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "L3GD20H.h"


//************************************************************
//         L3GD20H_initialize 
//************************************************************
void L3GD20H_initialize (uint8_t device_address, uint16_t baud_rate_KHz){

    L3GD20H_I2C_open (I2C_MASTER);
    L3GD20H_I2C_baud_rate (baud_rate_KHz);
    
    L3GD20H_software_reset (device_address);
    L3GD20H_set_mode (device_address, L3GD20H_MODE_NORMAL);
    L3GD20H_set_full_scale (device_address, L3GD20H_FULL_SCALE_2000);
        
    //As per datasheet, a typical time of 50ms must be wait before triggering 
    //the first measure. Some time is passed already during the initialization.
    delay_ms (55);
}


//************************************************************
//         L3GD20H_set_register 
//************************************************************
int8_t L3GD20H_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data){

    uint8_t control_byte = 0;

    control_byte = device_address <<1;

    return (L3GD20H_I2C_write_byte_to_external_device (control_byte,register_add, data));
}


//************************************************************
//           L3GD20H_get_register 
//************************************************************
int8_t L3GD20H_get_register (uint8_t device_address, uint8_t register_address, uint8_t * readback_value){
	uint8_t value = 0x00;
    uint8_t control_byte = 0;
    int8_t error = 0;
    
    control_byte = device_address <<1;

    error = L3GD20H_I2C_read_byte_from_external_device (control_byte, register_address, &value);
    
    *readback_value = value;
            
    return (error);
}


//************************************************************
//           L3GD20H_update_register 
//************************************************************
int8_t L3GD20H_update_register (uint8_t device_address, uint8_t register_address, uint8_t register_mask, uint8_t new_value){
    uint8_t readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = L3GD20H_get_register (device_address, register_address, &readback);

    //Reset the register with the mask
    readback = readback & register_mask;
    //Set the new bits values 
    readback = readback | new_value;
    //Update the register 
    code_error |= L3GD20H_set_register (device_address, register_address, readback);
        
    return (code_error); 
}

//************************************************************
//           L3GD20H_get_temperature 
//************************************************************
int8_t L3GD20H_get_temperature (uint8_t device_address, int8_t * temperature){
    uint8_t temperature_raw = 0;
    int8_t code_error = 0;
   
    //The data read back from the sensor is the ADC value and is 20 bits unsigned
    code_error = L3GD20H_get_register (device_address, L3GD20H_REG_OUT_TEMP, &temperature_raw);

    *temperature = ((int8_t)temperature_raw) * (-1);
    
    return (code_error);
}


//************************************************************
//           L3GD20H_get_angular_speed 
//************************************************************
int8_t L3GD20H_get_angular_speed (uint8_t device_address, int16_t *dps_X, int16_t *dps_Y, int16_t *dps_Z){
    int8_t code_error = 0;
    float dps_X_readback = 0;
    float dps_Y_readback = 0;
    float dps_Z_readback = 0;
    
    *dps_X = 0;
    *dps_Y = 0;
    *dps_Z = 0;
    
    code_error = L3GD20H_get_angular_speed_float (device_address, &dps_X_readback, &dps_Y_readback, &dps_Z_readback);
    
    *dps_X = (int16_t) dps_X_readback;
    *dps_Y = (int16_t) dps_Y_readback;
    *dps_Z = (int16_t) dps_Z_readback;
             
    return (code_error);
}


//************************************************************
//           L3GD20H_get_angular_speed_float 
//************************************************************
int8_t L3GD20H_get_angular_speed_float (uint8_t device_address, float *dps_X, float *dps_Y, float *dps_Z){
    int8_t code_error = 0;
    uint8_t value = 0;
    uint16_t dps_X_readback = 0;
    uint16_t dps_Y_readback = 0;
    uint16_t dps_Z_readback = 0;
    
    *dps_X = 0;
    *dps_Y = 0;
    *dps_Z = 0;
    
    //***************************
    // X read out
    //***************************
    code_error = L3GD20H_get_register (device_address, L3GD20H_REG_OUT_X_L, &value);
    dps_X_readback = (uint16_t) value;
    code_error |= L3GD20H_get_register (device_address, L3GD20H_REG_OUT_X_H, &value);
    dps_X_readback |= (((uint16_t) value) << 8);             
    
    //***************************
    // Y read out
    //***************************
    code_error |= L3GD20H_get_register (device_address, L3GD20H_REG_OUT_Y_L, &value);
    dps_Y_readback = (uint16_t) value;
    code_error |= L3GD20H_get_register (device_address, L3GD20H_REG_OUT_Y_H, &value);
    dps_Y_readback |= (((uint16_t) value) << 8);
    
    //***************************
    // Z read out
    //***************************
    code_error |= L3GD20H_get_register (device_address, L3GD20H_REG_OUT_Z_L, &value);
    dps_Z_readback = (uint16_t) value;
    code_error |= L3GD20H_get_register (device_address, L3GD20H_REG_OUT_Z_H, &value);
    dps_Z_readback |= (((uint16_t) value) << 8);
    
    *dps_X = (float)(((int16_t) dps_X_readback) * L3GD20H_active_full_scale)/1000.0;  
    *dps_Y = (float)(((int16_t) dps_Y_readback) * L3GD20H_active_full_scale)/1000.0;
    *dps_Z = (float)(((int16_t) dps_Z_readback) * L3GD20H_active_full_scale)/1000.0;  
                
    return (code_error);    
}


//************************************************************
//           L3GD20H_get_device_ID 
//************************************************************
int8_t L3GD20H_get_device_ID (uint8_t device_address, uint8_t * device_ID){
   
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    code_error = L3GD20H_get_register (device_address, L3GD20H_REG_WHO_AM_I, &value);
    *device_ID = value;
        
    return (code_error);
}


//************************************************************
//         L3GD20H_software_reset 
//************************************************************
int8_t L3GD20H_software_reset (uint8_t device_address){
    
    int8_t code_error = 0x00;
  
    code_error = L3GD20H_update_register (device_address, L3GD20H_REG_LOW_ODR, L3GD20H_RESET_MASK, L3GD20H_SOFTWARE_RESET);
    
    return (code_error);  
}


//************************************************************
//         L3GD20H_set_mode 
//************************************************************
int8_t L3GD20H_set_mode (uint8_t device_address, uint8_t mode){
    
    int8_t code_error = 0x00;
  
    code_error = L3GD20H_update_register (device_address, L3GD20H_REG_CTRL1, L3GD20H_MODE_MASK, mode);
    
    return (code_error);   
}


//************************************************************
//         L3GD20H_set_full_scale 
//************************************************************
int8_t L3GD20H_set_full_scale (uint8_t device_address, uint8_t full_scale){
    uint8_t readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = L3GD20H_get_register (device_address, L3GD20H_REG_CTRL4, &readback);

    //Reset the mode bits
    readback = readback & L3GD20H_FULL_SCALE_MASK;
    //Set the new bits values associated to the mode
    readback = readback | full_scale;
    
    code_error |= L3GD20H_set_register (device_address, L3GD20H_REG_CTRL4, readback);
    
    if (full_scale == L3GD20H_FULL_SCALE_245){
        L3GD20H_active_full_scale = L3GD20H_sensitivity_245;
    }
    
    if (full_scale == L3GD20H_FULL_SCALE_500){
        L3GD20H_active_full_scale = L3GD20H_sensitivity_500;    
    }
    
    if (full_scale == L3GD20H_FULL_SCALE_2000){
        L3GD20H_active_full_scale = L3GD20H_sensitivity_2000;
    }   
    
    //Update the thresholds, scaling it up with the new full scale
    L3GD20H_set_interrupt_threshold (device_address,L3GD20H_THRESHOLD_X, L3GD20H_active_threshold_X);
    L3GD20H_set_interrupt_threshold (device_address,L3GD20H_THRESHOLD_Y, L3GD20H_active_threshold_Y);
    L3GD20H_set_interrupt_threshold (device_address,L3GD20H_THRESHOLD_Z, L3GD20H_active_threshold_Z);
 
    return (code_error); 
}


//************************************************************
//         L3GD20H_set_event_duration 
//************************************************************
int8_t L3GD20H_set_event_duration (uint8_t device_address, uint8_t wait_enable, uint8_t duration){
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    //The end operation is done to make sure that the value does not exceed the 
    //Allowed one
    value = duration & 0x7F;
    value |= wait_enable;
    
    code_error = L3GD20H_set_register (device_address, L3GD20H_REG_IG_DURATION, value);
        
    return (code_error);    
}


//************************************************************
//         L3GD20H_set_interrupt_threshold 
//************************************************************
int8_t L3GD20H_set_interrupt_threshold (uint8_t device_address, uint8_t component, float threshold_value){
    uint8_t value_L = 0x00;
    uint8_t value_H = 0x00;
    int8_t code_error = 0x00;
    int16_t register_value = 0;
    
    register_value = (int16_t) ((threshold_value *1000) /L3GD20H_active_full_scale);
    
    value_L = (uint8_t) (register_value & 0x00FF);
    value_H = (uint8_t) ((register_value >> 8) & 0x00FF);
    
    if (component == L3GD20H_THRESHOLD_X){
        code_error = L3GD20H_set_register (device_address, L3GD20H_REG_IG_THS_XH, value_H);
        code_error |= L3GD20H_set_register (device_address, L3GD20H_REG_IG_THS_XL, value_L);
        L3GD20H_active_threshold_X = threshold_value;
    }
    
    if (component == L3GD20H_THRESHOLD_Y){
        code_error = L3GD20H_set_register (device_address, L3GD20H_REG_IG_THS_YH, value_H);
        code_error |= L3GD20H_set_register (device_address, L3GD20H_REG_IG_THS_YL, value_L);
        L3GD20H_active_threshold_Y = threshold_value;
    }
    
    if (component == L3GD20H_THRESHOLD_Z){
        code_error = L3GD20H_set_register (device_address, L3GD20H_REG_IG_THS_ZH, value_H);
        code_error |= L3GD20H_set_register (device_address, L3GD20H_REG_IG_THS_ZL, value_L);
        L3GD20H_active_threshold_Z = threshold_value;
    }
        
    return (code_error);      
}


//************************************************************
//         L3GD20H_set_interrupt_enabled 
//************************************************************
int8_t L3GD20H_set_interrupt_enabled (uint8_t device_address, uint8_t interrupt_enable){
    
    int8_t code_error = 0x00;
  
    code_error = L3GD20H_update_register (device_address, L3GD20H_REG_IG_CFG, L3GD20H_INT_ENABLE_MASK, interrupt_enable);
    
    return (code_error);   
}


//************************************************************
//         L3GD20H_set_interrupt_mode 
//************************************************************
int8_t L3GD20H_set_interrupt_mode (uint8_t device_address, uint8_t mode){
    
    int8_t code_error = 0x00;
  
    code_error = L3GD20H_update_register (device_address, L3GD20H_REG_IG_CFG, L3GD20H_INT_MODE_MASK, mode);
    
    return (code_error);   
}


//************************************************************
//         L3GD20H_get_interrupt_status 
//************************************************************
int8_t L3GD20H_set_interrupt_pin (uint8_t device_address, uint8_t pin_function){
    
    int8_t code_error = 0x00;
  
    code_error = L3GD20H_update_register (device_address, L3GD20H_REG_CTRL3, L3GD20H_INT_PIN_MASK, pin_function);
    
    return (code_error);   
}


//************************************************************
//         L3GD20H_set_interrupt_pin_mode 
//************************************************************
int8_t L3GD20H_set_interrupt_pin_mode (uint8_t device_address, uint8_t pin_mode){
    int8_t code_error = 0x00;
  
    code_error = L3GD20H_update_register (device_address, L3GD20H_REG_CTRL3, L3GD20H_INT_PIN_MODE_MASK, pin_mode);
    
    return (code_error);     
}


//************************************************************
//         L3GD20H_get_interrupt_status 
//************************************************************
int8_t L3GD20H_get_interrupt_status (uint8_t device_address, uint8_t * interrupt_status){
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    code_error = L3GD20H_get_register (device_address, L3GD20H_REG_IG_SRC, &value);
    *interrupt_status = value;
        
    return (code_error);
}

