/*******************************************************************************

Author : Mauro Laurenti
Version : 1.8
Created on Date : 09/03/2012
Last update     : 28/03/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/
									
#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "LCD_44780_I2C.h"

uint8_t data_buffer = 0;
uint8_t I2C_device_address = 0;


//************************************************************
//                  LCD_enable_pulse  Implementation
//************************************************************
void LCD_enable_pulse (void) {
	data_buffer = data_buffer | LCD_E;
    
	#ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif 
    
    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif 
         

	delay_ms (1);
    
	data_buffer = data_buffer & ~LCD_E;
    
	#ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif  

    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif     
}


//************************************************************
//                  LCD_send_command  Implementation
//************************************************************
void LCD_send_command (uint8_t data) {
    
    //clear the data bits
	data_buffer = data_buffer & ~(LCD_D0 | LCD_D1 | LCD_D2 | LCD_D3); 
    
    if (data & 0x01) {
      data_buffer |=   LCD_D0;
    }
    
    if (data & 0x02) {
      data_buffer |=   LCD_D1;
    }
    
    if (data & 0x04) {
      data_buffer |=   LCD_D2;
    }
      
    if (data & 0x08) {
      data_buffer |=   LCD_D3;
    }
   

	#ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif  

    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif     

	LCD_enable_pulse ();
}


//************************************************************
//                  LCD_home  Implementation
//************************************************************
void LCD_home (void) {
	
	LCD_send_command (0x00);
	LCD_send_command (0x02);
}


//************************************************************
//                  LCD_shift  Implementation
//************************************************************
void LCD_shift (uint8_t shift, uint8_t number_of_shift) {
	
	uint8_t i;
	
	for (i=0; i < number_of_shift; i++) {	
		LCD_send_command (0x01);
		LCD_send_command (0x08 | shift);
	}	
}


//************************************************************
//                  LCD_shift_cursor  Implementation
//************************************************************
void LCD_shift_cursor (uint8_t shift, uint8_t number_of_shift){
	
	uint8_t i;
	
	for (i=0; i < number_of_shift; i++) {
		LCD_send_command (0x01);
		LCD_send_command (0x00 | shift);
	} 		
}


//************************************************************
//                  LCD_goto_line  Implementation
//************************************************************
void LCD_goto_line (uint8_t line) {

switch(line) {

	case 1: LCD_send_command(0x08);
		LCD_send_command(0x00);
		break;

	case 2: LCD_send_command(0x0C);
		LCD_send_command(0x00);
		break;

	case 3: LCD_send_command(0x09);
		LCD_send_command(0x04);
		break;

	case 4: LCD_send_command(0x0D);
		LCD_send_command(0x04);
	}
}


//************************************************************
//                  LCD_goto_xy  Implementation
//************************************************************
void LCD_goto_xy (uint8_t x, uint8_t y){

    LCD_goto_line (y);
    LCD_shift_cursor (RIGHT, x-1);
}


//************************************************************
//                  LCD_write_char  Implementation
//************************************************************
void LCD_write_char (uint8_t value) {
	
	uint8_t preliminary_buffer;
	
	data_buffer = data_buffer | LCD_RS;
    
	#ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif  

    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif        

	// Splitting of the first nibble
	preliminary_buffer = (value & 0xF0) >> 4;	
					
	LCD_send_command (preliminary_buffer);
		
	// Splitting of the second nibble
	preliminary_buffer = (value & 0x0F);
			
	LCD_send_command (preliminary_buffer);
		
	data_buffer = data_buffer & ~LCD_RS;
    
	#ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif 
        
    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif     
}


//************************************************************
//         LCD_write_data_CGRAM  Implementation
//************************************************************
void LCD_write_data_CGRAM(uint8_t address, uint8_t data)
{
    uint8_t preliminary_buffer;
	
	data_buffer = data_buffer & ~LCD_RS;
    
    #ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif  

    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif   
		
	// Splitting of the first nibble
	preliminary_buffer = (address & 0xF0) >> 4;	
    preliminary_buffer |= 0b00000100;
					
	LCD_send_command (preliminary_buffer);
		
	// Splitting of the second nibble
	preliminary_buffer = (address & 0x0F);
			
	LCD_send_command (preliminary_buffer);
		
	data_buffer = data_buffer & ~LCD_RS;
    
    #ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif  

    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif 
    
    delay_ms(1);
    
    data_buffer = data_buffer | LCD_RS;
    
    #ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif  

    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif 
		
	// Splitting of the first nibble
	preliminary_buffer = (data & 0xF0) >> 4;	
					
	LCD_send_command (preliminary_buffer);
		
	// Splitting of the second nibble
	preliminary_buffer = (data & 0x0F);
			
	LCD_send_command (preliminary_buffer);
		
	data_buffer = data_buffer & ~LCD_RS;
    
    #ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif  

    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif   
    
    LCD_home ();
}


//************************************************************
//              LCD_write_message  Implementation
//************************************************************
void LCD_write_message (const char *buffer) {
	
	 // Write data to LCD up to null
    while(*buffer) {
	    
	    // Write character to LCD
	    LCD_write_char (*buffer);
	    // Increment buffer
	    buffer++;              
   }
}


//************************************************************
//                  LCD_write_string  Implementation
//************************************************************
void LCD_write_string (uint8_t *buffer) {
	
	// Write data to LCD up to null
	while(*buffer){
		
		// Write character to LCD
		LCD_write_char (*buffer);
		// Increment buffer
		buffer++;               
	}
}


//************************************************************
//                  LCD_write_integer  Implementation
//************************************************************
void LCD_write_integer (int16_t value, uint8_t number_of_digits, uint8_t zero_cleaning){
	
    // The array size is 6, plus end of string '\0'
	uint8_t convertedInt [7];

	uint8_t string_length;
    uint8_t counter;
    uint8_t shift_counter;
	
	// Integer is converted to string
    LTlib_int16_to_string (&convertedInt[0], value);
    
    string_length = LTlib_string_length (&convertedInt[0]);

    //Make the string as long as needed by appending 0 on the left
    for (counter = 0; counter < (number_of_digits - string_length); counter++){
        //shift right as needed by adding 0
        for (shift_counter = number_of_digits; shift_counter > 0; shift_counter--){
            convertedInt[shift_counter] = convertedInt[shift_counter-1];
        }   
        
        //After each shift a 0 is inserted for postive numbers
        //For negative numbers and empty space is added
        if (convertedInt[0] == '-' || convertedInt[0] == ' '){
            convertedInt[0] = ' ';
        } else {
            convertedInt[0] = '0';
        }        
    }    
    
    string_length = LTlib_string_length (&convertedInt[0]);
    
    //remove the 0 on the left, if required
    if (zero_cleaning == LCD_ZERO_CLEANING_ON) {       
        //I keep at least one 0, thus I do not chek the entire string length
        for (counter = 0; counter < string_length; counter++){
            if (convertedInt[counter] == '0'){
                convertedInt[counter] = ' ';
            } else {
                //seeting the cunter to this value will terminate the for.
                counter = string_length;
            }   
        }  
    }  

	LCD_write_string (convertedInt);		
}	


//*********************************************
//             LCD_write_float
//*********************************************
void LCD_write_float (float value, uint8_t number_of_digits, uint8_t number_of_fractional_digits, uint8_t zero_cleaning){
  
    uint8_t string_length;
    uint8_t counter;
    uint8_t shift_counter;
    
    uint8_t string_result[MAX_NUM_DIGITS];
    
    LTlib_float_to_string (string_result, value, number_of_fractional_digits);
    
    string_length = LTlib_string_length (&string_result[0]);
    
    //Make the string as long as needed by appending 0 on the left
    for (counter = 0; counter < (number_of_digits - string_length); counter++){
        //shift right as needed by adding 0
        for (shift_counter = number_of_digits; shift_counter > 0; shift_counter--){
            string_result[shift_counter] = string_result[shift_counter-1];
        }   
        
        //After each shift a 0 is inserted for positive numbers
        //For negative numbers and empty space is added
        if (string_result[0] == '-' || string_result[0] == ' '){
            string_result[0] = ' ';
        } else {
            string_result[0] = '0';
        }        
    }    
    
    string_length = LTlib_string_length (&string_result[0]);
    
    //remove the 0 on the left, if required
    if (zero_cleaning == LCD_ZERO_CLEANING_ON) {       
        //I keep at least one 0, thus I do not check the entire string length
        for (counter = 0; counter < string_length; counter++){
            
            if (string_result[counter] != '.'){
                
                if (string_result[counter] == '0'){
                    string_result[counter] = ' ';
                } else {
                    //setting the counter to this value will terminate the for.
                    counter = string_length;
                }   
            }
        }  
    }  

    
    LCD_write_string (string_result);
}


//************************************************************
//                  LCD_clear  Implementation
//************************************************************
void LCD_clear (void){
	
	LCD_send_command (0x00);
	LCD_send_command (0x01);
}


//************************************************************
//                  cursor_LCD  Implementation
//************************************************************
void LCD_cursor (uint8_t active, uint8_t blinking) {
	
	LCD_send_command (0x00);
	LCD_send_command (0x0C | active | blinking);
}


//************************************************************
//                  LCD_backlight  Implementation
//************************************************************
void LCD_backlight (uint8_t active) {

	//Clear the LED bit
	data_buffer = data_buffer & ~LCD_TURN_ON_LED;

	//write the data within the buffer
	data_buffer = data_buffer | active;
    
    #ifdef PCF8574_GENERIC
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif   
        
    #ifdef MCP23008
        MCP2300x_set_port_value (I2C_device_address, data_buffer); 
    #endif 
}


//************************************************************
//                  LCD_initialize  Implementation
//************************************************************
void  LCD_initialize (uint8_t quartz_frequency) {
        
	delay_set_quartz (quartz_frequency);

    data_buffer = 0;  
    
    #ifdef PCF8574_GENERIC
        I2C_device_address = DEVICE_ADDRESS_L;
        PCF8574_initialize (LCD_BUS_DATA_RATE);
        PCF8574_write_data (I2C_device_address, data_buffer);
    #endif

    //The first initialization is done at lower speed
    //After the initialization the bus speet is set to LCD_BUS_DATA_RATE    
    #ifdef MCP23008
        I2C_device_address = DEVICE_ADDRESS_H | (DEVICE_ADDRESS_L << 1);
        
        MCP2300x_initialize (50);
        //Set the device with all ouptut pins
        MCP2300x_set_port_direction (I2C_device_address, 0x00);
        //Set all the output to 0
        MCP2300x_set_port_value (I2C_device_address, data_buffer);        
    #endif

       
	delay_ms (100);
	LCD_send_command (0x03);
	delay_ms (10);
	LCD_send_command (0x03);
	delay_ms (1);
	LCD_send_command (0x03);
		
	LCD_send_command (0x02);

	LCD_send_command (0x02);
	LCD_send_command (0x08);

	LCD_send_command (0x00);
	LCD_send_command (0x08);
        
    LCD_send_command (0x00);
	LCD_send_command (0x01);
        
    LCD_send_command (0x00);
	LCD_send_command (0x06);
  
	LCD_cursor (LCD_TURN_OFF_CURSOR, LCD_BLINKING_OFF);
	LCD_clear ();
    
    delay_ms (10);
    
    #ifdef MCP23008
        MCP2300x_initialize (LCD_BUS_DATA_RATE);
        //Set the device with all ouptut pins
        MCP2300x_set_port_direction (I2C_device_address, 0x00);
        //Set all the output to 0
        MCP2300x_set_port_value (I2C_device_address, data_buffer);        
    #endif
    
}

