/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
 
Created on Date : 19/07/2023
Last update     : 19/07/2023
 
CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "LSM6DSM.h"


//************************************************************
//         LSM6DSM_initialize 
//************************************************************
void LSM6DSM_initialize (uint8_t device_address, uint16_t baud_rate_KHz){

    LSM6DSM_I2C_open (I2C_MASTER);
    LSM6DSM_I2C_baud_rate (baud_rate_KHz);
    
    LSM6DSM_software_reset (device_address);
    
    LSM6DSM_set_power_mode (device_address, LSM6DSM_POWER_DOWN_ALL);
        
    //As per datasheet, a typical time of 50ms must be wait before triggering 
    //the first measure. Some time is passed already during the initialization.
    delay_ms (55);
}


//************************************************************
//         LSM6DSM_set_register 
//************************************************************
int8_t LSM6DSM_set_register (uint8_t device_address, uint8_t register_add,  uint8_t data){

    uint8_t control_byte = 0;

    control_byte = device_address <<1;

    return (LSM6DSM_I2C_write_byte_to_external_device (control_byte,register_add, data));
}


//************************************************************
//           LSM6DSM_get_register 
//************************************************************
int8_t LSM6DSM_get_register (uint8_t device_address, uint8_t register_address, uint8_t * readback_value){
	uint8_t value = 0x00;
    uint8_t control_byte = 0;
    int8_t error = 0;
    
    control_byte = device_address <<1;

    error = LSM6DSM_I2C_read_byte_from_external_device (control_byte, register_address, &value);
    
    *readback_value = value;
            
    return (error);
}


//************************************************************
//           LSM6DSM_update_register 
//************************************************************
int8_t LSM6DSM_update_register (uint8_t device_address, uint8_t register_address, uint8_t register_mask, uint8_t new_value){
    uint8_t readback = 0x00;
    int8_t code_error = 0x00;
   
    code_error = LSM6DSM_get_register (device_address, register_address, &readback);

    //Reset the register with the mask
    readback = readback & register_mask;
    //Set the new bits values 
    readback = readback | new_value;
    //Update the register 
    code_error |= LSM6DSM_set_register (device_address, register_address, readback);
        
    return (code_error); 
}

//************************************************************
//           LSM6DSM_get_temperature 
//************************************************************
int8_t LSM6DSM_get_temperature (uint8_t device_address, int16_t * temperature){
    uint8_t temperature_L = 0;
    uint8_t temperature_H = 0;
    int16_t temperature_raw = 0;
    
    int8_t code_error = 0;
   
    //The data read back from the sensor is the ADC value and is 16 bits 2 comp.
    code_error = LSM6DSM_get_register (device_address, LSM6DSM_REG_OUT_TEMP_L, &temperature_L);
    code_error |= LSM6DSM_get_register (device_address, LSM6DSM_REG_OUT_TEMP_H, &temperature_H);

    temperature_raw =  (((int16_t)(temperature_H))<<8);
    temperature_raw |= ((int16_t)(temperature_L));
    
    *temperature = temperature_raw;
    
    return (code_error);
}


//************************************************************
//     LSM6DSM_read_acceleration function implementation
//************************************************************
int8_t LSM6DSM_get_acceleration (uint8_t device_address, float * Acc_X, float * Acc_Y, float * Acc_Z ){
   
    int8_t error_code = 0x00;
    
    int16_t Acc_X_readback = 0;
    int16_t Acc_Y_readback = 0;
    int16_t Acc_Z_readback = 0;
    
    uint8_t value = 0;
        
    
    //*********************************
    // Acceleration component X
    //*********************************
    error_code = LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTX_L_XL, &value);
    Acc_X_readback = ((int16_t) value);
    error_code = LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTX_H_XL, &value);
    Acc_X_readback = ((int16_t) value) << 8;
    
    //*********************************
    // Acceleration component Y
    //*********************************
    error_code = LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTY_L_XL, &value);
    Acc_Y_readback = ((int16_t) value);
    error_code = LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTY_H_XL, &value);
    Acc_Y_readback = ((int16_t) value) << 8;
    
    //*********************************
    // Acceleration component Z
    //*********************************
    error_code = LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTZ_L_XL, &value);
    Acc_Z_readback = ((int16_t) value);
    error_code = LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTZ_H_XL, &value);
    Acc_Z_readback = ((int16_t) value) << 8;
           
    *Acc_X = (((float) Acc_X_readback) * LSM6DSM_ACC_active_full_scale) /1000;
    *Acc_Y = (((float) Acc_Y_readback) * LSM6DSM_ACC_active_full_scale) /1000;
    *Acc_Z = (((float) Acc_Z_readback) * LSM6DSM_ACC_active_full_scale) /1000;
   
    return (error_code);
}


//************************************************************
//           LSM6DSM_get_angular_speed
//************************************************************
int8_t LSM6DSM_get_angular_speed (uint8_t device_address, float *dps_X, float *dps_Y, float *dps_Z){
    int8_t code_error = 0;
    uint8_t value = 0;
    int16_t dps_X_readback = 0;
    int16_t dps_Y_readback = 0;
    int16_t dps_Z_readback = 0;
    
    *dps_X = 0;
    *dps_Y = 0;
    *dps_Z = 0;
    
    //***************************
    // X read out
    //***************************
    code_error = LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTX_L_G, &value);
    dps_X_readback = (int16_t) value;
    code_error |= LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTX_H_G, &value);
    dps_X_readback |= (((int16_t) value) << 8);             
    
    //***************************
    // Y read out
    //***************************
    code_error |= LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTY_L_G, &value);
    dps_Y_readback = (int16_t) value;
    code_error |= LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTY_H_G, &value);
    dps_Y_readback |= (((int16_t) value) << 8);
    
    //***************************
    // Z read out
    //***************************
    code_error |= LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTZ_L_G, &value);
    dps_Z_readback = (int16_t) value;
    code_error |= LSM6DSM_get_register (device_address, LSM6DSM_REG_OUTZ_H_G, &value);
    dps_Z_readback |= (((int16_t) value) << 8);
    
    *dps_X = ((float)(dps_X_readback) * LSM6DSM_GYRO_active_full_scale)/1000.0;  
    *dps_Y = ((float)(dps_Y_readback) * LSM6DSM_GYRO_active_full_scale)/1000.0;
    *dps_Z = ((float)(dps_Z_readback) * LSM6DSM_GYRO_active_full_scale)/1000.0;  
                
    return (code_error);    
}

//************************************************************
//           LSM6DSM_set_sampling_frequency 
//************************************************************
int8_t LSM6DSM_set_sampling_frequency (uint8_t device_address, uint8_t sub_system, uint8_t sampling_frequency){
    
    int8_t code_error = 0;
    uint8_t sampling_reg = 0;
    uint8_t mask = 0;
    
    if (sub_system == LSM6DSM_ACCELEROMETER){
        sampling_reg = LSM6DSM_REG_CTRL1_XL;
        mask = LSM6DSM_ACC_SAMPLING_MASK;
    }
    
    if (sub_system == LSM6DSM_GYROSCOPE){
        sampling_reg = LSM6DSM_REG_CTRL2_G;
        mask = LSM6DSM_GYRO_SAMPLING_MASK;
    }
    
    code_error = LSM6DSM_update_register (device_address, sampling_reg, mask, sampling_frequency);
    
    return (code_error); 
}



//************************************************************
//         LSM6DSM_set_full_scale 
//************************************************************
int8_t LSM6DSM_set_full_scale (uint8_t device_address, uint8_t sub_system, uint8_t full_scale){
    
    int8_t code_error = 0;
    uint8_t fullscale_reg = 0;
    uint8_t mask = 0;
    
    if (sub_system == LSM6DSM_ACCELEROMETER){
        fullscale_reg = LSM6DSM_REG_CTRL1_XL;
        mask = LSM6DSM_ACC_FULLSCALE_MASK;
    }
    
    if (sub_system == LSM6DSM_GYROSCOPE){
        fullscale_reg = LSM6DSM_REG_CTRL2_G;
        mask = LSM6DSM_GYRO_FULLSCALE_MASK;
    }
   
    code_error = LSM6DSM_update_register (device_address, fullscale_reg, mask, full_scale);
    
    //Accelerometer Active full scale
    if (full_scale == LSM6DSM_ACC_FULLSCALE_2g){
        LSM6DSM_ACC_active_full_scale = LSM6DSM_ACC_sensitivity_2g;
    }
    
    if (full_scale == LSM6DSM_ACC_FULLSCALE_4g){
        LSM6DSM_ACC_active_full_scale = LSM6DSM_ACC_sensitivity_4g;
    }
    
    if (full_scale == LSM6DSM_ACC_FULLSCALE_8g){
        LSM6DSM_ACC_active_full_scale = LSM6DSM_ACC_sensitivity_8g;
    }
    
    if (full_scale == LSM6DSM_ACC_FULLSCALE_16g){
        LSM6DSM_ACC_active_full_scale = LSM6DSM_ACC_sensitivity_16g;
    }
    
    //Gyroscope Active full scale
    if (full_scale == LSM6DSM_GYRO_FULLSCALE_125_dps){
        LSM6DSM_GYRO_active_full_scale = LSM6DSM_GYRO_sensitivity_125;
    }
    
    if (full_scale == LSM6DSM_GYRO_FULLSCALE_250_dps){
        LSM6DSM_GYRO_active_full_scale = LSM6DSM_GYRO_sensitivity_250;    
    }
    
    if (full_scale == LSM6DSM_GYRO_FULLSCALE_500_dps){
        LSM6DSM_GYRO_active_full_scale = LSM6DSM_GYRO_sensitivity_500;
    }   
    
    if (full_scale == LSM6DSM_GYRO_FULLSCALE_1000_dps){
        LSM6DSM_GYRO_active_full_scale = LSM6DSM_GYRO_sensitivity_1000;
    }   
    
    if (full_scale == LSM6DSM_GYRO_FULLSCALE_2000_dps){
        LSM6DSM_GYRO_active_full_scale = LSM6DSM_GYRO_sensitivity_2000;
    }   

    return (code_error); 
}


//************************************************************
//           LSM6DSM_get_device_ID 
//************************************************************
int8_t LSM6DSM_get_device_ID (uint8_t device_address, uint8_t * device_ID){
   
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    code_error = LSM6DSM_get_register (device_address, LSM6DSM_REG_WHO_AM_I, &value);
    *device_ID = value;
        
    return (code_error);
}


//************************************************************
//         LSM6DSM_software_reset 
//************************************************************
int8_t LSM6DSM_software_reset (uint8_t device_address){
    
    int8_t code_error = 0x00;
  
    code_error = LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL3_C, LSM6DSM_RESET_MASK, LSM6DSM_SOFTWARE_RESET);
    
    return (code_error);  
}


//************************************************************
//         LSM6DSM_set_mode 
//************************************************************
int8_t LSM6DSM_set_power_mode (uint8_t device_address, uint8_t mode){
    
    int8_t code_error = 0x00;

    
    if (mode == LSM6DSM_POWER_DOWN_ALL){
        code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL1_XL, LSM6DSM_ACC_POWER_DOWN_MASK, 0x00);  
        code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL2_G, LSM6DSM_GYRO_POWER_DOWN_MASK, 0x00); 
    }
    
    if (mode == LSM6DSM_POWER_DOWN_ACC){
        code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL1_XL, LSM6DSM_ACC_POWER_DOWN_MASK, 0x00);  
    }
    
    if (mode == LSM6DSM_POWER_DOWN_GYRO){
        code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL2_G, LSM6DSM_GYRO_POWER_DOWN_MASK, 0x00); 
    }
    
    if (mode == LSM6DSM_POWER_ON_ALL){
        code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL1_XL, LSM6DSM_ACC_POWER_DOWN_MASK, LSM6DSM_ACC_SAMPLING_12_5_HZ);  
        code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL2_G, LSM6DSM_GYRO_POWER_DOWN_MASK, LSM6DSM_GYRO_SAMPLING_12_5_HZ); 
    }
    
    if (mode == LSM6DSM_POWER_ON_ACC){
        code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL1_XL, LSM6DSM_ACC_POWER_DOWN_MASK, LSM6DSM_ACC_SAMPLING_12_5_HZ);  
    }
    
    if (mode == LSM6DSM_POWER_ON_GYRO){
        code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL2_G, LSM6DSM_GYRO_POWER_DOWN_MASK, LSM6DSM_GYRO_SAMPLING_12_5_HZ); 
    }

    return (code_error);   
}


//************************************************************
//         LSM6DSM_set_interrupt_pin
//************************************************************
int8_t LSM6DSM_set_interrupt_pin_signals (uint8_t device_address, uint8_t int_pin, uint8_t signal){
    
    int8_t code_error = 0x00;
    uint8_t data = 0x00;
  

    if (int_pin == LSM6DSM_INT1_PIN){
        
        if (signal == LSM6DSM_INT1_RESET_ALL_SIGNALS){
            code_error |= LSM6DSM_set_register (device_address, LSM6DSM_REG_INT1_CTRL, 0x00);            
        } else {                
            code_error |= LSM6DSM_get_register (device_address, LSM6DSM_REG_INT1_CTRL, &data);
            data |= signal;
            code_error |= LSM6DSM_set_register (device_address, LSM6DSM_REG_INT1_CTRL, data);
        }
    
    }
    
    if (int_pin == LSM6DSM_INT2_PIN){
        if (signal == LSM6DSM_INT2_RESET_ALL_SIGNALS){
            code_error |= LSM6DSM_set_register (device_address, LSM6DSM_REG_INT2_CTRL, 0x00);
        }else {                
            code_error |= LSM6DSM_get_register (device_address, LSM6DSM_REG_INT2_CTRL, &data);
            data |= signal;
            code_error |= LSM6DSM_set_register (device_address, LSM6DSM_REG_INT2_CTRL, data);
        }      
    }    

    return (code_error);   
}


//************************************************************
//         LSM6DSM_set_interrupt_pin_mode 
//************************************************************
int8_t LSM6DSM_set_interrupt_pin_mode (uint8_t device_address, uint8_t pin_mode, uint8_t merge){
    int8_t code_error = 0x00;
  
    code_error = LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL4_C, LSM6DSM_INT_MERGE_RESET_MASK, merge);
    code_error |= LSM6DSM_update_register (device_address, LSM6DSM_REG_CTRL3_C, LSM6DSM_INT_MODE_RESET_MASK, pin_mode);
    return (code_error);     
}


//************************************************************
//         LSM6DSM_get_interrupt_status 
//************************************************************
int8_t LSM6DSM_get_interrupt_status (uint8_t device_address, uint8_t * interrupt_status){
    uint8_t value = 0x00;
    int8_t code_error = 0x00;
   
    code_error = LSM6DSM_get_register (device_address, LSM6DSM_REG_STATUS, &value);
    *interrupt_status = value;
        
    return (code_error);
}

