/*******************************************************************************

Author : Mauro Laurenti
Version : 1.6

Created on Date : 04/09/2006
Last update     : 19/11/2022

CopyRight 2006-2022 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/


#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "LTlib_delay.h"

volatile uint8_t delay_quartz_frequency_value = 20;
volatile uint16_t clock_counter_reference = 0;

#ifdef COMPILER_XC8
    #ifdef DELAY_INTERRUPT_BLOCKING
        volatile uint8_t delay_flag_GIE = 0;
        volatile uint8_t delay_flag_PEIE = 0;
    #endif
#endif

//************************************************************
//             delay_ms function implementation
//************************************************************

#ifdef COMPILER_XC8
void delay_ms (uint16_t value_ms) {

    #ifdef DELAY_INTERRUPT_BLOCKING
        delay_flag_GIE = INTCONbits.GIE;
        // Disable Interrupt
        INTCONbits.GIE = 0;

        delay_flag_PEIE = INTCONbits.PEIE;
        // Disable Interrupt
        INTCONbits.PEIE = 0;
    #endif	  

	clock_counter_reference = value_ms * delay_quartz_frequency_value;

    while (clock_counter_reference) {


        //1ms Delay at 1MHz trimmed down to 240 instead of 250
        //to compensate the main loop
        _delay(240);

        clock_counter_reference--;
	}

    #ifdef DELAY_INTERRUPT_BLOCKING
        // Reload old settings
        INTCONbits.GIE = delay_flag_GIE;
        INTCONbits.PEIE = delay_flag_PEIE;
    #endif       
}
#endif


#ifdef COMPILER_XC16
void delay_ms (uint16_t value_ms) {

    T1CON = 0x00;
    TMR1 = 0x00;
    
    // 1 ms at 1MHz (Tcy = 2us) PR1 = 0x01F4
    // Calibration with scope for the while - 10 (0x01EA)
    PR1 = 0x01EA;   
    
    IFS0bits.T1IF = 0x00;
    T1CONbits.TON = 0x01;
    
 	clock_counter_reference = value_ms * delay_quartz_frequency_value;

    //Each cycle waits for 1ms
    while (clock_counter_reference) {
         
        while (IFS0bits.T1IF == 0);
        
        IFS0bits.T1IF = 0x00;
        TMR1 = 0x00;
        
        clock_counter_reference--;
	}
    
}

void delay_us (uint16_t value_us){
    T1CON = 0x00;
    TMR1 = 0x00;
    
    // 1 ms at 1MHz (Tcy = 2us) PR1 = 0x01F4
    // 14 is equal to 20us
    PR1 = 0x0014;   
    
    IFS0bits.T1IF = 0x00;
    T1CONbits.TON = 0x01;
    
 	clock_counter_reference = (value_us * delay_quartz_frequency_value) / 20;

    //Each cycle waits for 20us
    while (clock_counter_reference) {
         
        while (IFS0bits.T1IF == 0);
        
        IFS0bits.T1IF = 0x00;
        TMR1 = 0x00;
        
        clock_counter_reference--;
	}    
}

#endif

#ifdef COMPILER_XC32
//************************************************************
//             delay_ms function implementation
//************************************************************
void delay_ms (uint16_t value_ms) {
    uint32_t counter = 0;
    
    //1 ms 
    counter = (delay_quartz_frequency_value * 1000)/2;
    
    counter = counter * value_ms;
    
    _CP0_SET_COUNT(0);
    while (counter > _CP0_GET_COUNT()); 
}

//************************************************************
//             delay_us function implementation
//************************************************************
void delay_us (uint16_t value_us){
    
    uint16_t counter = 0;
    uint16_t calibration = 0;
    
    //1 us 
    counter = delay_quartz_frequency_value/2;
    
    //The number 4 is an offset of 4us measured via the Oscilloscope.
    calibration = counter * 4;
    counter = (counter * value_us) - calibration;
    
    _CP0_SET_COUNT(0);
    while (counter > _CP0_GET_COUNT()); 
} 

#endif            


//************************************************************
//             delay_s function implementation
//************************************************************
void delay_s (uint8_t value_s) {

	uint8_t repeat_loop;

	for (repeat_loop = 0; repeat_loop < value_s; repeat_loop++)
		delay_ms (1000);
}


//************************************************************
//             setQuartz function implementation
//************************************************************
void delay_set_quartz (uint8_t frequency) {

	delay_quartz_frequency_value = frequency;	
} 
