/*******************************************************************************

Author : Mauro Laurenti
Version : 1.3
 
Created on Date : 13/03/2015
Last update     : 25/07/2019

CopyRight 2006-2019 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

This library supports the Real Time Clock Calendar MCP7940 or compatible
devices. The library is kept compatible with the PCF8563

The following functions have been added:

 set_seconds_alarm_RTCC (seconds associated to the first alarm)

 The second alarm is not handled but it can be easily implemented.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "MCP7940.h"


//************************************************************
//         RTCC_initialize function implementation
//************************************************************
void RTCC_initialize (uint16_t baud_rate_KHz){
    MCP7940_I2C_open (I2C_MASTER);
    MCP7940_I2C_baud_rate(baud_rate_KHz);
    
    RTCC_start_oscillator ();
}


//************************************************************
//         RTCC_start_oscillator function implementation
//************************************************************
void RTCC_start_oscillator (void){
    
    // A dummy write is required to let the 32KHz oscillator start
    RTCC_set_seconds (0);

    //short delay to make sure that the 32KHz crystal is oscillating
	delay_ms (1500);
}


//************************************************************
//             RTCC_set_seconds function implementation
//************************************************************
int8_t RTCC_set_seconds (uint8_t seconds) {
   
    //This or indirectly enable the internal oscillator buffer
    seconds |= 0x80;
    
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_SECONDS_ADDR,seconds));
}


//************************************************************
//             RTCC_get_seconds function implementation
//************************************************************
uint8_t RTCC_get_seconds (void) {
	
	uint8_t seconds;
	
	MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_SECONDS_ADDR, &seconds);
	
	// I set to 0 the bits which are not significant
	seconds = seconds & 0x7F;	
	return (seconds);
}


//************************************************************
//             RTCC_set_minutes function implementation
//************************************************************
int8_t RTCC_set_minutes (uint8_t minutes) {
	
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_MINUTES_ADDR,minutes));	
}


//************************************************************
//             RTCC_get_minutes function implementation
//************************************************************
uint8_t RTCC_get_minutes (void) {
	
	uint8_t minutes;
    
	MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_MINUTES_ADDR, &minutes);
	
	// I set to 0 the bits which are not significant
	minutes = minutes & 0x7F;	
	return (minutes);
}


//************************************************************
//             RTCC_set_hours function implementation
//************************************************************
int8_t RTCC_set_hours (uint8_t hours) {
		
    return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_HOURS_ADDR,hours));
}


//************************************************************
//             RTCC_get_hours function implementation
//************************************************************
uint8_t RTCC_get_hours (void){

    uint8_t hours;
    
    MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_HOURS_ADDR, &hours);
	
    // I set to 0 the bits which are not significant
    hours = hours & 0x3F;
    return (hours);
}


//************************************************************
//             RTCC_get_time_seconds function implementation
//************************************************************
uint8_t* RTCC_get_time_seconds (void) {
	
	static uint8_t time[9];
	uint8_t value;

	value = RTCC_get_hours();
	
	// with +48 I convert the number in ASCII number
	time[1] = (value & 0x0F)+48; 
	time[0] = (value >> 4)+48;
	
	time[2] = ':';
	
	value = RTCC_get_minutes();
	time[4] = (value & 0x0F)+48;
	time[3] = (value >> 4)+48;
	
	time[5] = '.';

	value = RTCC_get_seconds();
	time[7] = (value & 0x0F)+48;
	time[6] = (value >> 4)+48;
	
	time[8] = '\0';
		
	return (time);	
}


//************************************************************
//             RTCC_get_time function implementation
//************************************************************
uint8_t* RTCC_get_time (void) {
	
	static uint8_t time[6];
	uint8_t value;

	value = RTCC_get_hours();
	
	// with +48 I convert the number in ASCII number
	time[1] = (value & 0x0F)+48; 
	time[0] = (value >> 4)+48;
	
	time[2] = ':';
	
	value = RTCC_get_minutes();
	time[4] = (value & 0x0F)+48;
	time[3] = (value >> 4)+48;	
	
	time[5] = '\0';
		
	return (time);	
}


//************************************************************
//             RTCC_set_days function implementation
//************************************************************
int8_t RTCC_set_days (uint8_t days) {
	
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_DAYS_ADDR,days));
}


//************************************************************
//             RTCC_get_days function implementation
//************************************************************
uint8_t RTCC_get_days (void) {
	
	uint8_t days;
    
	MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_DAYS_ADDR, &days);
	
	// I set to 0 the bits which are not significant
	days = days & 0x3F;	
	return (days);
}


//************************************************************
//             RTCC_set_months function implementation
//************************************************************
int8_t RTCC_set_months (uint8_t months) {
	
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_MONTHS_ADDR,months));	
}
																				

//************************************************************
//             RTCC_get_months function implementation
//************************************************************
uint8_t RTCC_get_months (void) {
	
	uint8_t months;
    
	MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_MONTHS_ADDR, &months);
	
	// I set to 0 the bits which are not significant
	months = months & 0x1F;	
	return (months);
}


//************************************************************
//             RTCC_set_years function implementation
//************************************************************
int8_t RTCC_set_years (uint8_t years) {
	
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_YEARS_ADDR,years));
}


//************************************************************
//             RTCC_get_years function implementation
//************************************************************
uint8_t RTCC_get_years (void) {
	
	uint8_t years;
    
	MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_YEARS_ADDR, &years);
	return (years);
}


//************************************************************
//             RTCC_get_date function implementation
//************************************************************
uint8_t* RTCC_get_date (void) {
	
	static uint8_t date[9];
	uint8_t value;

	value = RTCC_get_days();
	
	// with +48 I convert the number in ASCII number
	date[1] = (value & 0x0F)+48; 
	date[0] = (value >> 4)+48;
	
	date[2] = '/';
	
	value = RTCC_get_months();
	date[4] = (value & 0x0F)+48;
	date[3] = (value >> 4)+48;
	
	date[5] = '/';

	value = RTCC_get_years();
	date[7] = (value & 0x0F)+48;
	date[6] = (value >> 4)+48;
	
	date[8] = '\0';
		
	return (date);	
}


//************************************************************
//             RTCC_set_seconds_alarm function implementation
//************************************************************
int8_t RTCC_set_seconds_alarm (uint8_t seconds, uint8_t alarm_enable) {
    
    uint8_t value;
		
    if (alarm_enable == RTCC_ENABLE_ON) {

        MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_ALARM_CONTROL_REG_1_ADDR, &value);    
        value = value | 0x10;
        MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_ALARM_CONTROL_REG_1_ADDR,value);   
    }
    
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_SECONDS_ALARM_ADDR,seconds));
}


//************************************************************
//             RTCC_set_minutes_alarm function implementation
//************************************************************
int8_t RTCC_set_minutes_alarm (uint8_t minutes, uint8_t alarm_enable) {
    
	uint8_t value;
    
    if (alarm_enable == RTCC_ENABLE_ON) {

        MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_ALARM_CONTROL_REG_1_ADDR, &value);    
        value = value | 0x10;
        MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_ALARM_CONTROL_REG_1_ADDR,value);   
    }
    
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_MINUTES_ALARM_ADDR,minutes));	
}


//************************************************************
//             RTCC_set_hours_alarm function implementation
//************************************************************
int8_t RTCC_set_hours_alarm (uint8_t hours, uint8_t alarm_enable) {
	
    uint8_t value;
    
    if (alarm_enable == RTCC_ENABLE_ON) {

        MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_ALARM_CONTROL_REG_1_ADDR, &value);    
        value = value | 0x10;
        MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_ALARM_CONTROL_REG_1_ADDR,value);   
    }
    
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_HOURS_ALARM_ADDR,hours));
}


//************************************************************
//             RTCC_set_days_alarm function implementation
//************************************************************
int8_t RTCC_set_days_alarm (uint8_t days, uint8_t alarm_enable) {
	
    uint8_t value;
    
    if (alarm_enable == RTCC_ENABLE_ON) {

        MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_ALARM_CONTROL_REG_1_ADDR, &value);    
        value = value | 0x10;
        MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_ALARM_CONTROL_REG_1_ADDR,value);   
    }
    
	return (MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_DAYS_ALARM_ADDR,days));
}


//************************************************************
//             RTCC_enable_alarm_interrupt function implementation
//************************************************************
int8_t RTCC_enable_alarm_interrupt (void) {

    uint8_t value;
    MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_CONFIGURATION_REG_ADDR, &value);    
    value = value | 0x80;
    MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_CONFIGURATION_REG_ADDR,value);   
    
    return (0);
}


//************************************************************
//      RTCC_disable_alarm_interrupt function implementation
//************************************************************
int8_t RTCC_disable_alarm_interrupt (void) {
	
    uint8_t value;
    
    MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD,RTCC_CONFIGURATION_REG_ADDR, &value);    
    value = value & 0x7F;
    MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD,RTCC_CONFIGURATION_REG_ADDR,value);   
    
    return (0);
}


//************************************************************
//       RTCC_is_alarm_ON function implementation
//************************************************************
uint8_t RTCC_is_alarm_ON (void) {
	
	uint8_t value;
    
	MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD, RTCC_CONFIGURATION_REG_ADDR, &value);
	
    //value = MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD, RTCC_ALARM_CONTROL_REG_1_ADDR);
    // 0b00001000
    
	// Just ALM1 and ALM2 bits are controlled
    //If one alarm is set I reset both flag. Currently only one alarm is suspported
	if (value & 0x30) {
		
        MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD, RTCC_ALARM_CONTROL_REG_1_ADDR, &value);
        //Reset the flags
		value = value & 0xF7;        
		// I clean the flag bits  without changing the other bits
		MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD, RTCC_ALARM_CONTROL_REG_1_ADDR,value);
        
        MCP7940_I2C_read_byte_from_external_device (RTCC_WRITE_ADD, RTCC_ALARM_CONTROL_REG_2_ADDR, &value);
        //Reset the flags
		value = value & 0xF7;
		// I clean the flag bits  without changing the other bits
		MCP7940_I2C_write_byte_to_external_device (RTCC_WRITE_ADD, RTCC_ALARM_CONTROL_REG_2_ADDR,value);     
        
		return (1);	
	
	} else {
		return (0);	
	}		
}


//************************************************************
//         RTCC_increment_minutes function implementation
//************************************************************
int8_t RTCC_increment_minutes (void) {
	
	uint8_t minutes;
	int8_t error;
	
	// Read the current minutes
	minutes = RTCC_get_minutes ();
	
	// Increment the minutes
	minutes++;
	
	// Check the minute limits

	if ((minutes&0x0F) > (uint8_t) 9 ) {		
		minutes &= 0xF0;
		minutes += 0x10;
	}
	
	if (minutes == (uint8_t) RTCC_MAX_MINUTES) {
		
		minutes = 0;
	}
	
	// Update the minutes
	error = RTCC_set_minutes (minutes);
	
	return (error);		
}


//************************************************************
//           RTCC_increment_hours function implementation
//************************************************************
int8_t RTCC_increment_hours (void) {
	
	uint8_t hours;
	int8_t error;
	
	// Read the current hours
	hours = RTCC_get_hours ();
	
	// Increment the hours
	hours++;
	
	// Check the hour limits	
	if ((hours&0x0F) > (uint8_t) 9 ) {		
		hours &= 0xF0;
		hours += 0x10;
	}
	
	if (hours == (uint8_t) RTCC_MAX_HOURS) {
		
		hours = 0;
	}
	
	// Update the hours
	error = RTCC_set_hours (hours);
	
	return (error);	
}


//************************************************************
//           RTCC_increment_years function implementation
//************************************************************
int8_t RTCC_increment_years (void) {
	
	uint8_t years;
	int8_t error;
	
	// Read the current years
	years = RTCC_get_years ();
	
	// Increment the years
	years++;
	
	// Check the year limits	
	if ((years&0x0F) > (uint8_t) 9 ) {		
		years &= 0xF0;
		years += 0x10;
	}
	
	if (years == (uint8_t) RTCC_MAX_YEARS) {
		
		years = 0;
	}
	
	// Update the years
	error = RTCC_set_years (years);
	
	return (error);
	
}


//************************************************************
//           RTCC_increment_months function implementation
//************************************************************
int8_t RTCC_increment_months (void) {
	
	uint8_t months;
	int8_t error;
	
	// Read the current months
	months = RTCC_get_months ();
	
	// Increment the months
	months++;
	
	// Check the month limits
	
	if ((months&0x0F) > (uint8_t) 9 ) {		
		months &= 0xF0;
		months += 0x10;
	}
	
	if (months == (uint8_t) RTCC_MAX_MONTHS) {
		
		months = 1;
	}
	
	// Update the months
	error = RTCC_set_months (months);
	
	return (error);	
}


//************************************************************
//           RTCC_increment_days function implementation
//************************************************************
int8_t RTCC_increment_days (void) {
	
	uint8_t days;
	int8_t error;
	
	// Read the current days
	days = RTCC_get_days ();
	
	// Increment the days
	days++;
	
	// Check the day limits
	
	if ((days&0x0F) > (uint8_t) 9 ) {		
		days &= 0xF0;
		days += 0x10;
	}
	
	if (days == (uint8_t) RTCC_MAX_DAYS) {
		
		days = 1;
	}
	
	// Update the days
	error = RTCC_set_days (days);
	
	return (error);	
}
