/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0
Created on Date : 18/02/2024
Last update     : 18/02/2024

CopyRight 2006-2024 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "OLED_SSD1306.h"
#include "display_utilities.h"


//*********************************************
//         Private global variables
//*********************************************
uint8_t Display_font_width = 0;
uint8_t Display_font_height = 0;

uint8_t Display_font_color = DISPLAY_FILLING_BLACK;

uint8_t Display_column_pointer = 0;
uint8_t Display_page_pointer = 0;

uint8_t Display_double_buffer = DISPLAY_DOUBLE_BUFFER_OFF;


// Set of commands to initialize the de OLED display
uint8_t SSD1306_init_array []  = { 

    0x20, 0b00,      // Set Memory Addressing Mode
    // 00=Horizontal Addressing Mode; 01=Vertical Addressing Mode;
    // 10=Page Addressing Mode (RESET); 11=Invalid
    0xC8,            // Set COM Output Scan Direction
    0xA1,            // Set Segment Re-map. A0=address mapped; A1=address 127 mapped.    
    0xA4,            // Output RAM to Display
					 // 0xA4=Output follows RAM content; 0xA5,Output ignores RAM content    
    0xD5,            // --set display clock divide ratio/oscillator frequency
    0xF0,            // --set divide ratio
    0xD9, 0x22,      // Set pre-charge period                      
    0xDB,            // --set vcomh
    0x20,            // 0x20,0.77xVcc
    0x8D, 0x14,      // Set DC-DC enable
        
};


//*********************************************
//        Display_write_command
//*********************************************
void Display_write_command (uint8_t * command_to_send, uint8_t length) {

    uint8_t Display_address_RW = 0;
    uint8_t Command_array[32];
    uint8_t counter = 0;
    
    //First byte defines the command for the display controller.
    Command_array[0] = 0x00; 
    
    for (counter = 0; counter < length; counter++){
        Command_array[counter + 1] = command_to_send[counter];
    }

    Display_address_RW = SSD1306_ADDRESS <<1;

    SSD1306_I2C_write_array_to_external_device (Display_address_RW, Command_array, length + 1);
}

//*********************************************
//        Display_write_data
//*********************************************
void Display_write_data (uint8_t data_to_send){
    
    uint8_t Display_address_RW = 0;        
    uint8_t Data_array [8];

    Display_address_RW = SSD1306_ADDRESS <<1;
       
    //Control Byte
    Data_array [0] = 0x40;
    Data_array [1] = data_to_send;
    
    //If the double buffer is ON, the controller is not updated. 
    if (Display_double_buffer == DISPLAY_DOUBLE_BUFFER_OFF){
        SSD1306_I2C_write_array_to_external_device (Display_address_RW, Data_array, 2);
    }

    //Update internal memory to keep track of the controller state
    Display_memory [Display_column_pointer] [Display_page_pointer] = data_to_send;
    
    //The column address is auto-incremented within the controller.
    //The pointer must be incremented manually at each data write
    Display_column_pointer++;
    
}


//*********************************************
//      Display_set_double_buffer_state
//*********************************************
void Display_set_double_buffer_state (uint8_t set_state){
    Display_double_buffer = set_state;
}


//*********************************************
//         Display_update_double_buffer
//*********************************************
void Display_update_double_buffer (void){
	uint8_t page = 0;
	uint8_t column = 0;
    
    uint8_t Display_column_pointer_backup = Display_column_pointer;
    uint8_t Display_page_pointer_backup = Display_column_pointer;
    
    Display_set_double_buffer_state (DISPLAY_DOUBLE_BUFFER_OFF);
	
	for (page = 0; page < DISPLAY_NUMBER_OF_PAGES; page++) {

            Display_set_column (0);
            Display_set_page (page);

            for (column = 0; column < DISPLAY_X_RESOLUTION; column++) {

                    Display_write_data (Display_memory [column] [page]);
            }
    }
    
    Display_column_pointer = Display_column_pointer_backup;
    Display_column_pointer = Display_page_pointer_backup;
    
    Display_set_double_buffer_state (DISPLAY_DOUBLE_BUFFER_ON);
}


//*********************************************
//             Display_set_display
//*********************************************
void Display_set_display (uint8_t display_status){
	
    uint8_t Command_List [8];
    
    if (display_status == DISPLAY_TURN_ON){

        Command_List[0] = 0xAF;
        
        Display_write_command (Command_List, 1);    
    }
    
    if (display_status == DISPLAY_TURN_OFF){
        
        Command_List[0] = 0xAE;
        Display_write_command (Command_List, 1);  
    }

}


//*********************************************
//             Display_set_orientation
//*********************************************
void  Display_set_orientation (uint8_t orientation){
    
    uint8_t Command_List[8];
    
    Command_List[0] = 0b00100111;
    Command_List[1] = 0b10000001;
    
    if (orientation == DISPLAY_ORIENTATION_NORTH){
        
        Command_List[0] = 0b10100000;
        Command_List[1] = 0b11001000;
    } 
    
    if (orientation == DISPLAY_ORIENTATION_SOUTH){
        Command_List[0] = 0b10100001;
        Command_List[1] = 0b11000000;
    }
    
    Display_write_command (Command_List, 2);
       
}


//*********************************************
//             Display_set_page
//*********************************************
void Display_set_page (uint8_t controller_page) {
    uint8_t command_to_be_sent = 0x00;
    uint8_t Command_List[8];
    
    //OR with the command 
    command_to_be_sent |= DISPLAY_COMM_SET_PAGE;
    
    //OR with the selected page 
    command_to_be_sent |= controller_page;
    
    Command_List[0] = command_to_be_sent;
        
    Display_write_command (Command_List, 1);

    Display_page_pointer = controller_page;
   
}


//*********************************************
//             Display_set_column
//*********************************************
void Display_set_column (uint8_t controller_column){
    uint8_t column_4_High_bits = 0x00;
    uint8_t column_4_Low_bits = 0x00;
    uint8_t Command_List[8];
    
    uint8_t command_to_be_sent = 0x00;
    
    column_4_High_bits = ((controller_column & 0xF0) >> 4);
    column_4_Low_bits = (controller_column & 0x0F);
    
    command_to_be_sent = DISPLAY_COMM_SET_COLUMN_H | column_4_High_bits;
    Command_List[0] = command_to_be_sent;
    
    
    command_to_be_sent = DISPLAY_COMM_SET_COLUMN_L | column_4_Low_bits;
    Command_List[1] = command_to_be_sent;
            
    Display_write_command (Command_List, 2);
    
    Display_column_pointer = controller_column;
    
}


//*********************************************
//             Display_set_start_line
//*********************************************
void Display_set_start_line (uint8_t controller_start_line){

    uint8_t command_to_be_sent = 0x00;
    uint8_t Command_List[8];
    
    //OR with the command 
    command_to_be_sent |= DISPLAY_COMM_DISPLAY_START_LINE;
    
    //OR with the selected page 
    command_to_be_sent |= controller_start_line;
    
    Command_List[0] = command_to_be_sent;
        
    Display_write_command (Command_List, 1);
}


//*********************************************
//             Display_clear
//*********************************************
void Display_clear (uint8_t filling_color) {
	
	uint8_t page = 0;
	uint8_t column = 0;

	Display_set_column (column);

	for (page = 0; page < DISPLAY_NUMBER_OF_PAGES; page++) {
		
        Display_set_page (page);        
        Display_set_column (0);
        
		for (column = 0; column < DISPLAY_X_RESOLUTION; column++) {

            Display_write_data (filling_color);
            
            if (filling_color == DISPLAY_FILLING_BLACK) {
                Display_memory[Display_column_pointer][Display_page_pointer] = 0xFF;
            } else {
                Display_memory[Display_column_pointer][Display_page_pointer] = 0x00;
            }
		}
	}
}


//*********************************************
//             Display_plot_xy
//*********************************************
void Display_plot_xy (uint8_t x_pos,  uint8_t y_pos, uint8_t filling_color){

	uint8_t data_byte;
	uint8_t data_mask;
    
    uint8_t column = x_pos;
    uint8_t page = 0;   

    //Divide by 8 to get the page
    page = y_pos >> 3;
    
    data_byte = Display_memory [column][page];
    
    data_mask = 0x01 << (y_pos & 0x07);	

	if (filling_color == DISPLAY_FILLING_BLACK)
		data_byte = data_byte | data_mask;
	else
		data_byte = data_byte & ~data_mask;
	
    Display_set_column(column);
    Display_set_page(page);
	Display_write_data (data_byte);

}


//*********************************************
//             Display_set_font
//*********************************************
void Display_set_font (uint8_t width, uint8_t height){

	Display_font_width = width;
	Display_font_height = height;
    
}


//*********************************************
//             Display_set_font_color
//*********************************************
void Display_set_font_color (uint8_t font_color){
    Display_font_color = font_color;
}


//*********************************************
//             Display_backlight
//*********************************************
void Display_backlight(uint8_t backlight) {

    //Not required for OLED display.
    //The empty implementation is for other Display software compatibility. 
    
}


//*********************************************
//             Display_Reset
//*********************************************
void  Display_Reset (void){

    uint8_t Command_List[8];
    
    Command_List[0] = 0b00100111;
        
    Display_write_command (Command_List, 1);
    
    delay_ms (1);
}


//*********************************************
//             Display_set_contrast
//*********************************************
void Display_set_contrast (uint8_t contrast_value){
    
    uint8_t contrast_to_set = 0;
    uint8_t Command_List[8];
            
    //Converter the % value to the 0...255 value)
    contrast_to_set = (uint8_t) ((255 * contrast_value)/100);

    Command_List[0] = 0x81;
    Command_List[1] = contrast_to_set;
        
    Display_write_command (Command_List, 2);   
}


//*********************************************
//         Display_set_invert
//*********************************************
void Display_set_invert (uint8_t invert_status){
    
    uint8_t Command_List[8];

    if (invert_status == DISPLAY_INVERT_ON){
       Command_List[0] = 0xA7;
    }
    
    if (invert_status == DISPLAY_INVERT_OFF){
       Command_List[0] = 0xA6; 
    }
  
    Display_write_command (Command_List, 1); 
    
}


//*********************************************
//         Display_initialize
//*********************************************
void  Display_set_offset (uint8_t display_offset){
    uint8_t Command_List[8];

    Command_List[0] = 0xD3;
    Command_List[1] = display_offset;

    Display_write_command (Command_List, 2);
 
}


//*********************************************
//         Display_initialize
//*********************************************
void  Display_initialize (void) {
    uint8_t Command_List[8];
    
    //wait for the power to stabilize    
    delay_ms (50);

    //Display_Reset ();
    
    SSD1306_I2C_open (I2C_MASTER);
    SSD1306_I2C_baud_rate (SSD1306_BIT_RATE);

    Display_set_start_line (0);
    Display_set_column (0);
    Display_set_page (0);
    
    //Set the commands according to the resolutions 128x64 or 128x32
#if DISPLAY_Y_RESOLUTION==32
    Command_List[0] = 0xA8;
    Command_List[1] = DISPLAY_Y_RESOLUTION-1;
    
    Command_List[2] = 0xDA;
    Command_List[3] = 0x02;
#endif    
    
#if DISPLAY_Y_RESOLUTION==64
    Command_List[0] = 0xA8;
    Command_List[1] = DISPLAY_Y_RESOLUTION-1;
    
    Command_List[2] = 0xDA;
    Command_List[3] = 0x12;
#endif    
    
    Display_write_command (Command_List, 4);

    Display_write_command (SSD1306_init_array, 13);
    
    //Display_set_orientation (DISPLAY_ORIENTATION_SOUTH);
    
    Display_set_offset (0);
    
    Display_set_invert (DISPLAY_INVERT_OFF);  
    
    Display_set_contrast (60);    
    
    Display_set_display (DISPLAY_TURN_ON);
    
    Display_clear (DISPLAY_FILLING_WHITE);   
	
    Display_set_font (DISPLAY_FONT_WIDTH_5, DISPLAY_FONT_HEIGTH_7);
    Display_set_font_color (DISPLAY_FILLING_BLACK);
    
    Display_column_pointer = 0;
    Display_page_pointer = 0;


}
