/*******************************************************************************

Author : Mauro Laurenti
Version : 1.0

Created on Date : 25/11/2023
Last update     : 25/11/2023

CopyRight 2006-2023 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef COMPILER_XC
    #include <xc.h>
#endif

#include "nRF24L01.h"

//************************************************************
//             NRF24L01_initialize  Implementation
//************************************************************
void NRF24L01_initialize (uint8_t TX_channel, uint8_t RF_power, uint8_t bit_rate){

    // Pin initialization
    NRF24L01_CS_TRIS = PIN_AS_OUTPUT;
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;

    NRF24L01_CE_TRIS = PIN_AS_OUTPUT;
    NRF24L01_CE_PORT = NRF24L01_CE_DISABLED;
    
    NRF24L01_IRQ_TRIS = PIN_AS_INPUT;

    NRF24L01_SPI_mode(SPI_MODE_0);
    NRF24L01_SPI_baudrate(NRF24L01_SPI_BAUDRATE);
    NRF24L01_SPI_open(SPI_MASTER_DEVICE);
    
    NRF24L01_set_power_mode (NRF24L01_TURN_ON);
    
    //Wait for the PLL to get in LOCK state and being able to work.
    delay_ms (2);

    NRF24L01_flush_buffer (NRF24L01_FLUSH_TX_BUFFER);
    NRF24L01_flush_buffer (NRF24L01_FLUSH_RX_BUFFER);
    
    NRF24L01_packet_format (NRF24L01_ADDRESS_5_BYTES, NRF24L01_CRC_ENABLED, NRF24L01_CRC_SIZE_1_BYTE);
    
    NRF24L01_set_channel (TX_channel);
        
    NRF24L01_set_RF_power (RF_power);
    
    NRF24L01_set_bit_rate (NRF24L01_BIT_RATE_1MHZ);
    
    NRF24L01_set_power_mode (NRF24L01_TURN_OFF); 
        
}


//************************************************************
//           NRF24L01_set_power_mode  Implementation
//************************************************************
void NRF24L01_set_power_mode (uint8_t status){
    
    
    if (status == NRF24L01_TURN_ON){
        NRF24L01_CE_PORT = NRF24L01_CE_ENABLED;
    }
    
    if (status == NRF24L01_TURN_OFF){
        NRF24L01_CE_PORT = NRF24L01_CE_DISABLED;    
    }
    
}


//************************************************************
//           NRF24L01_set_power_mode  Implementation
//************************************************************
void NRF24L01_send_command (uint8_t command, NRF24L01_packet packet){

    uint8_t counter = 0;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    NRF24L01_SPI_write_byte (command);
    
    for (counter = 0; counter < packet.size; counter++){
        NRF24L01_SPI_write_byte (packet.payload[counter]);  
    }
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;

}


//************************************************************
//           NRF24L01_flush_buffer  Implementation
//************************************************************
void NRF24L01_flush_buffer (uint8_t buffer){
    
    NRF24L01_packet packet;
    uint8_t command;
    
    packet.size = 0;
    
    if (buffer == NRF24L01_FLUSH_TX_BUFFER){
       command = NRF24L01_COM_FLUSH_TX; 
    }
    
    if (buffer == NRF24L01_FLUSH_RX_BUFFER){
       command = NRF24L01_COM_FLUSH_RX; 
    }
    
    NRF24L01_send_command (command, packet);
    
}


//************************************************************
//           NRF24L01_packet_format  Implementation
//************************************************************
void NRF24L01_packet_format (uint8_t ADDR_size, uint8_t CRC_enable, uint8_t CRC_size){
    NRF24L01_packet packet;
    uint8_t command;
    uint8_t data;
    
    command = NRF24L01_COM_W_REGISTER | NRF24L01_REG_SETUP_AW;
    packet.size = 1;
    //The mask make sure that only  bit 0 and 1 are used
    packet.payload[0] = ADDR_size & 0x03;
    NRF24L01_send_command (command, packet);
    
    
    //Set the CRC as required
    data = NRF24L01_get_register (NRF24L01_REG_CONFIG);
    
    if (CRC_enable == NRF24L01_CRC_ENABLED){
        data |=  NRF24L01_CRC_ENABLED;
    }
    
    if (CRC_enable == NRF24L01_CRC_DISABLED){
        data &=  NRF24L01_CRC_DISABLED;
    }
    
    if (CRC_size == NRF24L01_CRC_SIZE_1_BYTE){
        data &=  NRF24L01_CRC_SIZE_1_BYTE;
    }
    
    if (CRC_size == NRF24L01_CRC_SIZE_2_BYTE){
        data |=  NRF24L01_CRC_SIZE_2_BYTE;
    }
        
    NRF24L01_set_register (NRF24L01_REG_CONFIG,data );
}


//************************************************************
//           NRF24L01_set_channel  Implementation
//************************************************************
uint8_t NRF24L01_set_channel (uint8_t channel){

    NRF24L01_set_register (NRF24L01_REG_RF_CH, channel);   

}


//************************************************************
//           NRF24L01_set_address  Implementation
//************************************************************
void NRF24L01_set_address (uint8_t address_type, uint8_t* address, uint8_t size){
    
    NRF24L01_packet packet;
    uint8_t command = 0;
    uint8_t counter = 0;
    
    if(address_type == NRF24L01_ADDRESS_TX){
        command = NRF24L01_COM_W_REGISTER | NRF24L01_REG_TX_ADDR;
        packet.size = size;
    }
    
    if(address_type == NRF24L01_ADDRESS_RX_P0){
        command = NRF24L01_COM_W_REGISTER | NRF24L01_REG_RX_ADDR_P0;
        packet.size = size;
    }
    
    if(address_type == NRF24L01_ADDRESS_RX_P1){
        command = NRF24L01_COM_W_REGISTER | NRF24L01_REG_RX_ADDR_P1;
        packet.size = size;
    }
    
    if(address_type == NRF24L01_ADDRESS_RX_P2){
        command = NRF24L01_COM_W_REGISTER | NRF24L01_REG_RX_ADDR_P2;
        packet.size = 1;
    }
    
    if(address_type == NRF24L01_ADDRESS_RX_P3){
        command = NRF24L01_COM_W_REGISTER | NRF24L01_REG_RX_ADDR_P3;
        packet.size = 1;
    }
    
    if(address_type == NRF24L01_ADDRESS_RX_P4){
        command = NRF24L01_COM_W_REGISTER | NRF24L01_REG_RX_ADDR_P4;
        packet.size = 1;
    }
    
    if(address_type == NRF24L01_ADDRESS_RX_P5){
        command = NRF24L01_COM_W_REGISTER | NRF24L01_REG_RX_ADDR_P5;
        packet.size = 1;
    }          
    
    for (counter = 0; counter< size; counter++){
        packet.payload[counter] = address[counter];
    }
    
            
    NRF24L01_send_command (command, packet);
}


//************************************************************
//           NRF24L01_pipe_configuration  Implementation
//************************************************************
void NRF24L01_pipe_configuration (uint8_t pipe, uint8_t pipe_enable){

    uint8_t data = 0;
    
    data = NRF24L01_get_register (NRF24L01_REG_EN_RXADDR);
    
    if (pipe_enable == NRF24L01_PIPE_ENABLED){

        data |= pipe;
        NRF24L01_set_register (NRF24L01_REG_EN_RXADDR, data);     

    } else {
        
       NRF24L01_set_register (NRF24L01_REG_EN_RXADDR, data & (~pipe));  
    }
    
}


//************************************************************
//           NRF24L01_get_register  Implementation
//************************************************************
uint8_t NRF24L01_get_register (uint8_t register_address){
    
    uint8_t command = 0; 
    uint8_t data = 0;
    uint8_t status = 0;
    
    command = NRF24L01_COM_R_REGISTER | register_address;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    data = NRF24L01_SPI_write_read_byte (command);
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;
    
    return(data);
    
}


//************************************************************
//           NRF24L01_set_register  Implementation
//************************************************************
uint8_t NRF24L01_set_register (uint8_t register_address, uint8_t data){
    
    uint8_t command = 0; 
    uint8_t status = 0;
    
    command = NRF24L01_COM_W_REGISTER | register_address;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    NRF24L01_SPI_write_byte (data);
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;

    return(status);
}


//************************************************************
//           NRF24L01_set_RF_power  Implementation
//************************************************************
void NRF24L01_set_RF_power (uint8_t RF_power){
    uint8_t command = 0; 
    uint8_t data = 0;
    uint8_t status = 0;
    
    command = NRF24L01_COM_R_REGISTER | NRF24L01_REG_RF_SETUP;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    data = NRF24L01_SPI_write_read_byte (command);
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;
    
    data &= NRF24L01_RF_POWER_MASK;
    
    data |= RF_power;
    
    NRF24L01_set_register (NRF24L01_REG_RF_SETUP, data);

}


//************************************************************
//           NRF24L01_set_bit_rate  Implementation
//************************************************************
void NRF24L01_set_bit_rate (uint8_t bit_rate){
    uint8_t command = 0; 
    uint8_t data = 0;
    uint8_t status = 0;
    
    command = NRF24L01_COM_R_REGISTER | NRF24L01_REG_RF_SETUP;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    data = NRF24L01_SPI_write_read_byte (command);
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;
    
    data &= NRF24L01_BIT_RATE_MASK;
    
    data |= bit_rate;
    
    NRF24L01_set_register (NRF24L01_REG_RF_SETUP, data);
}


//************************************************************
//           NRF24L01_set_bit_rate  Implementation
//************************************************************
uint8_t NRF24L01_is_data_received (void){
    uint8_t command = 0; 
    uint8_t status = 0;
    uint8_t data_ready = 0;
    uint8_t pipe_ready = 0;
    
    command = NRF24L01_COM_R_REGISTER | NRF24L01_REG_STATUS;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
        
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;
    
    data_ready = status & NRF24L01_STATUS_RX_DR_MASK;
    pipe_ready = status & NRF24L01_STATUS_RX_P_NO_MASK;
    
    
    //Reset Data ready bit
    if (data_ready != 0x00){
        status =  status & (~NRF24L01_STATUS_RX_DR_MASK);               
        NRF24L01_set_register (NRF24L01_REG_STATUS, status);
        return ((pipe_ready >> 1));
    }
    
    return (NRF24L01_PIPES_EMPTY);
    
}

//************************************************************
//           NRF24L01_update_register_bit  Implementation
//************************************************************
uint8_t NRF24L01_update_register_bit (int8_t register_address, uint8_t bit_to_change, uint8_t value){
    uint8_t data = 0;
    uint8_t status = 0;
     
    data = NRF24L01_get_register (register_address);
    
    //Set to 0 the bit of interest, using bit masking
    data &=  bit_to_change;
    
    //Set to 1 the bit, if needed
    if (value == 1) {
        data |= (~bit_to_change); 
    }
    
    //Send back the updated value
    status = NRF24L01_set_register (register_address, data);
    
    return(status);
}


//************************************************************
//           NRF24L01_send_data  Implementation
//************************************************************
void NRF24L01_send_data (NRF24L01_packet packet){
    
    NRF24L01_set_power_mode (NRF24L01_TURN_OFF);
    
    NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CONFIG_BIT_PWR_UP, 1);
    NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CONFIG_BIT_PRIM_RX, 0);
    
    NRF24L01_send_command (NRF24L01_COM_W_TX_PAYLOAD, packet);
 
    NRF24L01_set_power_mode (NRF24L01_TURN_ON);
    delay_ms (1);
    NRF24L01_set_power_mode (NRF24L01_TURN_OFF);
    
    NRF24L01_flush_buffer (NRF24L01_FLUSH_TX_BUFFER);
}


//************************************************************
//           NRF24L01_is_TX_busy  Implementation
//************************************************************
uint8_t NRF24L01_is_TX_busy (void){
    
    uint8_t data = 0;
    
    data = NRF24L01_get_register (NRF24L01_REG_STATUS);
    
    //Extract TX_DS bit
    if (data & (~NRF24L01_STATUS_BIT_TX_DS) == 0){
        return (NRF24L01_TX_OVER);
    } else {
        return (NRF24L01_TX_BUSY);
    }
}


//************************************************************
//           NRF24L01_get_address  Implementation
//************************************************************
uint8_t* NRF24L01_get_address (uint8_t register_address, uint8_t size){
    static uint8_t address[NRF24L01_ADDRESS_MAX_SIZE];
    uint8_t command = 0;
    uint8_t status = 0;
    uint8_t counter = 0;
    
    command = NRF24L01_COM_R_REGISTER | register_address;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    for (counter=0; counter < size; counter++){
        address[counter] = NRF24L01_SPI_write_read_byte (command);
    }
        
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;
    
    return(address);
}

//************************************************************
//           NRF24L01_generate_carrier  Implementation
//************************************************************
void NRF24L01_generate_carrier (uint8_t status, uint8_t carrier_CH ){
    
    NRF24L01_packet packet;
    uint8_t counter =0;
    
    if (status == NRF24L01_CARRIER_ON){
       //Enable carrier
        NRF24L01_CE_PORT = NRF24L01_CE_DISABLED;  
        NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CHANGE_BIT_1, 1);
        delay_ms (2);
        NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CHANGE_BIT_0, 0);
        NRF24L01_set_register (NRF24L01_REG_EN_AA, 0x3F);
        NRF24L01_set_register (NRF24L01_REG_SETUP_RETR, 0xFF);
        NRF24L01_set_RF_power (NRF24L01_RF_POWER_0dBm);
        NRF24L01_update_register_bit (NRF24L01_REG_RF_SETUP, NRF24L01_CHANGE_BIT_4, 1);

        packet.size = 32;
        for (counter = 0; counter < 32; counter++){
            packet.payload[counter] = 0xFF;
        }

        NRF24L01_send_command (NRF24L01_COM_W_TX_PAYLOAD, packet);

        NRF24L01_packet_format (NRF24L01_ADDRESS_5_BYTES, NRF24L01_CRC_DISABLED, NRF24L01_CRC_SIZE_2_BYTE);
        NRF24L01_set_channel (carrier_CH);
        NRF24L01_CE_PORT = NRF24L01_CE_ENABLED;  
        delay_ms(1);
        NRF24L01_CE_PORT = NRF24L01_CE_DISABLED; 
        delay_ms(1);
        NRF24L01_CE_PORT = NRF24L01_CE_ENABLED;  

        packet.size = 0;
        NRF24L01_send_command (NRF24L01_COM_REUSE_TX_PL, packet); 
    }
    
    if (status == NRF24L01_CARRIER_OFF){
       NRF24L01_CE_PORT = NRF24L01_CE_DISABLED;  
    }
    
}


//************************************************************
//           NRF24L01_read_data  Implementation
//************************************************************
uint8_t NRF24L01_read_data (NRF24L01_packet *packet){
    
    uint8_t status = 0;
    uint8_t counter = 0;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (NRF24L01_COM_R_RX_PAYLOAD);
    
    for (counter=0;  counter < packet->size; counter++){
        packet->payload[counter] = NRF24L01_SPI_read_byte ();
    }
        
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;
    
    return(status);
}


//************************************************************
//           NRF24L01_set_pipe  Implementation
//************************************************************
uint8_t NRF24L01_set_pipe (uint8_t pipe, uint8_t payload_byte){
    
    uint8_t command = 0; 
    uint8_t status = 0;
    
    command = NRF24L01_COM_W_REGISTER | pipe;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    NRF24L01_SPI_write_byte (payload_byte);
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;

    return(status);
    
}


//************************************************************
//           NRF24L01_get_status  Implementation
//************************************************************
uint8_t NRF24L01_get_status (void){
    uint8_t status = 0;
   
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (NRF24L01_COM_NOP);
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;

    return(status);
}


//************************************************************
//         NRF24L01_get_lost_packets  Implementation
//************************************************************
uint8_t NRF24L01_get_lost_packets (void){
    uint8_t command = 0; 
    uint8_t data = 0;
    uint8_t status = 0;
    
    command = NRF24L01_COM_R_REGISTER | NRF24L01_REG_OBSERVE_TX;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    data = NRF24L01_SPI_read_byte ();
    
    data = (data >> 4) & 0x0F;
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;

    return(data);
}


//************************************************************
//    NRF24L01_get_retransmitted_packets  Implementation
//************************************************************
uint8_t NRF24L01_get_retransmitted_packets (void){
    uint8_t command = 0; 
    uint8_t data = 0;
    uint8_t status = 0;
    
    command = NRF24L01_COM_R_REGISTER | NRF24L01_REG_OBSERVE_TX;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    data = NRF24L01_SPI_read_byte ();
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;
    
    data = data & 0x0F;

    return(data);
}


//************************************************************
//    NRF24L01_set_RX_mode Implementation
//************************************************************
uint8_t NRF24L01_set_RX_mode (uint8_t RX_mode){
    
    uint8_t status = 0;
    
    if (RX_mode == NRF24L01_RX_MODE_OFF){
        NRF24L01_set_power_mode (NRF24L01_TURN_OFF);   
        status = NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CONFIG_BIT_PWR_UP, 0);
        status = NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CONFIG_BIT_PRIM_RX, 0);
    }
    
    if (RX_mode == NRF24L01_RX_MODE_ON){
        NRF24L01_set_power_mode (NRF24L01_TURN_ON);   
        status = NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CONFIG_BIT_PWR_UP, 1);
        status = NRF24L01_update_register_bit (NRF24L01_REG_CONFIG, NRF24L01_CONFIG_BIT_PRIM_RX, 1);
    }
    
    return (status);
}


//************************************************************
//    NRF24L01_is_carrier_detect Implementation
//************************************************************
uint8_t NRF24L01_is_carrier_detected (void){
    uint8_t command = 0; 
    uint8_t data = 0;
    uint8_t status = 0;
    
    command = NRF24L01_COM_R_REGISTER | NRF24L01_REG_CD;
    
    NRF24L01_CS_PORT = NRF24L01_CS_ENABLED;
    
    status = NRF24L01_SPI_write_read_byte (command);
    
    data = NRF24L01_SPI_read_byte ();
    
    NRF24L01_CS_PORT = NRF24L01_CS_DISABLED;
    
    data = data & 0x01;
        
    if (data == 0){
       return (NRF24L01_CARRIER_NOT_DETECTED);
    }else{
       return (NRF24L01_CARRIER_DETECTED); 
    }
}

